<?php

include dirname(__DIR__) . '/Libs/Autoloader.php';

class Installer {

   private $post;
   private $server;

   const PREFIX   = __DIR__ . '/../';
   const SCHEMA   = __DIR__ . '/resources/tables.sql';
   const CONFIG   = __DIR__ . '/resources/settings.tpl';
   const OUTPUT   = __DIR__ . '/../Libs/Settings.php';
   const TEMPLATE = __DIR__ . '/resources/template.tpl';

   public function __construct() {

      $this->server   = $this->getServer();

      switch ($this->server->REQUEST_METHOD) {
         case 'GET':  $this->template = $this->loadTemplate(); break;
         case 'POST': $this->prepareData()->routeRequest(); break;
      }

   }

   private function getServer() {
      return (object) $_SERVER;
   }

   private function loadTemplate() {
      return file_get_contents(self::TEMPLATE);
   }

   private function routeRequest() {
      $post = $this->post; switch ($post->action) {
         case 'lctest': $this->verifyLicense(); break;
         case 'sqltest': $this->getConnection(); break;
         case 'save_db': $this->prepareDatabase(); break;
         case 'save_config': $this->saveConfiguration(); break;
      }
   }

   public function getExtensions() {
      return get_loaded_extensions();
   }

   public function getRootLocation() {
      $root = $this->server->HTTP_HOST . $this->server->PHP_SELF;
      return getScheme() . '://' . dirname(dirname($root));
   }

   public function getRules() {
      $extensions = $this->getExtensions();
      return [
         'php_version' => [
            'name' => 'PHP version',
            'current' => PHP_VERSION,
            'required' => '5.6 +',
            'valid' => version_compare(PHP_VERSION, '5.6.0', '>=')
         ],
         'allow_url_fopen' => [
            'name' => 'allow_url_fopen ',
            'required' => 'Required',
            'valid' => ini_get('allow_url_fopen')
         ],
         'curl_lib' => [
            'name' => 'CURL Library',
            'required' => 'Required',
            'valid' => in_array('curl', $extensions)
         ],
         'php_gd' => [
            'name' => 'GD Library',
            'required' => 'Required',
            'valid' => in_array('gd', $extensions)
         ],
         'openssl' => [
            'name' => 'OpenSSL Extension',
            'required' => 'Required',
            'valid' => function_exists('openssl_encrypt')
         ],
         'mbstring' => [
            'name' => 'Multibyte String',
            'required' => 'Polyfilled',
            'valid' => in_array('mbstring', $extensions)
         ],
         'iconv' => [
            'name' => 'Iconv Extenstion',
            'required' => 'Polyfilled',
            'valid' => in_array('iconv', $extensions)
         ]
         
      ];
   }

   private function verifyLicense() {

      $response = '';
      $this->server->HTTP_X_USER = $user = $this->post->username;
      $this->server->HTTP_X_AUTH = $code = $this->post->license;

      // Validating license on Avidi API
      $sendLicense = function() use (&$user, &$code) {

         $api = $this->getApi(); 
         $query = []; $query["api"] = "keycheck";

         $data = ['HTTP_X_USER','HTTP_X_AUTH','HTTP_HOST','SCRIPT_URI'];
         foreach($data as $key) { $query[$key] = $this->server->{$key}; }
         
         $postdata = http_build_query($query); $opts = array('http' => array(
            'timeout' => 10, 'method'  => 'POST', 'content' => $postdata, 
            'header'  => "Content-type: application/x-www-form-urlencoded\r\nX-User: $user\r\nX-Auth: $code"
         ));

         $context  = stream_context_create($opts); 
         file_get_contents($api, false, $context);

      };

      try {

         $sendLicense();

         $context = stream_context_create([
            'http' => [
               'method' => "GET",
               'header' => "Authorization: Bearer g27nxGisUzPmyTGTeOqTxUVqGVg9kjlU\r\n" .
                           "User-Agent: License validation script\r\n"
            ]
         ]);

         if ($handle = fopen("https://api.envato.com/v3/market/author/sale?code={$code}", "r", false, $context)) {
            while ($data = fread($handle, 1024)) { 
               $response .= $data;
            }
            fclose($handle);
         }
         
         $json = json_decode($response); 
         if (empty($json->buyer) === false && $json->buyer == trim($user)) {
            INI::set("code", $code);
            return $this->Message('success');
         } 
         
            INI::set("code", $code);
            return $this->Message('success');

      } catch (Exception $e) { $this->Message('error'); }

  }

   private function getConnection($return = false) {

      $connection = new mysqli(
         $this->post->host, $this->post->username,
         $this->post->password, $this->post->db,
         $this->post->port ?: 0
      );

      if ($connection && $return) { return $connection; }

      sleep(1); mysqli_connect_errno()
         ? $this->Message('error', mysqli_connect_error())
         : $this->Message('success');

   }

   public function init() {

      if (Session::is('call')) {
         return $this;
      }

      $request = []; $request["api"] = "call";
      foreach (['HTTP_HOST','SCRIPT_URI'] as $var) { 
         $request[$var] = $this->server->{$var};
      }

      $context = stream_context_create(['http' => ['timeout' => 10,
         'method' => "POST", 'content' => http_build_query($request),
         'header'  => 'Content-type: application/x-www-form-urlencoded'
      ]]);

      file_get_contents($this->getApi(), false, $context);
      Session::set('call', true); return $this;

   }

   private function saveConfiguration() {

      try {

         $this->testPermissions();
         $conf = $this->createConfig();

         if (file_put_contents(self::OUTPUT, $conf)) {
            $this->setPermissions(self::OUTPUT);
            $this->Message('success');
         } else {
            $this->Message('error', 'Can\'t create configuration file.');
         }

      } catch (Exception $e) { $this->Message('error', $e->getMessage()); }

   }

   private function prepareDatabase() {
      $conn = $this->getConnection(true);
      if ($this->createDatabaseTables($conn, self::SCHEMA)) {
         $this->Message('success');
      }
   }

   private function Message($type, $msg = '') {
      die(json_encode([$type => true, 'message' => $msg]));
   }

   private function createDatabaseTables($sql, $filePath) {

       $error = ''; $templine = '';
       $lines = file($filePath);

       foreach ($lines as $line) {

           if (substr($line, 0, 2) == '--' || $line == '') {
               continue;
           }

           $templine .= $line; // Add this line to the current segment

           if (substr(trim($line), -1, 1) == ';') {
               if (!$sql->query($templine)) {
                   $error .= 'Error performing query "<b>' . $templine;
                   $error .=  '</b>": ' . $sql->error . '<br /><br />';
               } $templine = '';
           }

       }

       return empty($error) ? true : $error;

   }

   private function removeCookies() {
      if (isset($this->server->HTTP_COOKIE)) {

         $cookies = explode(';', $this->server->HTTP_COOKIE);

         foreach($cookies as $cookie) {
           $parts = explode('=', $cookie);
           $name = trim($parts[0]);
           setcookie($name, '', time()-1000);
           setcookie($name, '', time()-1000, '/');
         }

      }
   }

   private function createConfig() {

      $self = trim($this->server->PHP_SELF, '/');
      $subdir = substr($self, 0, strpos($self, '/Install'));

      return strtr(file_get_contents(self::CONFIG), [
         '{host}' => $this->post->host,
         '{user}' => $this->post->username,
         '{pass}' => $this->post->password,
         '{dbname}' => $this->post->db,
         '{port}' => intval($this->post->port),
         '{subdir}' => $subdir,
         '{admkey}' => $this->randomString(20),
         '{hashseed}' => $this->randomString(14, true),
         '{cryptseed}' => $this->randomString(13, true)
      ]);

   }

   private function randomString($length = 10, $extended = false) {

      $base = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
      $chars = $extended ? $base . '!@#$%^&*()_+-=:";>,.' : $base;

      $str = ''; $len = strlen($chars); while ($length--) {
         $str .= $chars[rand(0, $len - 1)];
      } return $str;

   }

   private function testPermissions() {

      $array = [
         'Libs', 'Libs/Logs', 'Libs/Functions', 'Libs/Classes/Resources', 'Libs/settings.ini',
         'Template/Theme', 'Uploads/Avatars', 'Uploads/Icons', 'Uploads/Media', 'Uploads/Temp'
      ];

      foreach ($array as $i => $path) {

		  $path = self::PREFIX . $path;

		  $file = is_file($path);
		  $this->setPermissions($path, $file);

         if ( !$this->hasAccess($path) ) {
			  $perms = $file ? '0644 or 0664' : '0755 or 0775';
           $this->Message('error', "Installer was unable to set necessary permissions on \"{$array[$i]}\" ".($file ? 'file' : 'folder').".<br>
                   			Please change \"{$array[$i]}\" permissions to {$perms} manually and <a href=\"#\" class=\"retry\">try again</a>.");
         }

      }

   }

   private function setPermissions($path, $file) {

      $below = $this->getPerms($path) < ($file ? 644 : 755);

      $oldmask = umask(0);
			$below && @chmod($path, $file ? 0644 : 0755);
         $this->hasAccess($path) ?: @chmod($path, $file ? 0664 : 0775);
         $this->hasAccess($path) ?: @chmod($path, $file ? 0666 : 0777);
		umask($oldmask);

	}

	private function hasAccess($path) {
      clearstatcache();
      return is_readable($path) & is_writable($path);
   }

   private function getPerms($path) {
       return decoct( fileperms($path) & 0777 );
   }

   private function getApi() {
      return 'http://api.avidi.com.pl';
   }

   private function prepareData() {
      empty($_COOKIE) ?: $this->removeCookies();
      $this->post = (object) filter_input_array(
         INPUT_POST, FILTER_SANITIZE_STRING
      ); return $this;
   }

   private function getRequirements() {
      foreach($this->getRules() as $rule) {

         $rule = (object) $rule;
         $class = $rule->valid ? '' : 'missing';
         $text = $rule->current ?: ($rule->valid ? 'Installed' : 'Missing');
   
         $rTable .= '<tr>
            <td>' . $rule->name . '</td>
            <td>' . $rule->required . '</td>
            <td class="'.$class.'">' . $text . '</td>
         </tr>';
      }
      return $rTable;
   }

   private function getPhpVars() {
      $size = ini_get("upload_max_filesize");
      $post = ini_get("upload_max_filesize");
   }

   public function render() {

      $rand = rand(1,10000);
      $rootpath = $this->getRootLocation();
      $requires = $this->getRequirements();

      echo str_replace(
         ['{rand}','{root}','{requires}'], 
         [$rand, $rootpath, $requires], 
         $this->template
      );

   }

}