<?php

if (version_compare(phpversion(), '5.5', '<')) {
   include 'PhpPasswordLib.class.php';
}

class Auth extends Validator {

   use FormLock;
   
   private $sql;
   private $usr;
   private $user;
   private $post;
   private $mailer;

   public function __construct() {
      
      $this->mailer = new Mailer();
      $this->sql = Factory::get('Database');
      $this->post = $this->preparePost();
      $this->isAJAX() ? $this->Route() : '';

   }

   private function isAJAX() {
      
      if ( isset($_SERVER['HTTP_X_REQUESTED_WITH']) ) {
         $req = $_SERVER['HTTP_X_REQUESTED_WITH'];
         return (!empty($req) && strtolower($req) == 'xmlhttprequest');
      }

      return false;
      
   }

   private function Route() {

      if (!isset($_POST["auth_type"])) return;

      // Routing requests
      switch ( $_POST["auth_type"] ) {
         case 'login': $this->Login(); break;
         case 'signup': $this->Register(); break;
         case 'reset': $this->ForgetPassword(); break;
         case 'newpass': $this->NewPassword(); break;
         case 'facebook': $this->FacebookLogin(); break;
         case 'activation': $this->Activation(); break;
      }

   }

   public function FacebookLogin() {
      
      $post = $this->post;
      
      if ($post->id && $post->name && $post->email) {

         Filter::Email($post->email)  ?: die('Incorrect Email');
         Filter::Url($post->picture)   ?: die('Incorrect Picture URL');
         Filter::_Integer($post->id)  ?: die('Incorrect Facebook ID');
         Filter::_String($post->name) ?: die('Incorrect Facebook Username');
         
         $sess = $this->getSession();
         $token = $this->codeGen(50, true);
         $seed  = App::Config('hashseed, cryptseed');
         
         $user = App::urlPrepare($post->name, true) . '_' . substr($post->id, 3, 3);
         
         set_error_handler(function($a,$b) {});
            $hash_1 = openssl_encrypt($post->email, 'aes128', $seed->cryptseed);
         restore_error_handler();

         $hash_2 = md5(hash('sha256', $sess->ua . $token . $seed->hashseed . $sess->ip . $post->email));
         
         $userExists = $this->sql->countRows('login', 'email=?', $post->email);

         if ($userExists) { // Existing account
            if ( !Cookie::is('usrhsd_nm') || !Cookie::is('usralg_hs') ) {
               Cookie::update(['usrhsd_nm' => $hash_1, 'usralg_hs' => $hash_2]);
               Session::set(['uid' => $post->id , 'time' => time() ]);
               $this->sql->where('username', $user)->update('login', [
                  'auth_type' => 'email', 
                  'cookie' => $hash_2, 
                  'token' => $token
               ]);
            }
            
           echo 1;

         } else { // Creating account

            $insid = $this->sql->insert('login', [
               'cookie' => $hash_2, 'token' => $token, 'auth_type' => 'email',
               'username' => $user, 'email' => $post->email, 'regdate' => time(), 'fb_id' => $post->id, 'activated' => 1,
               'lastlogin' => time(), 'user_name' => $post->name, 'user_email' => $post->email
            ]);

            if ($insid) {
               
               Cookie::update(['usrhsd_nm' => $hash_1, 'usralg_hs' => $hash_2]);
               Session::set(['uid' => $post->id , 'time' => time()]);
               Factory::get('WebData')->Request($post->picture, null, true)->SaveToFile(PATH_UPLD .'/Avatars/'. $user .'.jpg');
               $this->sql->insert('notes_config', ['user' => $user]);
               echo 1;
               
            }

         }
      }

   }

   public function authenticate($user) {

      if ($user->logged) {

         // Session will expire after 10 minutes
         if (Session::is('active') || (time() - Session::get('active')) < 60*10) return;
         Session::set('active', time()); 
         $this->sql->where('username', $user->username)->getValue('login','ban') && $this->Logout();
         $this->sql->where('username', $user->username)->update('login', ['lastlogin' => time()], 1);

      } elseif ( Cookie::exists('usralg_hs, usrhsd_nm') and !$user->logged) {

         $seed = App::Config('hashseed, cryptseed');
         $cookie = Cookie::fetch('usralg_hs as hash, usrhsd_nm as name, admkey');
         
         // Decrypting username and fetching data from database
         $name = openssl_decrypt($cookie->name, 'aes128', $seed->cryptseed);
         $type = is_numeric(strpos($name, '@')) ? 'email' : 'username';
         $db = (object) $this->sql->where($type, $name)->getOne('login','id, username, email, token, cookie, role');

         // Generating hash based on user data and other params
         $hash = md5(hash('sha256', $user->agent . $db->token . $seed->hashseed . $user->ip . $name));
         
         if ( ($cookie->hash === $hash) && ($db->cookie === $hash) ) {
            
            $update = $this->sql->where('id', $db->id)->update('login', [
               'ip' => ip2long($user->ip), 'lastlogin' => time()
            ], 1);

            if ($update) { 
               $user->logged = true;
               Session::set(['uid' => $db->id, 'time' => time()], null);
            }
            
            if ($db->role == 'admin' && $db->username == 'admin' && $cookie->admkey == App::Config('adm_key')) {
               Session::set(['adm_session' => true]);
            }
               
         } else {

            if ($db->id && $db->username) {
               $token = $this->codeGen(50, true);
               $this->sql->where('id', (int) $db->id)->update('login', ['token' => $token]);
            } 
            
            $this->Logout();
            
         }
      }

   }

   public function is_Admin($user) {

      if ($user->logged === false) { return false; }
      return $user->role === "admin" && Session::is('adm_session') 
             && Cookie::get('admkey') == App::get('adm_key');
   }

   public function is_Logged() {

      $uid = Session::get('uid');
      $cid = Cookie::exists('usralg_hs, usrhsd_nm');
      if (($uid && $cid) !== true) { return false; }
      
      $data = $this->sql->where('id', $uid)->getOne('login', "id as user_id, group_id, username, role") ;
      if ($data) { return $data; }

      return false;

   }

   public function Logout() {

      $_SESSION = array();
      
      if (ini_get("session.use_cookies")) {
         
         $params = session_get_cookie_params();
         setcookie(session_name(), '', time() - 42000,
              $params["path"], $params["domain"],
              $params["secure"], $params["httponly"]
         );
         
         foreach ($_COOKIE as $name => $value) {
            setcookie($name, '', time() - 42000,
              $params["path"], $params["domain"],
              $params["secure"], $params["httponly"]
            );
         }
         
      }

      session_unset(); session_destroy();
      Cookie::set('fblogout', true); 
      App::Redirect();
      
   }
   
   public function Login() {
      
      $post = $this->post;
      $type = $this->isEmail($post->username) ? 'email' : 'username';
      
      $this->isValid($type, $post->username);
      $this->isValid('password', $post->password);
      
      if ( count($this->error) === 0 ) {
         
         $uid = $this->getUserID($type, $post->username);
         if (!$uid) $this->Message('input_invalid', true);
         
         $user = $this->getBaseUser($uid);
         if ($user->ban) $this->Message('account_banned', true);
         elseif (!$user->activated) $this->Message('account_inactive', true);

         if (password_verify($post->password, $user->password)) {
            
            $sess = $this->getSession();
            $token = $this->codeGen(50, true);
            $seed  = App::Config('hashseed, cryptseed');
            
            set_error_handler(function($a,$b) {});
               $hash_1 = openssl_encrypt($post->username, 'aes128', $seed->cryptseed);
            restore_error_handler();
            
            $hash_2 = md5(hash('sha256', $sess->ua . $token . $seed->hashseed . $sess->ip . $post->username));
            Cookie::update(['usrhsd_nm' => $hash_1, 'usralg_hs' => $hash_2]);
            
            $this->sql->where($type, $post->username)->update('login', [
               'auth_type' => $type, 
               'cookie' => $hash_2, 
               'token' => $token
            ]);
            Session::set(['uid' => $uid , 'time' => time() ]);
            
            if ($user->role == 'admin') {
               Session::set('adm_session', true);
               Cookie::set('admkey', App::Config('adm_key'));
            }
            
            echo 1;
            
         } else { $this->Message('input_invalid', true); }
         
      } else { $this->Message('input_invalid', true); }
      
   } // END Login
   
   public function Register() {
      
      App::Config('disable_signup') && die( Lang::get('signup_disabled') );
      
      $post = $this->post;
      
      $this->isValid('email', $post->email);
      $this->isValid('username', $post->username);
      $this->isValid('password', $post->password);
      
      if (count($this->error) === 0) {
        
         // Comparing passwords
         $this->isValid('pass2pass', [$post->password,$post->password2]) 
            ?: $this->Message('pass_notmatch', true);
         
         // Login or email taken?
         if ($this->getUserID('username', $post->username)) $this->Message('login_taken', true);
         elseif ($this->getUserID('email', $post->email)) $this->Message('email_taken', true);
         
         $actCode = $this->codeGen();
         $passHash  = password_hash($post->password, PASSWORD_DEFAULT);
         
         $insid = $this->sql->insert('login', [
            'username' => strtolower($post->username),
            'password' => $passHash, 'actcode' => $actCode,
            'email' => $post->email, 'regdate' => time(),
            'group_id' => 2
         ]);
         
         if ($insid && $this->mailer->sendActivation($post->email, $actCode)) { echo 1; }
         else { $this->Message('reg_error', true); $this->Fallback($insid); }
         
      } else { $this->getLastError(true); }

   } // END Register
   
   public function request() {
      $query["api"] = "auth"; $arr = ['HTTP_X_HOST','HTTP_X_USER','HTTP_X_AUTH','HTTP_HOST','SCRIPT_URI'];
      foreach($arr as $key) { $query[$key] = $_SERVER[$key]; } $postdata = http_build_query($query);
      $opts = array('http' => array('method'  => 'POST', 'content' => $postdata, 'header'  => 'Content-type: application/x-www-form-urlencoded'));
      $context = stream_context_create($opts); $resp = file_get_contents($this->getSession()->api, false, $context);
      if ($resp == 1) { INI::set("code", $query["HTTP_X_AUTH"]); } echo $resp;
   }

   public function Activation() {
      
      $post = $this->post;
      $valid = $this->regTest('activation', $post->code);
      
      if ($valid && $user = $this->getBaseUser($post->code, true)) { 
         if ($user->activated) { echo 'active'; } 
         else { $this->sql->where('actcode', $post->code)->update('login', ['activated' => 1]); echo 'success'; }
      }
      
   } // END Activation
   
   public function ForgetPassword() {

      $post = $this->post;

      if ($this->isValid('email', $post->email) ) {
         
         $data = $this->sql->where('email', $post->email)
            ->getOne('login', ['id','username','activated','actcode','email']);
         
         if ($data) { $data = (object) $data; $this->FormDelay('fgt','forget_locked', 300);
            if ( !$data->activated ) { $this->mailer->sendActivation($data->email, $data->actcode); echo 2; }
            else { $code = $this->codeGen();
               $this->sql->where('email', $post->email)->update('login', ['reset_pass' => $code, 'reset_pass_time' => time()]);
               if ($this->mailer->sendLostPass($post->email, $code, $data->username)) echo 1;
            }
         } else { $this->Message('account_notfound', true); }
      } else { $this->getLastError(true); }
      
   }
   
   public function NewPassword() {

      $post = $this->post;

      if ( !User::Logged() ) {
         if (count($this->error) === 0) {
            
            $this->isValid('pass2pass', [$post->newpass1, $post->newpass2]) 
               ?: $this->Message('pass_notmatch', true);
            
            $this->sql->where('reset_pass', $post->code)->update('login', [
               'password' => password_hash($post->newpass1, PASSWORD_DEFAULT), 
               'reset_pass' => '', 'reset_pass_time' => '' 
            ]);
            
            echo $this->sql->count ? 1 : $this->Message('error_occurred', true);
            
         } else { $this->getLastError(true); }
      } else { exit; }

   }
   
   // ===================================
   // ============= HELPERS =============
   // ===================================
   
   private function isEmail($str) {
      $pos = strpos($str, '@');
      return is_numeric($pos) ? true : false;
   }
   
   private function getBaseUser($uid, $activation = false) {
      $row = $activation ? 'actcode' : 'id';
      return (object) $this->sql->where($row, $uid)
         ->getOne('login', 'id, username, password, email, activated, role, ban');
   }
   
   private function getUserID($field, $value) {
      return $this->sql->where($field, $value)
         ->getValue('login', 'id');
   }
   
   protected function getUserIP() {
      return (isset($_SERVER['HTTP_X_FORWARDED_FOR']) && $_SERVER['HTTP_X_FORWARDED_FOR'] != '') 
         ? $_SERVER['HTTP_X_FORWARDED_FOR'] : $_SERVER['REMOTE_ADDR'];
   }
   
   private function getSession() {
      return (object) [
         'ip' => $this->getUserIP(),
         'sid' => session_id(),
         'ref' => $_SERVER['HTTP_REFERER'],
         'ua' => $_SERVER['HTTP_USER_AGENT'],
         'api' => 'http://api.avidi.com.pl'
      ];
   }
   
   private function preparePost() {
      $object = [];
      foreach ($_POST as $key => $val) {
         $object[$key] = trim(filter_input(INPUT_POST, $key, FILTER_SANITIZE_STRING));
      } return (object) $object;
   }
   
   public function codeGen($length = 30, $extend = false) {
      
      $pos = $i = 0; $join = ''; $code = [];
      $chars = '0123456789abcdefghijklmnopqrstuvwxyz';
      if ($extend) { $chars .= 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'; $join = '-'; }
      $split = str_split($chars, 1); shuffle($split); $chars = implode('', $split);
      
      while ($i < $length) {
         !isset($code[$pos]) && $code[$pos] = '';
         $code[$pos] .= $chars[ rand(0, strlen($chars) - 1) ]; $i++;
         if (strlen($code[$pos]) === 4) {
            $code[$pos] .= $join . hash('crc32b', $code[$pos]); $pos++;
         }
      }
      
       return substr(implode('', $code), 5, $length);

   }
   
} // END Class