<?php

class Embed {
   
   use Stats;
   
   private $tpl; // Base template
   private $sql; // Database handle
   private $get; // GET params
   private $src; // Media source
   private $item; // Post data

   private $err = array();
   private $data = array();
   
   public function __construct() {
      
      $this->sql = Factory::get('Database');
      $this->prepareGet($_GET);
      
      $this->data = $this->prepareData();
      if (count($this->err) === 0) { $this->process(); }
      else { die( $this->getLastError() ); }
      
   }
   
   private function getLastError() {
      return array_shift($this->err);
   }
   
   private function process() {
      
      $this->tpl = new Template('base_embed');
      $this->pl_tpl = $this->getPlayerTemplate();
      
      if (count($this->err) === 0) {
         
         $data = isset($this->path) 
            ? array_merge((array) $this->data, ['path' => rtrim($this->path,'/')])
            : (array) $this->data;
               
         $vars = compact('data');
         $this->updateViews($this->item["id"], $this->item["type"]);
         
         $tpl = new Template( $this->pl_tpl );
         $this->tpl->extend('player', $tpl->render( $vars ));
         $this->tpl->display();
         
      } else { die( $this->getLastError() ); }
      
   }
   
   private function getPlayerTemplate() {
      
      isset($this->data->source) && $src = $this->data->source; 
      isset($this->data->from) && $from = $this->data->from;
      $local = ['ninegag','giphy','instagram','uploaded','mp3','mp4','webm','ogg'];
      $embed = ['openload','vk','liveleak','coub','dailymotion','twitter','soundclouds',
                'streamable','vevo','vimeo','metacafe','dtube','mixcloud'];
      
      switch (true) {
         case ($src == 'cda'): $e = 'cda'; break;
         case (isset($from) && $from == 'gifurl'): $e = 'gif'; break;
         case ($src == 'youtube'): $e = 'yt'; break;
         case ($src == 'facebook'): $e = 'fb'; break;
         case ($src == 'soundcloud'): $e = 'sc'; break;
         case (in_array($src, $local)): $e = 'video'; break;
         case (in_array($src, $embed)): $e = 'other'; break;
         default: $e = null;
      }
      
      if ($e !== null && file_exists(THEME .'/embed_player_'. $e . '.tpl')) { 
         return 'embed_player_' .$e;
      } else { array_push($this->err, 'Template not found'); }
      
   }
   
   private function prepareData() {
      
      if (isset($this->get->id) || isset($this->get->media_link)) {

         $data = isset($this->get->id) 
            ? $this->getSqlData() 
            : $this->fetchData();
         
         return (gettype($data) === "object") 
            ? $data : false;
         
      } else { array_push($this->err, 'Media not found'); } // TODO

   }
   
   private function fetchData() {
      if (Filter::Url($this->get->media_link)) {
         $data = App::mediaData($this->get->media_link);
         if ( $json = json_decode($data) ) { $this->src = 'url'; return $json; }
      } return null;
   }
   
   private function getSqlData() {
      if ($this->sql->countRows('posts', 'id=?', $this->get->id)) {
         $this->item = $this->sql->where('id', $this->get->id)->getOne('posts',"id,dir,hash,type,mediadata");
         if ( $json = json_decode($this->item['mediadata']) ) {
            $this->path = DOMAIN . '/Uploads/Media/' . $this->item['dir'] .'/'. $this->item['hash'] .'/';
            $this->src = 'db';  return $json;
         }
      } return null;
   }
   
   private function prepareGet($array) {
      $this->get = new stdClass;
      array_walk($array, function($val, $key) {
         if (is_array($val)) { $this->prepareGet($val); }
         else { $this->get->{$key} = trim(strip_tags($val)); }
      });
   }
   
}