<?php

class Mailer {
   
   private $conf;
   
   public function __construct() {
      
      // Fetching page settings
      $this->conf = App::Config();
      
   }
   
   public function sendMail($to, $topic, $message) {

      // *** Preparing email headers for php mail() ***
      $headers = "From: {$this->conf->app_name} <no-reply@" . $this->conf->app_host . ">\r\n";
      $headers .= "MIME-Version: 1.0\r\n";
      $headers .= "Content-Type: text/html; charset=utf-8\r\n";

      if ($this->conf->email_func == "phpmail") {
         $status = @mail($to, $topic, $message, $headers, "-f no-reply@{$this->conf->app_host}");
         if ($status) { return true; }
      }

      return $this->PHPMailer($to, $topic, $message);

   }
   
   private function PHPMailer($to, $topic, $message) {

      require 'PHPMailer/SMTP.php';
      require 'PHPMailer/PHPMailer.php';
      
      $debug = true;
      $conf = $this->conf;

      $mail = new PHPMailer($debug);

      try {
         
         #  SMTP authentication (optional)
         #
         #  SMTP must be able to connect to your email provider,
         #  which is why it requires additional setup
         #
         #  Port - The most common SMTP ports are 587 and 465.
         #  Host - This is the address to the host’s SMTP server.
         #  Username - Typically your email address for the email provider.
         #  Password - This is your email account password.
         #  SMTPSecure - The most widely used encryptions are ssl and tls.
      
         $useSMTP = (bool) $conf->smtp; // Set this to true to enable SMTP authentication

         if ($useSMTP) {
            $mail->Host       = $conf->smtp_host;
            $mail->Port       = (int) $conf->smtp_port;
            $mail->Username   = $conf->smtp_username;
            $mail->Password   = $conf->smtp_password;
            $mail->SMTPSecure = $conf->smtp_secure;
            $mail->SMTPAuth   = (bool) $conf->smtp_auth;
            $mail->IsSMTP();
         }

         $mail->IsHTML(true);
         $mail->CharSet = 'UTF-8';
         $mail->AddAddress($to);
         $mail->Subject = $topic;
         $mail->Body = $message;
         if (!$useSMTP) $mail->isSendmail();
         $mail->AddReplyTo("no-reply@{$conf->app_host}");
         $mail->setFrom("no-reply@{$conf->app_host}", $conf->app_name);

         if ($mail->send()) { return true; }
         
      } catch (Exception $e) { if ($debug) echo $e->getMessage(); }
      
   }
   
   private function EmailTemplate($params) {
   
      $tpl = new Template('email');
      return $tpl->render( compact('params') );

   }
   
   public function sendActivation($email, $code) {
      
      $conf = $this->conf;
      $act_url = $conf->app_url . '/auth?' . $code;
      
      $params = (object) [
         'href' => $act_url,
         'topic' => $conf->app_name . ' - ' . Lang::get('account_activation_head'),
         'anchor'  => Lang::get('click_to_activate'),
         'message' => Lang::get('welcome_to', ['site' => $conf->app_name]) . '!<br><br>' .
                      Lang::get('not_fully_activated') . ' ' . Lang::get('click_below'),
         'message2' => Lang::get('if_link_invalid', ['link' => $act_url]) . ' ' . Lang::get('by_system')
      ];
      
      $message = $this->EmailTemplate($params);
      return $this->sendMail($email, $params->topic, $message);
      
   }
   
   public function sendLostPass($email, $code, $user) {
      
      $conf = $this->conf;
      $params = (object) [
         'logo' => $conf->app_url . '/' . $conf->app_logo,
         'href' => $conf->app_url . '/page/account/reset?token=' . $code,
         'topic' => $conf->app_name . ' - ' . Lang::get('pass_recovery'),
         'anchor' => Lang::get('reset_pass'),
         'message' => Lang::get('lostpass_message_top', ['user' => $user]),
         'message2' => Lang::get('lostpass_message_bottom')
      ];

      $message = $this->EmailTemplate($params);
      return $this->sendMail($email, $params->topic, $message);

   } // END sendLostPass
   
} // END Mailer