<?php

class Messages {
   
   private $sql;
   private $user;
   
   public function __construct() {
      $this->sql = Factory::get('Database');
      $this->user = User::get('username');
   }
   
   public function getMessage($id) {
      if ($msg = $this->sql->ObjectBuilder()->where('id', $id)->getOne('messages')) {
         $msg->time = App::zmienczas($msg->time);
         $msg->avatar = App::getAvatar($msg->sender);
         return $msg;
      } return false;
   }
   
   public function countMessages() {
      return (int) $this->sql->countRows('messages', 'receiver=? and rcv < 1', $this->user);
   }
   
   public function getMessages($mode = null, $page = null) {

      $this->sql->pageLimit = 10;

      // 0 - unseen
      // 1 - seen
      // 2 - track
      // 3 - deleted

      // $this->user - self

      switch ($mode) {
         case 'self':   $this->sql->where('sender', $this->user)->where('snd', 2, '<'); break;
         case 'seen':   $this->sql->where('receiver', $this->user)->where('rcv', 1); break;
         case 'unseen': $this->sql->where('receiver', $this->user)->where('rcv', 0); break;
         case 'trash':  $this->sql->where('(receiver=? and rcv=2) or (sender=? and snd=2)', [$this->user, $this->user]); break;
         default:       $this->sql->where('receiver', $this->user)->where('rcv', 2, '<'); break;
      }
      
      $resp = $this->sql->ObjectBuilder()
         ->orderBy('id','desc')->paginate('messages', $page ? ++$page : 1);

      return $resp;
      
   }
   
   public function setSeen($id) {
      $this->sql->where('id', $id)->update('messages',[
         'rcv' => 1
      ]);
   }
   
   public function trash($id, $self) {
      $this->sql->where('id', $id)
         ->where($self ? 'sender' : 'receiver', $this->user)
         ->update('messages', [$self ? 'snd' : 'rcv' => 2]);
   }
   
   public function delete($id, $self) {
      var_dump($id, $self, $this->user, $self ? 'sender' : 'receiver');
      $this->sql->where('id', $id)
         ->where($self ? 'sender' : 'receiver', $this->user)
         ->update('messages', [$self ? 'snd' : 'rcv' => 3]);
   }
   
   public function send($to, $topic, $message) {
      if (User::userExists($to)) {
         return (int) $this->sql->insert('messages', [
            'sender' => User::get('username'),
            'receiver' => $to, 'snd' => 1,
            'time' => time(), 'topic' => $topic,
            'message' => $message
         ]);
      }
   }
   
}