<?php

abstract class Panels {
  
   const POST_LIMIT = 15;
  
   protected $get;
   protected $sql;
   protected $page;
   protected $nick;
	protected $currPage;
   
   public function __construct() {
      $this->sql = Factory::get('Database');
		$this->nick = User::get('username');
		$this->prepareVars();
	}
	
	private function prepareVars() {
		$this->get = App::getUrlParams();
		$params = explode('&', $_SERVER['QUERY_STRING']); 
		$this->page = rtrim(array_shift($params), '=');
		parse_str($_SERVER['QUERY_STRING'], $this->params);
      $this->currPage = isset($this->get->page) ? $this->get->page + 1 : 1;
	}

   private function paginator($pages, $limit, $data) {

		$array = $this->params; array_shift($array); unset($array["page"]);
		if (!empty($data->search)) { $array["search"] = $data->search; }

		$current = $data->page; 
		$next = $current + 1;
		$prev = $current - 1;
		
		$separator = '&page='; $add = ''; $stages = 2;
		$target = rtrim($data->href .'&'. http_build_query($array), '&');

		$iLeft = '<i class="icon-angle-double-left"></i>';
		$iRight = '<i class="icon-angle-double-right"></i>';

		if ($pages > 1) { // Displaying pagination

			$paginate = '<div class="paging wide">' . ($current > 0
				? '<a class="prev-arrow" href="' . $target . ($prev > 0 ? $separator . $prev : '') . $add . '">'.$iLeft.'</a>'
				: '<span class="current prev-arrow">'.$iLeft.'</span>');

				$paginate .= '<div class="num-buttons">';

			if ($pages < 7 + ($stages * 2)) {
				for ($counter = 0; $counter < $pages; $counter++) {
					$paginate .= ($counter == $current)
					? '<span class="current pagenum">' . ($counter+1) . '</span>'
					: '<a href="' . $target . ($counter > 0 ? $separator . $counter : '') . $add .'">' . ($counter+1) . '</a>';
				}
			}

			elseif ($pages > 5 + ($stages * 2)) {

				$iterate = function($i) use (&$current, &$target, &$separator) {
					return ($i == $current)
						? '<span class="current pagenum">' . ($i + 1) . '</span>'
						: '<a href="' . $target . $separator . ($i > 0 ? $i : ''). $add . '">' . ($i + 1) . '</a>';
				};

				$stepDown =
					'<a onclick="return:false">...</a>' .
					'<a href="' . $target . $separator . ($pages - 2) . $add . '">' . ($pages - 1) . '</a>' .
					'<a href="' . $target . $separator . ($pages - 1) . $add . '">' . $pages . '</a>';

				$stepUp =
					'<a href="' . $target . $add . '">1</a>' .
					'<a href="' . $target . $separator . 1 . $add . '">2</a>' .
					'<a onclick="return:false">...</a>';

				if ($current < 1 + ($stages * 2)) {
					for ($counter = 0; $counter < 4 + ($stages * 2); $counter++) {
						$paginate .= $iterate($counter);
					}; $paginate .= $stepDown;
				}

				elseif ($pages - ($stages * 2) > $current && $current > ($stages * 2)) {
					$paginate .= $stepUp;
					for ($counter = $current - $stages; $counter <= $current + $stages; $counter++) {
						$paginate .= $iterate($counter);
					}; $paginate .= $stepDown;
				}

				else {

					$paginate .= $stepUp;
					for ($counter = $pages - (2 + ($stages * 2)); $counter < $pages; $counter++) {
						$paginate .= $iterate($counter);
					}
				}

			}

			$paginate .= '</div>';
			$paginate .= (($current < $counter - 1)
				? '<a class="next-arrow" href="' . $target . $separator . $next . $add . '">' . $iRight . '</a>'
				: '<span class="current next-arrow">' . $iRight . '</span>') . '</div>';

			return $paginate;

		} else { return '<span></span>'; }

 }
  
  protected function FetchResults($adm, $data) {

    $this->sql->pageLimit = self::POST_LIMIT;
	 $this->sql->ObjectBuilder();
	 
	 switch ($data->type) {
		case 'videos': $this->sql->where('type', 'video'); break;
      case 'gifs': $this->sql->where('type', 'gif'); break;
      case 'photos': $this->sql->where('type', 'photos'); break;
      case 'articles': $this->sql->where('type', 'article'); break;
	 }
   
    switch ($data->status) {

		// By status
		case 'unpublished': $this->sql->where('status', 1, '<'); break;

      case 'hidden': $this->sql->where('status', -2); break;
      case 'drafts': $this->sql->where('status', -1); break;
      case 'awaiting': $this->sql->where('status', 0); break;
		case 'profile': $this->sql->where('status', 1); break;
		case 'published': $this->sql->where('status', 2); break;
		case 'unpublished': $this->sql->where('status', 1, '<'); break;

	 }
	 
	 switch ($data->order) {

		// By status
		case 'newest': $this->sql->orderBy('id', 'desc'); break;
		case 'oldest': $this->sql->orderBy('id', 'asc'); break;
		case 'likes': $this->sql->orderBy('likes', 'desc'); break;
		case 'views': $this->sql->orderBy('views', 'desc'); break;
		default: $this->sql->orderBy('czas','desc');

	 }

	 if (isset($data->user)) { $this->sql->where("user", $data->user); }
	 if (isset($data->search)) { $this->sql->where("tytul", "%{$data->search}%", "like"); }

    if (!$adm) { $this->sql->where('user', $this->nick); }
    return $this->sql->paginate('posts', $this->currPage);

  }
  
  protected function getPostPaging($adm, $total, $page, $search = '') {
    
    isset($this->get->page) ?: $this->get->page = 0;
     
    $var = (object) [
      'href' => ($adm ? 'adm' : 'profil') . '?' . $page,
      'next' => (int) $this->get->page + 1,
      'prev' => (int) $this->get->page - 1,
      'page' => (int) $this->get->page,
		'current' => abs($this->currPage),
		'search' => $search
	 ];

    return $this->paginator($total, self::POST_LIMIT, $var);
    
  }
   
  protected function getStatusIcon($id, $status, $comm = false) {

    switch ($status) {
		case -2: $st_icon = 'eye-blocked'; $st_text = Lang::get('post_hidden'); break;
		case -1: $st_icon = 'hammer'; $st_text = Lang::get('post_draft'); break;
		case  0: $st_icon = 'hour-glass'; $st_text = Lang::get('awaiting_approval'); break;
		case  1: $st_icon = 'user'; $st_text = Lang::get('post_profile'); break;
		case  2: $st_icon = 'checkmark'; $st_text = Lang::get('published'); break;
	 }
	 
	 if ($comm) {

			$flagged = $this->sql->where('id', $id)->getValue('comments','flagged');

			if ($flagged) {
				$st_icon = 'warning';
				$st_text = Lang::get('flagged', ['num' => $flagged]);
			} else {
				$st_icon = $status === 1 ? 'checkmark' : 'hour-glass';
				$st_text = $status === 1 ? Lang::get('published') : Lang::get('awaiting_approval');
			}

	 }

	  return '<a id="waiting_'.$id.'" title="'.$st_text.'" data-pos="left" data-offset="30" class="tip comm_status events-active icon-'.$st_icon.'"></a>';
	  
  }

  protected function parseUrlQuery($param) {
		$array = $this->params; 
		unset($array[ $param ], $array['page'], $array['search']);
		return str_replace('=&', '&', http_build_query($array));
  }
  
  protected function PostList($mode = false) {

    $adm = ($mode == 'admin') && User::isAdmin();
    
	 $_data = new stdClass;
    $_data->template = $adm ? 'Adm/posts_list' : 'Panel/panel_posts';
	 $_data->title = Lang::get($adm ? 'manage_posts' : 'submitted_posts');
	 
	 $_data->type   = isset($this->get->type) ? $this->get->type : null;
	 $_data->user   = isset($this->get->user) ? $this->get->user : null;
	 $_data->order  = isset($this->get->order) ? $this->get->order : null;
	 $_data->status = isset($this->get->status) ? $this->get->status : null;
	 $_data->search = filter_input(INPUT_POST, "search", FILTER_SANITIZE_SPECIAL_CHARS)
	 					?: filter_input(INPUT_GET, "search", FILTER_SANITIZE_SPECIAL_CHARS) ?: '';

	 $_data->order_query = $this->parseUrlQuery('order');
	 $_data->type_query  = $this->parseUrlQuery('type');

	 // Fetching results
    $results = $this->FetchResults($adm, $_data);
    
    if ($this->sql->count) {
      
      $_data->total = $this->sql->totalPages;
      $_data->paging = $this->getPostPaging($adm, $_data->total, 'posts', $_data->search);

       array_map(function($elem) {
         
          $elem->avatar = App::getAvatar($elem->user);
          $elem->czas = App::unixTimeArray($elem->czas);
			 $elem->url = App::postLink($elem);

			 $elem->published = $elem->status < 1;
			 $elem->numstatus = $elem->status;
			 $elem->draft = in_array($elem->status, [-2,-1]);
			 $elem->status = $this->getStatusIcon($elem->id, $elem->status);
			 $elem->catlink = 'category,' . App::addLine($elem->category);

     }, $results);
     
       $Loop_tpl = new Template( $adm ? 'Adm/posts_loop' : 'Panel/panel_posts_loop' );
       $_data->items = $Loop_tpl->render( compact($Loop_tpl->getTags()) );

    } else { $_data->items = '<div id="empty">'.Lang::get('nothing_found').'</div>'; }

    return $_data;
    
  }

  protected function _getMethod($page) {
		return implode('', array_map("ucfirst", explode('_', $page)));
	}
  
}