<?php

defined('APP_VERSION') || die("This file can't be accessed directly");

// Abstract class that shares common
// functionality for admin and users panel

abstract class PanelsEdit extends Validator {

   use Prepare;

   protected $sql;
   protected $conf;
   protected $post;

   protected $user;
   protected $admin;

   public function __construct() {

      User::Logged() ?: exit;

      $this->admin = User::isAdmin();
      $this->conf  = App::getConfig();
      $this->user  = User::get('username');
      $this->sql   = Factory::get('Database');
      $this->post  = $this->preparePost();

      $this->routeRequests($this->post->type);

   }

   private function routeRequests( $request ) {
      switch ($request) {
         case 'editpost': $this->EditPost(); break;
         case 'remove': $this->RemovePost(); break;
         case 'post_multi': $this->PostActions(); break;
      }
   }

   
   private function PostActions() {

      $posts  = $this->post->posts;
      $action = $this->post->action;

		is_array($posts) && !empty($posts) || die();
      $idArray = array_map('intval', $posts);

		// Updating posts statuses
      if ($action == "remove") {

         foreach ($idArray as $post_id) {
            $this->RemovePost( $post_id );
         }

		} else {

         $postStatus = $this->admin || $action == "hide"
            ? $this->statusByAction($action) 
            : $this->getPublishStatus()->id;

         $this->admin ?: $this->sql->where('user', $this->user);
         $result = $this->sql->where('id', ['IN' => $idArray])->update('posts', ['status' => $postStatus]);
			if (empty($this->sql->getLastError())) { echo $postStatus; }

		}

   }

   protected function getPublishStatus() {

      $user = Factory::get('User');
      $conf = App::Config('in_profile,autoaccept');

      switch (true) {
         case ($user->isadmin || $conf->autoaccept): $icon = 'checkmark'; $status = 2; break;
         case ($conf->in_profile): $icon = 'user'; $status = 1; break;
         default: $icon = 'hour-glass'; $status = 0;
      }

      return (object) [
         'icon' => $icon,
         'id' => $status
      ];

   }

   private function EditPost() {
      
      $id = (int) $this->post->pid;
      $this->isValid('title', $this->post->title);

      $rule   = $this->admin ? 'id=?' : 'id=? and user=?';
      $params = $this->admin ? [$id] : [$id, $this->user];

      if ($this->sql->countRows('posts', $rule, $params)) {
      
         if ($this->admin === false) {
            $locked = $this->sql->countRows('posts', 'id=? and locked=1', [$id]);
            if ($locked) { $this->Message('editing_locked', true); }
         }
         
         $tags = $this->prepareTags($this->post->tags);

         $data['lastedit'] = time();
         $data['tytul'] = $this->post->title;
         $data['tagi'] = str2var($tags, 'tags') ?: '';
         $data['category'] = $this->post->category ?: str2var($tags, 'category') ?: '';

         if ($this->post->post_type != 'article') { $data['opis'] = $this->post->desc ?: ''; }
         else { $data['opis'] = $this->prepareText($_POST['desc']); }
         
         $resp = $this->sql->where('id', $id)->update('posts', $data);
         if ($resp) echo 1;
         
      }

   }

   private function RemovePost($postid = NULL) {

      $id = (int) isset($postid) ? $postid : $this->post->id; $query = '';
      $results = $this->sql->ObjectBuilder()->where('id', $id)->getOne('posts');
      
      if ($this->sql->count && $id > 0 && empty($results->dir) === false) {

         $query = "DELETE ps.* FROM avm_posts ps
            LEFT JOIN avm_favorites   fv ON fv.pid    = ps.id
            LEFT JOIN avm_photos      ph ON ph.pid    = ps.id
            LEFT JOIN avm_comments    cm ON cm.wpis   = ps.id
            LEFT JOIN avm_notes       nt ON nt.parent = ps.id
            LEFT JOIN avm_daily_stats ds ON ds.pid    = ps.id
         WHERE ps.id = $id";

         if ($this->admin OR ($this->user == $results->user)) {

            $this->sql->rawQuery($query . (
               $this->admin ? ';' : " AND ps.user = '{$this->user}';"
            ));
            
            $mediadir = MEDIA . $results->dir;
            App::removeDir($mediadir);

         } // Has permission to delete

      } // Row exists

   }

   private function statusByAction($action) {
      switch ($action) {
			case 'hide': return -2;
			case 'draft': return -1;
			case 'moderation': return 0;
			case 'profile': return 1;
			case 'publish': return 2;
      }
   }

}