<?php

class Permissions {

    private $sql;
    private $user;
    private $permissions;
 
    public function __construct(Database $sql, $user)
    {
        $this->sql = $sql;
        $this->user = $user;
        $this->permissions = array();
        $this->parsePermissions();
    }

    public function get($perm) {
        $superadmin = $this->user->isadmin;
        $perm = str_replace('.', '-', $perm);
        return $superadmin ?: $this->permissions->$perm;
    }

    public function listPermissions() {

        $sql = "SELECT 
                t3.permission_id as id,
                t3.permission_name as 'name', 
                t3.permission_slug as 'slug', 
                MAX(CASE WHEN t4.group_name='guests' THEN t1.value END) as guests,
                MAX(CASE WHEN t4.group_name='members' THEN t1.value END) as members,
                MAX(CASE WHEN t4.group_name='moderators' THEN t1.value END) as mods,
                MAX(CASE WHEN t4.group_name='administrators' THEN t1.value END) as admins,
                IFNULL (t2.value, t1.value) as value 
            FROM permissions_groups as t1
            LEFT JOIN permissions_users as t2 ON t2.permission_id = t1.permission_id
            LEFT JOIN permissions as t3 ON t1.permission_id = t3.permission_id
            LEFT JOIN groups as t4 ON t4.group_id = t1.group_id
            GROUP BY t3.permission_name ORDER BY t3.list_order";

            return $this->sql->rawQuery($sql); 

    }

    public function getData() {
        return $this->permissions;
    }

    private function parsePermissions() {

        $sql = "SELECT t3.permission_slug as name, IFNULL (t2.value, t1.value) as value FROM permissions_groups as t1
                LEFT JOIN permissions_users as t2 ON t2.permission_id = t1.permission_id
                LEFT JOIN permissions as t3 ON t1.permission_id = t3.permission_id
                WHERE t1.group_id = {$this->user->group_id} OR t2.user_id = {$this->user->user_id}
                ORDER BY t3.permission_id";
        
        $data = $this->sql->rawQuery($sql);

        if ($this->sql->count !== 0) {
            foreach ($data as $row) {
                $this->permissions[ $row["name"] ] = (bool) $row["value"];
            }

            $this->permissions = (object) $this->permissions;
        }

    }

}