<?php

Class Render {
   
   private $cache;
   private $buffer;
   private $wrapper;
   
   public function __construct() {
      if (!defined('FILE_BUFFER')) define("FILE_BUFFER", 0x1);
      if (!defined('FILE_RETURN')) define("FILE_RETURN", 0x2);
   }
   
   // ================================
   // ======= Wrapping methods =======
   // ================================
   
   # Creates wrapper in memory
   # $tag and $selectors are params for createTag method
   # $tag (string) body, div etc.
   
   public function createWrapper($tag, $selectors) {
      $this->wrapper['open'] = $this->createTag($tag, $selectors, true);
      $this->wrapper['close'] = "</{$tag}>"; 
      return $this;
   }
   
   # Appends data to wrapper created with createWrapper method
   # You can append multiple items to wrapper using this method
   # $code (array/string) html code to put inside wrapper

   public function addToWrapper($code) {
      if (is_array($code)) foreach ($code as $html) $this->wrapper['inner'][] = $html;
      else $this->wrapper['inner'][] = $code;
      return $this;
   }
   
   # Returning html created with createWrapper and addToWrapper methods
   # $return (boolean) if true return output, else echo output
   # return $output or $this according to $return param value

   public function printWrapper($return = false) {
      $_w = $this->wrapper; $this->wrapper = [];
      $output = $_w['open'] . implode('', $_w['inner']) . $_w['close'];
      if ($return) return $output;
      echo $output; return $this;
   }
   
   # Creating wrapper for provided html code
   # $html (string) html code to wrap
   # $tag (string) html selector for wrapper 
   # $selectors (string) param for _makeTagAttr method
   # $opts (associative array) [before/after] code to add before / after wrapper

   public function wrap($html, $tag, $selectors = '', $opts = null, $return = false) {

      if (!$html || !$tag) return $this;
      
      $open = '<' . $tag . ($selectors ? $this->_makeTagAttr($selectors) : '') . '>';
      $close  = '</' . $tag . '>';

      $output = isset($before) ? $opts['before'] : '' . $open . $html . $close . $opts['after'];

      if ($return) return $output;
      echo $output; return $this;

   }
   
   public function createHeader($array, $return = false) {
      if (!empty($array)) { $output = '';
         foreach ($array as $key => $value) {
            switch ( (string) $key) {
               case 'title': $output .= "<title>{$value}</title>\n"; break;
               case 'meta' : foreach($value as $name => $cont) $output .= "<meta name=\"{$name}\" content=\"{$cont}\">\n"; break;
               case 'prop' : foreach($value as $name => $cont) $output .= "<meta property=\"{$name}\" content=\"{$cont}\">\n"; break;
               case 'js' : foreach($value as $path) $output .= "<script src=\"{$path}\"></script>\n"; break;
               case 'css' : foreach($value as $path) $output .= "<link href=\"{$path}\" rel=\"stylesheet\" type=\"text/css\">\n"; break;
               default : $output .= $value; break;
            }            
         }
         if ($return) return $output;
         echo $output; return $this;
      }
   }
   
   public function cache($data) { # Return Chain
      $this->cache .= $data; return $this; 
      
   }
   
   public function getCache($return = false) {
      if ($return) return  $this->cache;
      echo $this->cache; return $this;
   }
   
   public function clearCache() { # Return Chain
      $this->cache = '';
      return $this; 
   }
   
   public function rawHtml($html, $return = false) {
      if ($return) return $html; 
      else echo $html; return $this;
   }
   
   public function createTag($tag, $selectors='', $return = false, $close = false) {
      $output = '<' . $tag . ($selectors ? $this->_makeTagAttr($selectors) : '') . '>';
      if ($close) $output .= "</{$tag}>";
      if ($return) return $output;
      else echo $output; return $this;
   }
   
   public function closeTag($tag, $return = false) {
      $output = "</{$tag}>";
      if ($return) return $output;
      else echo $output; return $this;
   }
   
   private function _makeTagAttr($tags) {
      if ($tags) {
         $array = array_filter(array_map('trim', explode(',', $tags)));
         
         foreach ($array as $item) {
            $item = trim($item);
            switch ($item) {
               case $item[0] == '%' : $elem[] = substr($item,1,strlen($item)); break;
               case $item[0] == '#' : $elem[] = 'id="' .substr($item, 1, strlen($item)). '"'; break;
               case $item[0] == '.' : $classes[] = substr($item, 1, strlen($item)); break;
               case $item[0] == '+' : $exp = explode('=', substr($item, 1, strlen($item)));
                                      $data[] = $exp[0] . '="' .$exp[1]. '"'; break;
               case $item[0] == '@' : $exp = explode('=', substr($item, 1, strlen($item)));
                                      $data[] = 'data-' . $exp[0] . '="' .$exp[1]. '"'; break;
            }
         }
         
         isset($classes) && $elem[] = 'class="' . implode(' ', $classes). '"';
         isset($data) && $elem[] = implode(' ', $data);
         return ' ' . implode(' ', $elem);
      }
   }
   
   public function _injectScope() {
      $vars = ['_GET','_POST','_SERVER','_COOKIE','_FILES','_SESSION','GLOBALS','sql'];
      foreach ($GLOBALS as $name => $variable) if (!in_array($name, $vars)) $_vars[$name] = $variable;
      return isset($_vars) ? $_vars : [];
   }
   
   public function incFile($file, $flags = null, $vars = null) {
      
      extract($this->_injectScope());
      is_array($vars) && extract($vars);
      
      if ($flags & (FILE_BUFFER | FILE_RETURN)) ob_start();
      
      if (!empty($file)) {
         if (is_array($file)) foreach ($file as $elem) include($elem);
         else include($file);
      } 
      
      if ($flags & FILE_BUFFER) { $this->buffer[] = ob_get_clean(); }
      if ($flags & FILE_RETURN) return ob_get_clean();
      return $this;
      
   }
   
   public function buff_print($key = 0, $flush = false) {
      
      if (!empty($this->buffer)) {
         
         # Print all cached data and flush buffer
         
         if ( is_bool($key) ) {
            echo implode('', $code);
            $flush = true;
         }
         
         # Print selected keys from buffer 
         # and clear stored data
         
         elseif ( is_array($key) ) {
            foreach ($key as $pos) {
               echo $this->buffer[$pos];
               $this->buffer[$pos] = '';
            }
         }
         
         # Print specific key from buffer 
         # and clear stored data
         
         elseif ( is_integer($key) ) {
            echo $this->buffer[$key];
            $this->buffer[$key] = '';
         }
         
         if ($flush) $this->buffer = [];

      } return $this;
      
   } // END buff_print
   
   public function buff_get($key = false, $flush = false) {
      if (is_integer($key)) $output = $this->buffer[$key];
      else $output = implode('', $this->buffer);
      if ($flush) $this->buffer = [];
      return $output;
   }
   
   public function buff_flush() {
      $this->buffer = [];
      return $this;
   }
   
   private function prepareTag($path, $defer = false) {
      
      $ext = pathinfo($path, PATHINFO_EXTENSION);
      
      if ($ext == 'css' || strstr($path,'.css')) $mode = 'css';
      if ($ext == 'js' || strstr($path,'.js')) $mode = 'js';
      
      if ($defer) $defer = 'defer';
      
      switch($mode) {
         case 'css': return "<link href=\"{$path}\" rel=\"stylesheet\" type=\"text/css\">\r\n"; break;
         case 'js':  return "<script src=\"{$path}\" {$defer}></script>\r\n"; break;
      }
   }
   
   public function incTags($tag, $defer = false) {
      if (!empty($tag)) {
         if (is_array($tag)) foreach ($tag as $path) echo $this->prepareTag($path, $defer);
         else echo $this->prepareTag($tag, $defer);
      } return $this;
   }
   
   public function incModule($module, $flags = false) {

      $ver = APP_VERSION;
      $jsPath = 'Libs/Javascript';
      $mdPath = 'Libs/Modules';
      
      if ($flags & (FILE_BUFFER | FILE_RETURN)) ob_start();
      
      switch ($module) {
         case 'players': $this->incTags(['Template/Css/vsplayer.css?'.$ver], true); break;
         case 'jcrop': $this->incTags([$jsPath.'/jCrop/jcrop.css?'.$ver]); break;
         case 'dropzone': $this->incTags([$jsPath.'/Dropzone/dropzone.css?'.$ver]); break;
         case 'loginmodal': $this->incTags([$jsPath.'/LoginModal/loginmodal.css?'.$ver]); break;
         case 'tagseditor': $this->incTags([$jsPath.'/TagEditor/jquery.tagsinput.min.css?'.$ver]); break;
         case 'modal': $this->incFile("Template/AuthModal.php"); $this->incModule('loginmodal'); break;
         case 'photoswipe': $this->incTags([$jsPath.'/PhotoGallery/photoswipe.css?'.$ver, $jsPath.'/PhotoGallery/default-skin/default-skin.css?'.$ver]); break;
         case 'upload_pics': $this->incModule('dropzone'); break;
         case 'upload_article': $this->incTags([$mdPath.'/upload_article/article.css?'.$ver]); break;
         case 'upload_video': $this->incModule('players'); $this->incTags([$mdPath.'/upload_video/video.css?'.$ver]); break;
         case 'upload_gif': $this->incModule('players'); break;
      }
      
      if ($flags & FILE_RETURN) return ob_get_clean();
      
      return $this;

   }
   
   public function incModules($array, $flags = false) {
      if (!empty($array)) {
         if ($flags & (FILE_BUFFER | FILE_RETURN)) ob_start();
         foreach ($array as $module) $this->incModule($module);
         if ($flags & FILE_RETURN) return ob_get_clean();
      } return $this;
   }
   
   public function printScript($script) {
      $this->wrap($script,'script');
      return $this;
   }
   
}