<?php

class Request {
   
   use RequestErr;
   
   private $sql;
   private $time;
   private $user;
   private $post;
   private $get;
   private $web;
   private $notes;
   
   public function __construct() {

      if ($_SERVER["CONTENT_TYPE"] == 'ajax/reset') {
         $this->ResetCache();
      }
      
      if ($this->isAJAX()) { 
         $this->time = microtime(true);
         $this->user = Factory::get('User')->getData();
         $this->get  = App::array2obj($_GET); 
	      $this->post = App::array2obj($_POST);
         $this->sql  = Factory::get('Database');
         $this->web  = Factory::get('WebData');
         $this->notes = Factory::get('Notes');
         $this->Route();
      }
      
   }
   
   private function isAJAX() {
      return 
         !empty($_SERVER['HTTP_X_REQUESTED_WITH']) &&
         strtolower($_SERVER['HTTP_X_REQUESTED_WITH']) == 'xmlhttprequest';
   }
   
   private function Route() {
      
      if (isset($this->post->infinite)) {
         switch ($this->post->infinite) {
            case 'freewall': $this->Freewall(); break;
            default: $this->Infiniload();
         }
      }
      
      elseif ( isset($this->get) && isset($this->get->mode) ) { // $_GET requests
         switch ($this->get->mode) {
            case 'auth': (new Auth)->request(); break;
            case 'autocomplete': $this->Autocomplete(); break;
         }
      }
      
      elseif ( isset($this->post) && $this->post->mode ) { // $_POST requests
         switch ($this->post->mode) {
            case 'loadpics': $this->LoadPics(); break;
            case 'reset': $this->ResetCache(); break;
            case 'getmp4': $this->Convert(); break;
            case 'flag': $this->Flag(); break;
            case 'checkurl': $this->CheckUrl(); break;
            case 'checkurl2': $this->CheckUrl2(); break;
            case 'voting': new Voting('post'); break;
            case 'progress': $this->Progress(); break;
            case 'mediadata': $this->MediaData(); break;
            case 'url2dataURI': $this->Url2Data(); break;
            case 'getheaders': $this->GetHeaders(); break;
            case 'videotest': $this->VideoTest(); break;
            case 'download': $this->Download(); break;
            case 'sticky': $this->Sticky(); break;
            case 'follow': $this->Follow(); break;
            case 'stats': $this->Stats(); break;
            case 'guest_nick': $this->testGuestNick(); break;
            case 'post_table': $this->PostTable(); break;
            
            // Comments [ TODO: single mode / route by other value ]
            case 'comvote': case 'editcmt': case 'showcmt': case 'show_replies':
            case 'addrpl': case 'addcmt': case 'delcmt': case 'show_comments': $this->Comments();
         }
      }
      
   } // END Route

   private function testGuestNick() {
      (new Validator)->isValid('guest_nick', $this->post->nick);
   }

   private function LoadPics() {
      $pid = (int) $this->post->pid;
      $data = $this->sql->where('pid', $pid)->orderBy('position','asc')->get('photos');
      echo json_encode($data);
   }
   
   private function ResetCache() {
      App::removeDir(PATH_TEMP);
	}
	
	private function PostTable() {

		$params = ['sort' => $this->post->sort, 'limit' => 5];
      $fields = ['id','czas','tytul','type','user','status','category','dir','hash'];
      $query = $this->sql->prepareQuery($params, false, $fields);

		if ($results = $this->sql->rawQuery($query)) {

         $results = json_decode(json_encode($results));

			if ($this->sql->count) {
				array_map(function($elem) {
					$elem->catlink = 'category,' . App::addLine($elem->category);
					$elem->avatar = App::getAvatar($elem->user);
               $elem->czas = App::unixTimeArray($elem->czas);
               $elem->url = App::postLink($elem);
				}, $results);
  
				$Loop_tpl = new Template('Adm/popular_loop');
				echo $Loop_tpl->render( compact($Loop_tpl->getTags()) );
		  }

		}

	}
   
   private function Progress() {

      if (filter_var($this->post->url, FILTER_VALIDATE_URL)) {
         
         $mime = $this->post->mime;
         $filename = $this->post->url;
         $skip = $this->post->skipname;
         $filesize = (int) $this->post->size;
         
         if (!preg_match('/^[a-z0-9-]+$/i', $this->post->hash) && !$skip) {
            header("Request-Status: error"); 
            $this->message('error','0x017', true); 
         }
         
         if (!preg_match('/^[a-z34]{3,4}$/i', $this->post->ext)) {
            header("Request-Status: error"); 
            $this->message('error', '0x022', true);
         }

         header("Content-Transfer-Encoding: Binary");
         header("Content-Length: ". $filesize);
         header("Content-Disposition: attachment");

         App::createTempdir();
         $path = PATH_TEMP .'/'. $this->post->hash . '.' . $this->post->ext;

         $output = fopen($path, "w");
         $handle = fopen($filename, "rb");

         if (FALSE === $handle) { 
            header("Request-Status: error");
            $this->message('error', '0x021'); 
         }
         
         if (FALSE === $output) { 
            header("Request-Status: error");
            $this->message('error', '0x023'); 
         }
         

         while (!feof($handle)) {
             $binary = fread($handle, 1024*1024*10);
             fwrite($output, $binary); echo $binary;
         }

         fclose($handle);
         fclose($output);

         chmod_u($path, 0666);
         
      }
      
   }
   
   private function Flag() {
      
      $post = $this->post; $pid = (int) $post->id;
      $type = ($post->type == 'comment') ? 'comm_' : 'post_';
      
      $flagid = $type . $pid;
      $cook = Cookie::get('flags');
      // $sess = Session::get('flags');
      
      if ($cook || $sess) { // Flags exists
         
         $data = $cook ? unserialize($cook) : unserialize($sess);
         if ( in_array($flagid, $data) ) {
            $msg = ['status' => 'error', 'message' => Lang::get( $type . 'already_flagged')];
            die( json_encode($msg) );
         } else {
            array_push($data, $flagid);
            $this->addFlag($data, $flagid, $pid, $type);
         }
         
      } else { // No flags
         
         $data = array();
         $this->addFlag($data, $flagid, $pid, $type);
      }
      
   }
   
   private function addFlag($data, $flagid, $pid, $type) {
      
      array_push($data, $flagid);
      
      Cookie::set('flags', serialize($data));
      Session::set('flags', serialize($data));
      
      $msg = ['status' => 'success', 'message' => Lang::get($type .'flagged')];
      $this->sql->where('id', $pid)->update('comments', ['flagged' => $this->sql->inc(1)]);
      die( json_encode($msg) );
      
   }
   
   private function Stats() {

   $range = preg_replace('/[^a-z0-9]/i', '', $this->post->range);

      switch ($range) {

			case 'today': 
				$chartQuery = 'date = CURDATE()'; 
				$fnQuery = function($row) { return "$row >= UNIX_TIMESTAMP(CURDATE())"; };
				break;

			case 'week': 
				$chartQuery = 'YEARWEEK(`date`, 1) = YEARWEEK(CURDATE(), 1)'; 
				$fnQuery = function($row) { return "YEARWEEK(FROM_UNIXTIME($row), 1) = YEARWEEK(CURDATE(), 1)"; };
				break;

			case 'lastweek': 
				$chartQuery = 'date >= CURDATE() - INTERVAL DAYOFWEEK(CURDATE())+6 DAY AND date < CURDATE() - INTERVAL DAYOFWEEK(CURDATE())-1 DAY'; 
				$fnQuery = function($row) { return "$row >= UNIX_TIMESTAMP( CURDATE() - INTERVAL DAYOFWEEK(CURDATE()) +6 DAY ) and $row <= UNIX_TIMESTAMP( CURDATE() - INTERVAL DAYOFWEEK(CURDATE()) -1 DAY )"; };
				break;

			case 'lastmonth': 
				$chartQuery = 'YEAR(date) = YEAR(CURDATE() - INTERVAL 1 MONTH) AND MONTH(date) = MONTH(CURDATE() - INTERVAL 1 MONTH)'; 
				$fnQuery = function($row) { return "YEAR(FROM_UNIXTIME($row)) = YEAR(CURDATE() - INTERVAL 1 MONTH) AND MONTH( FROM_UNIXTIME($row) ) = MONTH(CURDATE() - INTERVAL 1 MONTH)"; };
				break;

			case '3months': 
				$chartQuery = 'date > CURDATE() - INTERVAL 3 MONTH'; 
				$fnQuery = function($row) { return "$row >= UNIX_TIMESTAMP(CURDATE() - INTERVAL 3 MONTH)"; };
				break;

			case 'year': 
				$chartQuery = 'YEAR(`date`) = YEAR(CURDATE())';
				$fnQuery = function($row) { return "YEAR( FROM_UNIXTIME($row) ) = YEAR(CURDATE())"; };
				break;

			case 'month': default: 
				$chartQuery = 'MONTH(`date`) = MONTH(CURDATE()) AND YEAR(`date`) = YEAR(CURDATE())';
				$fnQuery = function($row) { return "MONTH( FROM_UNIXTIME($row) ) = MONTH(CURDATE()) AND YEAR( FROM_UNIXTIME($row) ) = YEAR(CURDATE())"; };

      }

		// Fetching data for chart
		$results = $this->sql->rawQuery("SELECT date, SUM(views) AS data FROM daily_stats WHERE $chartQuery GROUP BY date ORDER BY date");
		$results = array_map(function($row) { $date = new DateTime( $row["date"] ); $row["date"] = $date->format('d M'); return $row; }, $results);

		$stats = $this->sql->rawQuery("

			SELECT 
			(SELECT SUM(views) FROM daily_stats WHERE $chartQuery LIMIT 1) as period_total,
			(SELECT count(id) FROM avm_posts WHERE {$fnQuery('czas')} LIMIT 1) as period_posts,
			(SELECT count(id) FROM avm_login WHERE {$fnQuery('regdate')} LIMIT 1) as period_users,

			(SELECT count(id) FROM avm_login LIMIT 1) as users,
			(SELECT count(id) FROM avm_posts LIMIT 1) as posts


		");

		$stats = $stats[0];
		$stats["period_total"] = is_numeric($stats["period_total"]) ? $stats["period_total"] : 0;
		$stats['period_average'] = round($stats['period_total'] / count($results), 2);

		$output['data'] = $results;

		echo json_encode( array_merge($output, $stats) );

   }
   
   private function Comments() {
      $e = new Comments;
      switch ($this->post->mode) {
         case 'delcmt' : $e->DeleteComment(); break;
         case 'showcmt': $e->ListComments(); break;
         case 'editcmt': $e->EditComment(); break;
         case 'show_replies': $e->ShowReplies(); break;
         case 'show_comments': $e->ShowComments(); break;
         case 'comvote': $e->VoteComment(); break;
         case 'addrpl': case 'addcmt' : $e->AddComment(); break;
      }
   }
   
   private function Infiniload() {
      
      // wait for 2 seconds
      usleep(250000);

      if (!$this->post->params) exit;
      $type = '';
      $params = json_decode($this->post->params);
      if ($this->post->infinite == 'profile') {
         
         switch ($params->sort) {
            case 'old': $order = 'id'; break;
            case 'new': $order = 'id desc'; break;
            case 'like': $order = 'likes desc'; break;
            case 'view': $order = 'views desc'; break;
            case 'fav': $order = 'fav desc'; break;
         }
         
         switch ($params->type) {
            case 'photos': $type = 'AND type="photos"'; break;
            case 'videos': $type = 'AND type="video"'; break;
            case 'music': $type = 'AND type="audio"'; break;
            case 'gifs': $type = 'AND type="gif"'; break;
            case 'news': $type = 'AND type="article"'; break;
         }
         
         $params->limit = ($params->view == 'list') 
            ? App::get('prof_num_lists') 
            : App::get('prof_num_icons');
         
         $page  = (int) $params->page;
         $start = (int) $page * $params->limit;
         $user  = App::sqlEsc($params->user);
         
         // Status > 0 ( 1: visible in profile only 2: published )
         $sqlRaw = "SELECT * FROM posts WHERE status > 0 AND logged = 1 AND user = '$user' $type
                    ORDER BY $order LIMIT $start, $params->limit";

      } elseif ($this->post->infinite == 'posts') {
         
         $params->limit = ($params->view == 'list') 
            ? App::get('page_num_lists') 
            : App::get('page_num_icons');
         
         if (isset($params->sort)) {
            if ($params->sort == 'following') {
               $params->following = $this->getFollowingList();
            }

            if ($params->sort == 'views') {
               $params->sort = 'thismonth';
               $params->subsort = 'views';
            }
         }
         
         $sqlRaw = $this->sql->prepareQuery($params)['query'];

      }

      $Posts = new Posts('posts'); usleep(200000);
      
      $results = ($params->view == 'list') ? $Posts->listView($sqlRaw) : $Posts->iconView($sqlRaw);
      if ($Posts->rows) { echo $results;
         if ($Posts->rows >= $params->limit) echo '<div id="loadmore"></div>';
      }
      
      
   }
   
   private function getFollowingList() {
      $users = $this->sql->where('kto', $this->user->username)->get('follow', null, ['kogo']);
      foreach ((array) $users as $user) { $following[] = 'user = "' . $user["kogo"] . '"'; }
      return $following ?: false;
   }
   
   private function Freewall() {
      
      $post = $this->post;
      $post->toload = $post->toload ?: 20;

      if ($post->toload && ($post->mode || $post->sort)) {
	
         $to = (int) $post->toload;
         $add = null; $type = null; $order = '';
         $from = (int) $post->page * $post->toload;
         
         switch ($post->mode) {
            case 'pics': $type = 'photos'; $add = 'type="photos"'; break;
            case 'gifs': $type = 'gif'; $add = 'type="gif"'; break;
            case 'vids': $type = 'video'; $add = 'type="video"'; break;
            case 'audio': $type = 'audio'; $add = 'type="audio"'; break;
            case 'news': $type = 'articles'; $add = 'type="article"'; break;
         }

         if (!$post->sort) { $order = 'order by id desc'; }
         else { switch ($post->sort) {
            case 'day': $add = $this->sql->popular('DAY', $type); break;
            case 'week': $add = $this->sql->popular('WEEK', $type); break;
            case 'month': $add = $this->sql->popular('MONTH', $type); break;
            case 'year': $add = $this->sql->popular('YEAR', $type); break;
         }}
         
         $Where = $add ? 'where status="2" and ' . $add : 'where status="2"';

         usleep(500000);

         $items = new Template('Wall/wall_item', ['scope' => false]);
         $results = $this->sql->ObjectBuilder()->query("select id,tytul,dir,hash,views,mediadata,type,category,mode,user,likes,fav,czas from posts {$Where} {$order}", [$from, $to]);

         array_map(function($elem) {
            
            $json = json_decode($elem->mediadata);
            
            $elem->encoder = str2var($json, 'encoder') ?: 'mp4';
            $elem->avatar = App::getAvatar($elem->user);
            $elem->time = App::zmienczas($elem->czas);
            $elem->uname = App::fakeName($elem->user);
            $elem->link = App::postLink($elem);
            $elem->path = 'Uploads/Media/' . $elem->dir;
            $elem->icon = App::getBrand(
               $elem->type, $elem->mode, str2var($json, 'source')
            );

         }, $results);

         $data = $items->render( compact('results') );

         echo '<div class="grid-sizer"></div><div class="gutter-sizer"></div>' . $data;
         if ($this->sql->count) { echo '<span class="wallappend"></span>'; }

      }
   }
   
   private function Follow() {
      
      $who  = $this->post->who;
      $user = $this->user->username;

      if ($this->user->logged && $this->sql->countRows('login','username=?', $who)) {
         if ($user != $who) {

            $exists = $this->sql->countRows('follow','kto=? and kogo=?',[$user, $who]);
            $exists ? $this->sql->where('kto=? and kogo=?', [$user, $who])->delete('follow')
                    : $this->sql->insert('follow', ['kto' => $user, 'kogo' => $who, 'czas' => $this->time]);

            $this->notes->followNote($exists ? 'unfollow' : 'follow', $who);
            echo $exists ? 2 : 1;

         } else { echo Lang::get('follow_self'); }
      } else { echo Lang::get('login2follow'); exit; }

   }
   
   private function Sticky() {
      $start = (int) $this->post->start; 
      App::smallList($start, 20);
   }
   
   private function Download() {
      if (filter_var($this->post->url, FILTER_VALIDATE_URL)) {
         set_time_limit(0); App::createTempdir();
         $path = PATH_TEMP .'/'. $this->post->hash .'.'. $this->post->ext;
         $this->web->Request($this->post->url, null, true)->SaveToFile($path);
         chmod_u($path, 0666);
         echo USER_TEMP .'/'. $this->post->hash .'.'. $this->post->ext;
      }
   }
   
   private function GetHeaders() {
      $headers = $this->web->Request($this->post->url, ['head' => true, 'json' => true]);
      die( $headers );
   }
   
   private function CheckUrl() {
      $video_data = App::parseMediaUrl($this->post->url);
      sleep(1); die( $video_data );
   }
   
   private function CheckUrl2() {
      $video_data = App::parseMediaUrl($this->post->url,true,false);
      die( $video_data );
   }
   
   private function VideoTest() {
      if (filter_var($this->post->url, FILTER_VALIDATE_URL)) {
         
         $video = json_decode(App::parseMediaUrl($this->post->url));
         
         if ( $video->source ) {
            
            $exs = $this->sql->countRows('posts', "mediadata like '%{$video->id}%'");
            if ($exs) die('exists');
            
            switch ($video->source) {
                  
               case 'twitter': 
               case 'soundcloud':
                  echo 'true'; 
                  break;
                  
               case 'youtube': 
                  $link = 'https://www.youtube.com/oembed?format=json&url=' . $this->post->url; 
                  $code = $this->web->Request($link, ['head' => true])['status-code'];
                  echo ($code == '200') ? 'true' : 'false';
                  break;
                  
               case 'facebook': 
                  $reg = '/(?:(?:http|https):\/\/)?(?:www.)?facebook.com\/[\.\w\/]+\/videos\/(\d+)/i';
                  $link = preg_match($reg, $this->post->url, $fb) ? "https://graph.facebook.com/{$fb[1]}" : NULL;
                  $code = $this->web->Request($link, ['head' => true])['status-code'];
                  echo ($code == '200') ? 'true' : 'false';
                  break;
                  
               default:
                  $code = $this->web->Request($this->post->url, ['head' => true])['status-code'];
                  echo ($code == '200' || $code == '501') ? 'true' : 'false';

            }
         }

      }
	}
	
	private function base64ToMime($base64) {

		$signatures = [
			'image/jpeg' => '/\/9j\//',
			'image/png' => '/iVBORw0KGg/',
			'image/gif' => '/R0lGOD/',
			'video/webm' => '/GkXfowEAAA/',
			'video/mp4' => '/AAAA[IG]GZ0eX/',
			'audio/mpeg' => '/SUQz[AB][gwA]AAAA/'
		]; 
		
		$slice = substr($base64, 0, 10);
		foreach ($signatures as $mime => $signature) {
			if (preg_match($signature, $slice)) {
				return $mime;
			}
		}

		return false;

	}
   
   private function Url2Data() {

		if (filter_var($this->post->url, FILTER_VALIDATE_URL)) {

			$binary = $this->web->Request($this->post->url);
			$base64 = base64_encode($binary);
			
			if (extension_loaded('fileinfo') && function_exists('finfo_open') && function_exists('finfo_buffer')) {
				
				$finfo = finfo_open(FILEINFO_MIME_TYPE);
				$mime = finfo_buffer($finfo, $binary);
				finfo_close($finfo);

			} else {

				// Mime type from signature
				$mime = $this->base64ToMime( $base64 );

    		}
	 
			 echo empty($mime) ? $base64 : 'data:' . $mime . ';base64,' . $base64;

		}

  }
   
   private function MediaData() {
      if (filter_var($this->post->url, FILTER_VALIDATE_URL)) {
         echo App::getMediaData($this->post->url);
      }
   }
   
   private function Convert() {
      if (filter_var($this->post->url, FILTER_VALIDATE_URL)) {
          
         $url = $this->post->url;
         
         $data = $this->web->Request("https://ezgif.com/gif-to-mp4?url={$url}");
         preg_match('[<img(.*?)src?="https(.*?)\.gif"(.*?)>]im', $data, $gifout);
         preg_match_all('[<input type="hidden" value="(.*?)" name="(.*?)">]is', $data, $output);

         $file = $output[1][0]; $token = $output[1][1];
         $options = ['file' => $file, 'token' => $token];
         $postvars = ['postvars' => $options];

         $resp = $this->web->Request('https://ezgif.com/gif-to-mp4/' . $file, $postvars);
         preg_match('[<source src="(.*?)" type="video/mp4">]', $resp, $mp4);
         echo (substr($mp4[1],0,2) == '//') ? 'https:' . $mp4[1] : $mp4[1];

      }
   }
   
   private function Autocomplete() {
      if ($this->get->e && preg_match('/[a-zA-Ząćęłńóś ]+/i', $this->get->e)) {
         $res = $this->sql->where('keyword','%'.($this->get->e).'%','LIKE')->get('tags');
         foreach ($res as $out) { $tagsArray[] = $out['keyword']; }
         echo $this->Arr2json($tagsArray);
      }
   }
   
   private function Arr2json($arr) {
      echo json_encode(array_unique($arr));
   }
   
}