<?php

Class Uploader {

   use RequestErr;

   const AVATAR_QUALITY = 95;
   const AVATAR_MAX_WIDTH = 300;

   private $sql;
   private $post;
   private $files;
   private $admin;
   private $logged;
   private $Drive;

   public function __construct() {

      $this->sql = Factory::get('Database');

      $this->files = $_FILES;
      $this->post = App::array2obj($_POST);

      $this->admin = User::isAdmin();
      $this->logged = User::Logged();
      $this->user = User::get('username');

      if (isset($this->post->option)) {
        $this->route($this->post->option);
      }

   }

   private function route($opt) {
      if ($opt) {switch ($opt) {
         case 'merge': $this->mergeVideo(); break;
         case 'imgedit': $this->thumbEdit(); break;
         case 'video': $this->uploadVideo(); break;
         case 'cover': $this->SetCover(); break;
         case 'avatar': $this->uploadAvatar(); break;
         case 'thumbnail': $this->uploadImage(); break;
         case 'import': $this->importImage(); break;
         case 'crop_image': $this->cropImage(); break;
         case 'chunk': $this->uploadChunk(); break;
         case 'profile_bg': $this->profileBg(); break;
         case 'remove': $this->removeUploadedImage(); break;
         case 'updateGallery': $this->updateGallery(); break;
      }}
   }

   private function profileBg() {

      User::Logged() ?: exit;

      if (isset($this->post->mode)) {

         $keywords = ['wallpaper','wallpapers','nature','life','colors','light','travel','technology','mobile','people','office'];
         $key = $keywords[array_rand($keywords)];

         switch ($this->post->mode) {

            case 'reset':
               $image = PATH_ROOT . '/Uploads/Avatars/' . $this->user . '-bg.jpg';
               file_exists($image) && unlink($image);
               die('Uploads/Avatars/profile-bg.jpg');
               break;

            case 'random':
               $image = new Image('https://source.unsplash.com/1280x720?' . $key);
               $imPath = 'Uploads/Avatars/' . $this->user . '-bg.jpg';
               $image->resize(1280, null)->toFile(PATH_ROOT . '/' . $imPath, 'image/jpeg', 75);
               die($imPath . '?' . rand(100,999));

         }

      }

      $file = $_FILES['image'];

      if (is_uploaded_file($file['tmp_name'])) {
         $im = new Image($file['tmp_name']);
         $imPath = 'Uploads/Avatars/' . $this->user . '-bg.jpg';
         $im->resize(1920, null)->toFile(PATH_ROOT . '/' . $imPath, 'image/jpeg', 80);
         echo $imPath . '?' . rand(100,999);
      }

   }

   private function SetCover() {
      if ($this->logged) {

         $id = (int) $this->post->id;
         $hash = preg_replace('/[^a-z0-9]/i','',$this->post->hash);

         // Searching and updating post in database
         if (!$this->admin) $this->sql->where('user', $this->user);
         $post = $this->sql->where('id', $id)->getOne('posts');
         if ($post) {
            echo (int) $this->sql->where('id', $id)->update('posts', ['hash' => $hash]);
         }

      }
   }

   private function getImages() {

      $files = glob(PATH_TEMP.'/*.{png,jpeg,jpg,gif}', GLOB_BRACE);
      if ($files) { foreach ($files as $i => $file) {
         if (strstr($file, 'hq_')) { @unlink($file); unset($files[$i]); }
      }}

      return $files;

   }

   private function updateGallery() {

      $id = (int) $this->post->pid;
      $data = $this->sql->ObjectBuilder()->where('id', $id)->getOne('posts');
      $perm = $data->user == $this->user || $this->admin;

      if ($this->logged && $perm) {

         $json = (array) json_decode($this->post->assets);
         $pics = $this->sql->where('pid', $id)->get('photos');

         // Processing remove queue if set
         if ($queue = json_decode($this->post->queue)) {
            foreach ($queue as $img) $this->removeExistingImage($img);
         }

         // Updating indexes
         if ($pics) { foreach($pics as $pic) {
            $desc = $json[ $pic["file"] ]->desc;
            $title = $json[ $pic["file"] ]->title;
            $index = (int) $json[ $pic["file"] ]->index;
            $this->sql->where('pid=? and file=?', [$id, $pic["file"]])
                      ->update('photos', [
                          'title' => $title ?: '',
                          'position' => $index, 
                          'opis' => $desc ?: ''
                        ]);
         }}

         // Appending new images to gallery
         if ($files = $this->getImages()) {

            foreach($files as $image) {

               $img = App::getImageData($image);
               $name = $img->filename;

               $index = (int) $json[ $name ]->index;
               $type = preg_replace('/[^a-z]/','',$json[ $name ]->type);
               $desc = Filter::_String($json[ $name ]->desc);

               $dim = round($img->width / $img->height, 2);
               rename($image, MEDIA . $data->dir .'/'. $name);

               if (!stristr($name, 'm_') && $img->extension != 'gif') {
                  $values[] = "($id, '{$data->user}', '{$name}', $data->czas, $img->width, $img->height, $dim, '{$data->dir}', '{$desc}', $index, '{$type}')";
               } unset($type);

            }

            $this->sql->rawQuery("
               INSERT INTO photos (pid,user,file,time,width,height,orientation,dir,opis,position,type)
               VALUES " . implode(',', $values)
            );

         }

      }

   }

   private function thumbEdit() {

      $id = (int) $this->post->id;
      $data = (object) $this->sql->where('id', $id)->getOne('posts', 'dir,hash,user,logged');
      $author = $this->post->type == 'useredit' && User::get('username') === $data->user;

      if ($this->admin || $author) {

         $relPath = $data->dir . '/' . $data->hash;
         $tmpName = $this->files['file']['tmp_name'];

         if ($this->imageValid($tmpName)) {

            $this->saveCover($tmpName, MEDIA . $data->dir, $data->hash);
            $rand = rand(10000,99999); echo json_encode([
               'status' => 'saved',
               'img' => 'Uploads/Media/' .$relPath. '.jpg?' . $rand
            ]);

         }

      }

   }

   private function uploadAvatar() {

    if (!$this->logged) exit;

      $Path = PATH_UPLD . '/Avatars';
      $User = User::get('username');

      $tmpName = $this->files['file']['tmp_name'];

      if ($getsize = $this->imageValid($tmpName))
         {
            $width = $getsize[0]; $height = $getsize[1];
            $image = new Image($tmpName);
            $image->resize(self::AVATAR_MAX_WIDTH, null);
            $image->toFile($Path.'/'.$User.'.jpg', 'image/jpeg', self::AVATAR_QUALITY);
            $fileTab['status'] = 'saved';
            $fileTab['filename'] = $User.'.jpg';
            echo json_encode($fileTab);
         }

   }

   private function removeExistingImage($img = null) {

      if ($this->post->pid || $img->pid) {

         $id = (int) $this->post->pid ?: $img->pid;
         $file = $this->post->image ?: $img->image;

         if (!$this->admin) { $this->sql->where('user', $this->user); }
         $data = $this->sql->where('pid=? and file=?',[ $id , $file.'.jpg' ])->getOne('photos');

         if ($data) {
            $path = MEDIA .'/'. $data["dir"];
            foreach(glob($path . "/*$file.*") as $rm) @unlink($rm);
            $this->sql->where('id', $data["id"])->delete('photos');
         }

      }

   }

   private function removeUploadedImage() {

      // Removing related files
      $files = glob(PATH_TEMP . "/*{$this->post->image}.*");
      foreach($files as $ftr){ @unlink($ftr); }

   }

   private function cropImage() {

      $img = new Image(PATH_ROOT .'/'. $this->post->cropPath);
      $cropArea = explode('#', $this->post->cropData);
      $saveFile = substr(basename($this->post->cropPath), 3);

      $x1 = round($cropArea[0]);
      $y1 = round($cropArea[1]);
      $x2 = round($x1 + $cropArea[2]);
      $y2 = round($y1 + $cropArea[3]);

      $img->crop($x1, $y1, $x2, $y2)->toFile(PATH_TEMP .'/'. $saveFile, 'image/jpeg');
      $img->resize(App::get('thumb_max_width'), null)->toFile(PATH_TEMP .'/m_'. $saveFile, 'image/jpeg');

   }

   private function saveHQimage($source, $hash) {
      $img = new Image($source);
      $binary = $img->toString('image/jpeg');
      file_put_contents(PATH_TEMP . "/hq_$hash.jpg", $binary);
      chmod_u(PATH_TEMP . "/hq_$hash.jpg", 0666);
   }

   private function saveCover($source, $destination, $hash, $cover = true) {

      $size = getimagesize($source);
      $prefix = $cover ? '' : 'm_';

      $width = $cover ? App::get('cover_max_width') : App::get('thumb_max_width');
      $quality = $cover ? App::get('cover_quality') : App::get('thumb_quality');

      $_Image = new Image($source);
      if ($size > $width || !$cover) { $_Image->resize($width, null); }

      $binary = $_Image->toString('image/jpeg', $quality);
      file_put_contents($destination . "/{$prefix}{$hash}.jpg", $binary);
      chmod_u($destination . "/{$prefix}{$hash}.jpg", 0666);

      $cover && $this->saveCover($source, $destination, $hash, false);

   }

   private function importImage() {

        $mime = ['image/gif','image/png','image/jpeg'];
        $headers = (new WebData)->Request($this->post->url, ['head' => true]);

        if ($headers && is_array($headers) && in_array($headers["content-type"], $mime)) {

            $binary = '';      // Binary data holder
            $saveHQ = false;   // In case of gif file
            $hash = $this->post->hash;

            header("Content-Encoding: none");
            header("Content-Transfer-Encoding: Binary");
            header("Content-Length: ". intval($this->post->size));
            header("Content-Disposition: attachment");
    
            App::createTempdir();
    
            $handle = fopen($this->post->url, "rb");
                while (!feof($handle)) {
                    $chunk = fread($handle, 1024*1024*10);
                    $binary .= $chunk; echo $chunk;
                }
            fclose($handle);
    
            if ($this->post->mime == 'image/gif') {
                $saveHQ = true;
                $filename = $hash . '.gif';
            } else {
                $filename = 'hq_' . $hash . '.jpg';
            }

            $path = PATH_TEMP . '/' . $filename;
    
            file_put_contents($path, $binary);
            $saveHQ && $this->saveHQimage($path, $hash);
            $this->saveCover($path, PATH_TEMP, $hash);
            chmod_u($path, 0666);

        } // Is valid

   }

   private function uploadImage() {

      $post = $this->post;
      $file = $this->files['file'];

      header("Content-Transfer-Encoding: Binary");
      header("Content-Length: ". filesize($file['tmp_name']));
      header("Content-Disposition: attachment");

      $file = (object) [
            'mime' => $file['type'],
            'name' => $file['name'],
            'temp' => $file['tmp_name'],
            'hash' => hash_file('md5', $file['tmp_name'])
        ];

      if ($this->imageValid($file->temp)) {

         $video = isset($post->type) && $post->type == 'video';
         App::createTempdir();

         $hash = $video || isset($this->post->hash) 
            ? preg_replace('/[^a-z0-9]/', '', $this->post->hash)
            : hash('crc32b', microtime() . $file->name);

         $video ?: $this->saveHQimage($this->files['file']['tmp_name'], $hash);
         $this->saveCover($file->temp, PATH_TEMP, $hash);

         if ($file->mime == 'image/gif') {
            rename($file->temp, PATH_TEMP . "/$hash.gif");
            chmod_u(PATH_TEMP . "/$hash.gif", 0666);
         }

         echo json_encode([
             'status' => 'saved', 
             'filename' => "$hash.jpg",
             'userdir' => User::get('hash')
         ]);

      }

   }

   private function mediaValid($file, $options) {

      foreach($options as $test) { switch ($test) {

         case 'file_valid':
            if (empty($this->files) || !is_uploaded_file($file)) {
               $this->message('error', '0x013');
            } break;

         case 'file_mime':
            if (!in_array($this->getMime($file), ['video/mp4','video/webm','audio/mpeg','video/quicktime'])) {
               $this->message('error','0x019');
            } break;

         case 'file_size':
            $size = App::get('max_video_size');
            if ($size && number_format(filesize($file) / 1048576 , 2) > $size) {
               $this->message('error','0x020');
            } break;

         case 'file_error':
            if ($this->files['file']['error'] !== UPLOAD_ERR_OK) {
               $msg = $this->fileErrHandler($this->files['file']['error']);
               $this->message('error', $msg);
            } break;

      }}

      return true;

   }

   private function uploadVideo() {

      $file = $this->files['file']['tmp_name'];
      $size = (int) $this->files['file']['size'];

      if ($this->mediaValid($file, ['file_error','file_valid','file_mime','file_size'])) {

         $this->createTempDir();
         $mime = $this->getMime($file);
         $type = App::mime2type($mime);
         $extension = App::mime2ext($mime);
         $hash = isset($this->post->hash) ? $this->post->hash : false;

         // File hash validation (error on fail)
         $fileHash = ($hash && preg_match('/^[a-z0-9]+$/', $hash))
            ? $hash : $this->message('error','0x017');

         $output = "{$fileHash}.{$extension}";
         $move = move_uploaded_file($file, PATH_TEMP . "/$output");
         if (!$move) { $this->message('error', '0x016'); }

         $fdata = [
            'binary' => true,
            'mime'   => $mime,
            'bytes'  => $size,
            'type'   => $type,
            'encoder' => $extension
         ];

         // Saving file data for later processing
         file_put_contents(PATH_TEMP.'/mediadata.json', json_encode($fdata));

         chmod_u(PATH_TEMP . "/$output", 0666);
         chmod_u(PATH_TEMP.'/mediadata.json', 0666);

         if ($phpErr = error_get_last()) {
            $this->message('error', $phpErr["message"]);
         } else { $this->message('success', 'File saved'); }

      } else { $this->message('error', '0x013'); }

   }

   private function uploadChunk() {

      $index = isset($_SERVER['HTTP_X_INDEX']) ? $_SERVER['HTTP_X_INDEX'] : $this->message('error', '0x001');
      $fname = isset($_SERVER['HTTP_X_FILE_NAME']) ? $_SERVER['HTTP_X_FILE_NAME'] : $this->message('error', '0x002');
      $fsize = isset($_SERVER['HTTP_X_FILE_SIZE']) ? $_SERVER['HTTP_X_FILE_SIZE'] : $this->message('error', '0x003');

      if (!preg_match('/^[-a-z0-9_][-a-z0-9_.]*$/i', $_SERVER['HTTP_X_FILE_NAME'])) {
         $this->message('error', '0x004');
      }

      $file = $this->files['file']['tmp_name'];

      if ($this->mediaValid($file, ['file_error','file_valid'])) {
         if ($this->files['file']['type'] == "application/octet-stream") {

            $this->createTempDir();

            $serverFile = $fname . '-' . $index;
            $move = move_uploaded_file($file, PATH_TEMP . "/$serverFile");
            if (!$move) { $this->message('error', '0x016'); }
            else { chmod_u(PATH_TEMP . "/$serverFile", 0666); }

         } else { $this->message('error', '0x014'); }
      } else { $this->message('error', '0x013'); }

   }

   private function mergeVideo() {

      $hash = isset($this->post->hash) ? $this->post->hash : false;

      // File hash validation (error on fail)
      $fileHash = ($hash && preg_match('/^[a-z0-9]+$/', $hash))
         ? $hash : $this->message('error','0x017');

      $prefix = PATH_TEMP ."/{$fileHash}";
      $tempfile = "{$prefix}.temp";

      if ($handle = fopen($tempfile, 'wb')) {

         for ($i = 0; $i < $this->post->index; $i++) {
             $slice = $prefix . '-' . $i;
             $src = fopen($slice, 'rb');
               stream_copy_to_stream($src, $handle);
             fclose($src); unlink($slice);
         }

         fclose($handle);
         chmod_u($tempfile, 0666);

         // Validating mime type and file size
         $this->mediaValid($tempfile, ['file_mime','file_size']);

         $mime = $this->getMime($tempfile);
         $extension = App::mime2ext($mime);
         $output = "{$prefix}.{$extension}";

         rename($tempfile, $output);

         $fdata['mime'] = $mime;
         $fdata['binary'] = true;
         $fdata['bytes'] = filesize($output);
         $fdata['type'] = App::mime2type($mime);
         $fdata['encoder'] = $extension;

         // Saving file data for later processing
         file_put_contents(PATH_TEMP.'/mediadata.json', json_encode($fdata));
         chmod_u(PATH_TEMP.'/mediadata.json', 0666);

         if ($phpErr = error_get_last()) {
            $this->message('error', $phpErr["message"]);
         } else { $this->message('success', 'File saved'); }

      } else { $this->message('error','0x018'); }

   }

   // ====================== HELPERS =============================

   private function createTempDir() {
      if (!is_dir(PATH_TEMP)) {
         $oldmask = umask(0); $mk = mkdir(PATH_TEMP, 0777, true); umask($oldmask);
         if (!$mk) { $this->message('error', '0x015'); }
      }
   }

   private function getMime($path) {

      // FileInfo Extension
      if (extension_loaded('fileinfo') && function_exists('finfo_open') && function_exists('finfo_file')) {

         $finfo = finfo_open(FILEINFO_MIME_TYPE);
         $mime = finfo_file($finfo, $path);
         finfo_close($finfo); return $mime;

      } elseif (function_exists('mime_content_type')) { // Could be depreciated

         return mime_content_type($path);

      } else { // Let's try determine mime by signature

         // Fetching first 1024 bytes from binary
         $fp = fopen($path, 'r'); $binary = fgets($fp, 1024); fclose($fp);

         $bs64 = substr(base64_encode($binary), 0, 10);
         if (preg_match('/^GkXfowEAAA$/', $bs64)) { return 'video/webm'; }
         elseif (preg_match('/^AAAA[IG]GZ0eX$/', $bs64)) { return 'video/mp4'; }
         elseif (preg_match('/^SUQz[AB][gwA]AAAA$/', $bs64)) { return 'audio/mpeg'; }

      }

      return false;

   }

   private function imageValid($file) {
      $size = getimagesize($file);
      $uploaded = is_uploaded_file($file);
      if ($size && $uploaded) return $size;
   }

   // Upload errors related to $_FILES
   private function fileErrHandler($code) {
      switch ($code) {
         case UPLOAD_ERR_INI_SIZE:     return "0x005";
         case UPLOAD_ERR_FORM_SIZE:    return "0x006";
         case UPLOAD_ERR_PARTIAL:      return "0x007";
         case UPLOAD_ERR_NO_FILE:      return "0x008";
         case UPLOAD_ERR_NO_TMP_DIR:   return "0x009";
         case UPLOAD_ERR_CANT_WRITE:   return "0x010";
         case UPLOAD_ERR_EXTENSION:    return "0x011";
         default:                      return "0x012";
        }
    }

}