<?php

class Validator {
   
   protected $error;

   public function __construct() {
      $this->sql = Factory::get('Database');
   }
   
   protected function regTest($field, $value) {
      $regExp['activation'] = '/^[0-9a-z]{30}$/i';
      $regExp['fakename'] = '/^[\p{L}A-Za-z0-9\s]+$/iu';
      $regExp['category'] = '/^[\p{L}A-Za-z0-9\s]+$/iu';
      $regExp['user'] = '/^[a-zA-Z0-9_-]{' . App::get('user_minlen') . ',' . App::get('user_maxlen') . '}$/';
      $regExp['pass'] = '/^(?=.*\d)[a-zA-Z0-9!#$%_-]{' . App::get('pass_minlen') . ',}$/';
      return !empty($value) ? preg_match($regExp[$field], $value) : false;
   }
   
   public function isValid($rule, $value = '') {

      $return = false; $rules = [
         
         'title' => function($value) {
            mb_strlen($value) >= (int) App::get('title_minlen') ?: $this->Message('title_minlen', 1);
            mb_strlen($value) <= (int) App::get('title_maxlen') ?: $this->Message('title_maxlen', 1);
         },

         'email' => function($value) {
            filter_var($value, FILTER_VALIDATE_EMAIL) ?: $this->Message('email_invalid');
         },

         'guest_nick' => function($value) use (&$rules) {
            $rules["username"]($value, true);
            $out = $this->sql->countRows('login', "username=?", $value);
            if ($out) $this->Message('name_taken', true);
            echo 1;
         },
         
         'fakename' => function($value) use (&$return) {
            $return = true;
            if ($this->regTest('fakename', $value)) {
               $vars = [$value, $value, User::get('username')];
               $out = $this->sql->countRows('login', "(username=? or user_name=?) and username != ?", $vars);
               return $out ? $this->Message('name_taken', true) : $value;
            } return false;
         },
         
         'username' => function($value, $die = false) {
            mb_strlen($value) >= (int) App::get('user_minlen') ?: $this->Message('user_minlen', $die);
            mb_strlen($value) <= (int) App::get('user_maxlen') ?: $this->Message('user_maxlen', $die);
            $this->regTest('user', $value) ?: $this->Message('login_invalid');         
         },
         
         'category' => function($value) {
            mb_strlen($value) >= (int) App::get('cat_minlen') ?: $this->Message('cat_minlen');
            mb_strlen($value) <= (int) App::get('cat_maxlen') ?: $this->Message('cat_maxlen');
            $this->regTest('category', $value) ?: $this->Message('category_invalid');         
         },
         
         'comment' => function($comm, $nick) {
            mb_strlen($comm) > (int) App::get('comm_minlen') ?: $this->Message('comm_minlen');
            User::get('logged') ? true : (!User::userExists($nick) ?: $this->Message('login_taken'));
         },
         
         'password' => function($value) {
            mb_strlen($value) >= (int) App::get('pass_minlen') ?: $this->Message('pass_minlen');
            $this->regTest('pass', $value) ?: $this->Message('pass_invalid');
         },
         
         'pass2pass' => function($pass1, $pass2) use (&$return) {
            $return = true;
            !empty($pass1) && !empty($pass2) ?: $this->Message('pass_minlen', true);
            mb_strlen($pass1) >= (int) App::get('pass_minlen') ?: $this->Message('pass_minlen', true);
            return $pass1 === $pass2;
         }
         
      ];
      
      $output = is_array($value) 
         ? call_user_func_array($rules[$rule], $value)
         : call_user_func($rules[$rule], $value);
      
      return $return ? $output : ($this->error ? false : true);
      
   } // END isValid
   
   protected function Message($code, $die = false) {

      $error = Lang::get($code);
      
      if ($error) {
         $this->error[] = $error;
         return $die ? die($error) : $error;
      } else {
         die('Translation for "' . $code .'" not found.');
      }
      
   } // END Message
   
   protected function getLastError($die = false) {
      $error = array_shift($this->error);
      return $die ? die($error) : $error;
   }
   
}