<?php

/* Post voting class */

Class Voting {
   
   use Stats;
   
   private $sql;
   private $post;
   private $user;
   private $notes;
   
   public function __construct() {
      
      $this->sql = Factory::get('Database');
      $this->notes = Factory::get('Notes');
      $this->prepare(); $this->submit();
      
   }
   
   private function submit() {
      switch ($this->post->vote) {
         case 'fav': $this->favorite(); break;
         default: $this->likes(); break;
      }
   }
   
   private function likes() {
      
      User::can('vote') ?: $this->info('log2vote', true);
      
      $post = $this->post;
      
      $query = "pid=? and item=? and (action='likes' OR action='dislikes') and userhash=?";
      $params = [$post->id, $post->item, $this->user->hash];

      if ( !$this->sql->countRows('user_stats', $query, $params) ) {

         $this->insert('user_stats');
         $this->notes->postVoting( $this->post );

         if ($this->user->signed and $post->vote == 'likes') {
            $this->insert('favorites');
         }

         if ($post->vote == 'likes') {
            $this->message = ($post->item == 'photo')
               ? $this->info('pic_like')
               : $this->info('pos_like');
         } else { 
            $this->message = $this->info('thx'); 
         }

         $this->update('add');

         
      } else {
      
         $stats = $this->getStats('user_stats', $query, $params);
         
         $this->action = $stats->action;
         $this->inserted = $stats->date;
         
         if ($post->vote == $this->action) { // Cofnięcie głosu

            $tag = $post->item == 'post' ? 'psvt' : 'phvt';
            $this->notes->removeNote($tag, $stats->time);

            if ($stats->username and $post->vote == 'likes') {
               $query = "username=? and pid=? and item=? and action='likes'";
               $params = [$stats->username, $post->id, $post->item];
               $this->sql->where($query, $params)->delete('favorites');
            }
            
            $this->sql->where($query, $params)->delete('user_stats');
            $this->update('remove');
            
         } else { // Zmiana głosu

            $this->sql->where($query, $params)->update('user_stats', ['action' => $post->vote]);
            $this->notes->postVoting( $this->post , 'update' , $stats->time );

            if ($this->user->signed) {
               if ($post->vote == 'likes') {
                  $this->insert('favorites');
               } else {
                  $this->sql->where("pid=? and item=? and username=? and action='likes'",
                     [ $post->id, $post->item, $this->user->nick ]
                  )->delete('favorites');
               }
            }

            $this->update('mixed');
            
         }
      }
      
   }
   
   private function favorite() {
      
      if ($this->user->signed) {
         
         $item = $this->post->item;
         
         $query = "pid=? and item=? and action='fav' and username=?";
         $params = [$this->post->id, $item, $this->user->nick];
         
         if ( !$this->sql->countRows('favorites', $query, $params) ) { // Jeśli postu nie ma w ulubionych
            
            $this->insert('favorites');
            $this->message = ($item == 'photo') 
               ? $this->info('pic_fav')
               : $this->info('pos_fav');
            
            $this->notes->postVoting( $this->post ); 
            $this->update('add');

         } else {

            $stats = $this->getStats('favorites', $query, $params);
            
            $this->inserted = $stats->date;
            $this->message = ($item == 'photo') 
               ? $this->info('pic_unfav')
               : $this->info('pos_unfav');

            $tag = ($item == 'post') ? 'psfv' : 'phfv';
            $this->notes->removeNote($tag, $stats->time); 
            $this->sql->where($query, $params)->delete('favorites');
            $this->update('remove');

         }
         
      } else {
         $this->info('login', true);
      }
      
   }
   
   private function getStats($table, $query, $params) {
      
      $rows = ($table == 'favorites') 
         ? 'time, date' 
         : 'time, date, username, action';
         
      return $this->sql->ObjectBuilder()
         ->where($query, $params)
         ->getOne($table, $rows);
      
   }
   
   private function insert($table) {
      $this->sql->insert($table, [
         'pid' => $this->post->id, 
         'item' => $this->post->item, 
         'action' => $this->post->vote, 
         'userhash' => $this->user->hash, 
         'username' => $this->user->nick,
         'date' => $this->getDate(), 
         'time' => $this->getTime()
      ]);
   }
   
   private function update($action) {

      $post = $this->post;
      $table = $post->item == 'photo' ? 'photos' : 'posts';

      if ($action == 'add') {
         $this->sql->where('id', $post->id)->update($table, [$post->vote => $this->sql->inc(1)]);
         $this->dailystats($post->id, $post->item, $post->type, $post->vote); 

      } elseif ($action == 'remove') {
         $this->sql->where('id', $post->id)->update($table, [$post->vote => $this->sql->dec(1)]);
         $this->dailystats($post->id, $post->item, $post->type, $post->vote, true, $this->inserted); 

      } elseif ($action == 'mixed') {

         $this->sql->where('id', $post->id)->update($table, [$this->action => $this->sql->dec(1), $post->vote => $this->sql->inc(1)]);
         $this->dailystats($post->id, $post->item, $post->type, $this->action, true, $this->inserted); 
         $this->dailystats($post->id, $post->item, $post->type, $post->vote); 

      }
      
      $results = $this->sql->where('id', $post->id)->getOne($table, 'likes,dislikes,fav');
      $results['info'] = isset($this->message) ? $this->message : '';
      $results['allvotes'] = $results['likes'] - $results['dislikes'];
      if ($results['likes'] !== 0 && ($results['likes'] + $results['dislikes']) !== 0) {
         $results['prc'] = (
            $results['likes'] / ( $results['likes'] + $results['dislikes'] )
         ) * 100;
      } else { $results['prc'] = 0; }

      echo json_encode($results);

   }
   
   private function info($type, $break = false) {
      
      $info = [
         'thx' => Lang::get('vote_saved'),
         'log2vote' => Lang::get('log_2_vote'),
         'login' => Lang::get('log_2_fav'),
         'pic_like' => Lang::get('vote_submitted'),
         'pos_like' => Lang::get('vote_submitted'),
         'pic_fav' => Lang::get('img_fav_added'),
         'pos_fav' => Lang::get('post_fav_added'),
         'pic_unfav' => Lang::get('img_fav_removed'),
         'pos_unfav' => Lang::get('post_fav_removed')
      ];

      return $break ? die( $info[$type] ) : $info[$type];
   }
   
   private function getTime() {
      if (!isset($this->time)) {
         $this->time = microtime(true);
      }
      return $this->time;
   }
   
   private function getDate() {
      if (!isset($this->date)) {
         $this->date = date("Y-m-d");
      }
      return $this->date;
   }
   
   private function sanitize($string) {
      return strip_tags( $string );
   }
   
   private function prepare() {

      if (!$this->post) $this->post = new stdClass();
      
      $this->post->id = (int) $_POST['id'];
      $this->post->time = $this->getTime();
      $this->post->item = $this->sanitize($_POST['voteItem']);
      $this->post->type = $this->sanitize($_POST['postType']);
      $this->post->vote = $this->sanitize($_POST['voteType']);
      
      $this->user = (object) [
         'signed' => User::Logged(),
         'nick' => User::get('username') ?: '',
         'hash' => User::get('iphash')
      ];
      
   }
   
}