<?php

class AddArticle extends Submit {
   
   private $remove;
   private $cover;
   
   public function __construct($_type) {
      parent::__construct($_type);
      $this->prepareText();
      $this->prepareImages();
      $this->submit();
   }
   
   private function deleteImages() {
      $images = (array) glob(PATH_TEMP .'/hq_*');
      foreach ($this->remove as $filename) {
         if ($files = glob(PATH_TEMP .'/*'. $filename)) {
            foreach ($files as $file) { $images[] = $file; }
         }
      } foreach ($images as $del) { 
         @unlink($del); 
      }
   }
   
   private function submit() {

      $field = $this->output;
      
      $this->countTags($field->tags);
      
      $postID = $this->sql->insert('posts', [
         'tytul' => $field->title, 'opis' => $field->desc,
         'czas' => $field->time, 'user' => $field->user, 
         'tagi' => isset($field->tags) ? $field->tags : '',
         'category' => isset($field->category) ? $field->category : '', 
         'logged' => $field->logged, 'status' => $field->status, 
         'type' => $field->type
      ]);
      
      if ($postID) {
         
         $this->deleteImages();
         $this->addTags($field->tags);
         
         $dir = $this->makeFolder($postID);
         $files = $this->moveFiles($dir->save);
         
         $fname = basename( $files[0] );
         $hash = $this->cover ? 'cover' : substr($fname, 0, strpos($fname, '.'));
         
         if ($field->logged) { $this->note->addNote('', $field->user, 'post', '', $postID, json_encode( array('post_type' => $field->type) )); }
         $this->output->desc = str_replace(USER_TEMP, $dir->path, $this->output->desc);
			
         $this->sql->where('id', $postID)->update('posts', [
            'opis' => $this->output->desc, 'dir' => $dir->date, 'hash' => $hash
         ]);
         
         sleep(1); echo 1;

      }
		
   }
   
   private function prepareImages() {

      if (!preg_match('/(<img[^>]+>)/i', $this->output->desc)) {
         glob(PATH_TEMP .'/*.jpg') ?: $this->Error('no_images');
      }
      
      $local = '/<img\s+[^>]*src=("|\')Uploads\/([^"\']+\.(jpe?g|png|gif))/i';
      $external = '/<img\s+[^>]*src=("|\')(https?[^"\']+\.(jpe?g|png|gif))/i';

      $is_loc = preg_match_all($local, $this->output->desc, $loc);
      $is_ext = preg_match_all($external, $this->output->desc, $ext);
      $saved = preg_grep('/(.*)\/(hq_|m_)(.*)/', glob(PATH_TEMP . '/*.jpg'), PREG_GREP_INVERT);

      foreach ( $loc[2] as $file ) { $inText[] = basename($file); }
      foreach ( $ext[2] as $file ) { $inText[] = hash('crc32b', $file) . '.jpg'; }
      foreach ( $saved as $file ) { 
         $base = basename($file); 
         if ($base == "cover.jpg") { $this->cover = true; }
         else { $onDrive[] = $base; }
      }

      $this->remove = array_diff((array) $onDrive, (array) $inText);

      if ($is_ext) { foreach ($ext[2] as $tmp) {

         $hash = hash('crc32b', $tmp);
         $path = USER_TEMP .'/'. $hash . '.jpg';
         
         if (!file_exists(PATH_TEMP .'/'. $hash . '.jpg')) {
            if ($info = getimagesize($tmp)) {

               $img = new Image($tmp); 
               if ($info[0] > 700) $img->resize(700, null);

               $img->toFile(PATH_TEMP .'/'. $hash . '.jpg', 'image/jpeg');
               $img->resize(350, null)->toFile(PATH_TEMP .'/m_'. $hash . '.jpg', 'image/jpeg');
               $this->output->desc = str_replace($tmp, $path, $this->output->desc);
               
            }
         } else {
            $this->output->desc = str_replace($tmp, $path, $this->output->desc);
         }
         
      }}
      
   }
   
   protected function prepareText() {

      $rawText = strip_tags($this->raw->desc);

      if ( count(mb_split('\s', $rawText)) < App::get('art_min_words') ) { $this->Error('desc_length'); }
      if ( strlen($rawText) < 100) { $this->Error('desc_length'); }

      $tags  = 'iframe,blockquote,strong,ul,ol,li,a,p,span,hr,u,i,b';
      $tags .= 'img,pre,h1,h2,h3,h4,h5,h6,em,br,table,tr,td,tbody,div';
      $toRemove = '<' . implode('><', explode(',', $tags)) . '>';
      
      $this->output->desc = $this->stripContent($this->raw->desc, $toRemove);
      
   }
   
   protected function stripContent($text, $tags = '', $invert = FALSE) {

      $text = preg_replace('/<div(.*?)>(.*?)<\/div>/is','$2',$text);
      preg_match_all('/<(.+?)[\s]*\/?[\s]*>/si', trim($tags), $tags); $tags = array_unique($tags[1]);

      if (is_array($tags) AND count($tags) > 0) { 
         if ($invert == FALSE) { 
            return preg_replace('@<(?!(?:'. implode('|', $tags) .')\b)(\w+)\b.*?>.*?</\1>@si', '', $text); 
         }
         else { return preg_replace('@<('. implode('|', $tags) .')\b.*?>.*?</\1>@si', '', $text); }
      } elseif ($invert == FALSE) { return preg_replace('@<(\w+)\b.*?>.*?</\1>@si', '', $text); }

      return $text;

   }
   
   
}