<?php

class AddVideo extends Submit {
   
   private $video;
   
   public function __construct($_type) {
      parent::__construct($_type);
      $this->json = $this->getJSON();
      $this->video = $this->prepareData();
      $this->submit();
   }
   
   private function getJSON() {
      $path = PATH_TEMP . '/mediadata.json';
      if (file_exists($path)) {
         $json = file_get_contents($path);
         return json_decode($json);
      } else { return NULL; }
   }
   
   private function prepareData() {

      if ( !$this->post->rawgif ) { // Video form
         
         $vars = (object) $this->validate([
            'id' => $this->json->id,
            'from' => $this->post->from,
            'source' => $this->post->source,
            'hash' => $this->post->hash,
            'file' => $this->json->file,
            'width' => (int) ($this->json->width ?: $this->post->width),
            'height' => (int) ($this->json->height ?: $this->post->height),
            'external' => $this->son->external,
            'embed_url' => $this->json->embed_url ?: null
         ]);
         
      } else { // Gif form
         
         $headers = $this->web->Request($this->post->external, ['head' => true]);
         if ($headers['content-type'] != 'image/gif') exit;

         $vars = (object) [
            'from' => 'gifurl',
            'hash' => $this->post->hash,
            'width' => (int) $this->post->width,
            'height' => (int) $this->post->height,
            'gifsize' => (int) $this->post->size
         ];
         
      }
      
      $array = ['youtube','facebook','vk','twitter','cda','vevo','soundcloud','coub',
                'dailymotion','liveleak','streamable','vimeo','metacafe','openload','mixcloud'];
      
      $this->mode = in_array($vars->source, $array) ? 'embed' : 'local';
      return (object) array_filter( (array) $vars );
      
   }
   
   private function validate($vars) {
      if ( !preg_match('/[a-z0-9]/', $vars['hash'])
        or !preg_match('/[a-z0-9]/', $vars['source'])
        or !in_array($vars['from'], ['url','downloaded','uploaded'])
      ) { exit; } else { return $vars; }
   }
   
   private function submit() {

      $field = $this->output;
      $this->countTags($field->tags);

      $postID = $this->sql->insert('posts', [
         'tytul' => $field->title, 'type' => $field->type,
         'czas' => $field->time, 'user' => $field->user, 
         'opis' => isset($field->desc) ? $field->desc : '', 
         'tagi' => isset($field->tags) ? $field->tags : '',
         'category' => isset($field->category) ? $field->category : '', 
         'logged' => $field->logged, 'status' => $field->status
      ]);

      if ($postID) {

         if ($this->post->rawgif) {
            $this->saveGif(); 
            $this->saveThumbs();
         }

         $this->addTags($field->tags);
         $dir = $this->makeFolder($postID);
         $this->moveFiles($dir->save);

         if ($field->logged) {
            $this->note->addNote('', $field->user, 'post', '', $postID , json_encode(['post_type' => $field->type]));
         }

         $json = array_merge((array) $this->json, (array) $this->video);
         if ($this->post->rawgif) { unset($json["file"]); }
         unset($json["image"],$json["binary"]); $json = json_encode($json);
         
         // Saving output JSON file
         file_put_contents("{$dir->save}/mediadata.json", $json);
         chmod_u("{$dir->save}/mediadata.json", 0666);

         $this->sql->where('id', $postID)->update('posts', [
            'mode' => $this->mode, 'dir' => $dir->date,
            'mediadata' => $json, 'hash' => $this->video->hash
         ]); $this->clearFolder(); sleep(1); echo 1;

      } else { $this->Error('submit_error'); }

   }
   
   private function saveGif() {
      $path = PATH_TEMP . '/' . $this->post->hash . '.gif';
      $this->web->Request($this->post->external, null, true)->SaveToFile($path);
   }
   
   private function saveThumbs() {
      $path = PATH_TEMP . '/' . $this->post->hash;
      $img = new Image($path . '.gif');
      $img->toFile($path . '.jpg', 'image/jpeg');
      $img->resize(350, null)->toFile(PATH_TEMP . '/m_' . $this->post->hash .'.jpg', 'image/jpeg');
   }
   
}