<?php

class Comments extends Validator {
   
   use FormLock;
   
   private $sql;
   private $row;
   private $post;
   private $user;
   private $time;
   private $notes;

   public function __construct() {
      $this->sql   = Factory::get('Database');
      $this->user  = Factory::get('User')->getData();
      $this->notes = Factory::get('Notes');
      $this->setLocalVars();
   }
   
   private function setLocalVars() {
      $this->time = microtime(true);
      $this->raw  = (object) $_POST;
      $this->post = App::array2obj($_POST);
   }
   
   private function VoteInfo($arr, $id) {
      return $this->sql
         ->where("pid=? and item='comment' and {$arr[0]}=?", [$id, $arr[1]])
         ->getOne('user_stats','action, time');
   }
   
   public function VoteComment() {
      
      $user = $this->user;
      $post = $this->post;
      $nick = $user->username;
      
      if ($post->commid && in_array($post->vote, ['up','down'])) {

         $arr = []; $post->id = (int) $post->commid;
         array_push($arr, ($nick ? 'username' : 'userhash'), ($nick ?: $user->iphash));
         
         $vote = $this->VoteInfo($arr, $post->id);
         $case = !$vote['action'] ? 'insert' : (
            $vote['action'] != $post->vote ? 'update' : 'delete'
         );
         
         if ($case == 'insert') {

            $this->sql->insert('user_stats', [
               'pid' => $post->id, 'date' => $this->sql->now(), 'item' => 'comment', 
               'action' => $post->vote, $arr[0] => $arr[1], 'time' => $this->time
            ]);

            $this->sql->where('id', $post->id)->update('comments', ["vote_{$post->vote}" => $this->sql->inc(1)]);
            $cData = $this->sql->where('id', $post->id)->getOne('comments', 'wpis, user, logged');
            
            if ($cData['logged'] && $user->username != $cData['user']) {
               $cmArr = array(
                  'comment_id' => $post->id, 'parent_id' => $cData['wpis'],
                  'created' => $this->time, 'vote' => $post->vote);
               $this->notes->commentVote('addvote', $cData['user'], App::array2obj($cmArr));
            }

         }
         
         elseif ($case == 'update') {
         
            if ($post->vote == 'up') {
               $this->sql->where('vote_down','0','>')->where('id', $post->id)
                  ->update('comments', ['vote_up' => $this->sql->inc(1), 'vote_down' => $this->sql->dec(1)]);
            } else {
               $this->sql->where('vote_up','0','>')->where('id',$post->id)
                  ->update('comments', ['vote_up' => $this->sql->dec(1), 'vote_down' => $this->sql->inc(1)]);
            }

            $cmArr = ['comment_id' => $_post->id, 'vote_type' => $_post->vote];
            $this->notes->commentVote('updatevote', $vote['time'], App::array2obj($cmArr, true));
            $this->sql->where("pid=? and item='comment' and {$arr[0]}=?",[$post->id, $arr[1]])
               ->update('user_stats',['action' => $post->vote]);
         }
         
         elseif ($case == 'delete') {
            $this->notes->commentVote('removevote' , $vote['time']);
            $this->sql->where("pid=? and action=? and item='comment' and {$arr[0]}=?",[$post->id, $post->vote,$arr[1]])->delete('user_stats');
            $this->sql->where("id=? and vote_{$post->vote} > 0",[$post->id])->update('comments', ["vote_{$post->vote}" => $this->sql->dec(1)]);
         }

         $data = $this->sql->where('id=?',[$post->id])->getOne('comments','vote_up,vote_down');
         echo json_encode($data);

      }
   }

   public function ShowReplies() {

      usleep(500000);
      
      $user = $this->user;
      $id = (int) $this->post->id;

      $data = $this->sql->ObjectBuilder()->rawQuery("
         SELECT * FROM comments
         WHERE reply_id='{$id}' AND status='1'
         ORDER BY id DESC LIMIT 1,18446744073709551615
      ");

      $out = '';
      $this->PrepareComments($data);
      $tpl = new Template('comment_reply');

      foreach ($data as $post) {
         $post->nick = $post->nick;
         $post->avatar = $post->avatar;
         $post->comment = $post->tresc;
         $post->postid = $post->wpis;
         $post->name = App::fakeName($post->nick);
         $out .= $tpl->render( compact($tpl->getTags()) ); 
      }

      echo $out;
      
  }

  public function ShowComments() {
      $id = (int) $this->post->id;
      $this->PostComments($id, 'post', 10);
  }
   
   public function DeleteComment($id = null, $rep = null, $pid = null) {
      
       $iphash = $this->user->iphash;
       $commid = $id ?: (int) $this->post->commid;
       $replyid = $rep ?: (int) $this->post->replid;
       $postid = $pid ?: (int) $this->post->postid;

       // Only admin can delete comment with replies
       if ($this->user->isadmin) { $query = "DELETE FROM comments WHERE id='{$commid}'"; $repliesQuery = "DELETE FROM comments WHERE reply_id='{$commid}'"; }
       elseif ($this->user->logged) { $query = "DELETE FROM comments WHERE id='{$commid}' AND ( (logged='1' AND user='{$this->user->username}') OR iphash='{$iphash}')"; }
       else { $query = "DELETE FROM comments WHERE id='{$commid}' AND replies='0' AND logged='0' AND iphash='{$iphash}'"; }

       // Removing notification
       if ($replyid) { $data  = $this->sql->ObjectBuilder()->where('id', $replyid)->getOne('comments', 'logged'); }
       else { $data  = $this->sql->ObjectBuilder()->where('id', $postid)->getOne('posts', 'logged'); }

       if ($data->logged) {
         $time = $this->sql->ObjectBuilder()->rawQueryOne(str_replace('DELETE ', 'SELECT data ', $query));
         $this->notes->removeNote('pscm', $time->data); 
          if ($replyid) { $this->notes->removeNote('cmre', $time->data); }
       }

       // Removing comment and replies
       $this->sql->rawQuery($query);
       if (isset($repliesQuery)) { $this->sql->rawQuery($repliesQuery); }

       // Decrementing counts for post parent comment
       $this->sql->where('comments', '0', '>')->where('id' ,$postid)->update('posts', ['comments' => $this->sql->dec(1)]);
       if ($replyid) { $this->sql->where('replies', '0', '>')->where('id', $replyid)->update('comments', ['replies' => $this->sql->dec(1)]); }
          
       
      if (!$id) {echo 1; } else { return 'success'; }

   }
   
   private function getStatus() {
      return User::Logged() ? 1 : 0;
   }

   public function AddComment() {

      usleep(500000); 
      
      User::can('comment') ?: die( App::printJSON('error', 
          Lang::get($this->user->logged ? 'commenting_disabled' : 'log2comment')
      ));
      
      $post = $this->post; 
      $user = $this->user;
      $status = $this->getStatus();
      $validTags = '<blockquote><strong><em><span><a><b><i><p><img><br><div>';

      $post->nick = $user->username ?: $post->username;
      $post->parent = isset($post->parent) ? $post->parent : 0;
      $post->comment = strip_tags($this->raw->comment, $validTags);

      if ($this->Validate($post->nick, $post->comment)) {

         $delay = (int) App::get('comm_delay');
         $this->FormDelay('_cmt_dl_','err_comm_delay', $delay, true);

         $post->menu = $user->logged;
         $post->vote_up = $post->vote_down = 0;
         $post->name   = App::fakeName($post->nick);
         $post->avatar = App::getAvatar($user->username);
         $comment_type = $post->item === "photo" ? "photo" : "post";

         $post->id = $this->sql->insert('comments', [
            'wpis' => (int) $post->postid, 'user' => $post->nick, 'tresc' => $post->comment,
            'iphash' => $user->iphash, 'reply_id' => (int) $post->parent, 'data' => $this->time,
            'status' => $status, 'logged' => (int) $user->logged, 'type' => $comment_type, 'hash' => $user->hash,
         ]);

         echo $this->sql->getLastError();

         if ($user->logged) {

            if ($post->parent) { $this->sql->where('id', $post->parent)->update('comments', ['replies' => $this->sql->inc(1)]); }
            if ($comment_type == "post") { $this->sql->where('id', $post->postid)->update('posts', ['comments' => $this->sql->inc(1)]); }

            $post->time = $this->time;
            $this->notes->commentNote($post);

         } else { Cookie::set("username", $post->nick); }
         
         $message = $status ? 'comment_submitted' : 'comment_moderation';
         
         $tpl = new Template('comment_reply');
         $resp['html'] = $tpl->render( compact($tpl->getTags()) ); 
         $resp['status'] = 'success';
         $resp['message'] = Lang::get($message);

         echo App::printJSON($resp); exit;

      }

   }
   
   public function EditComment() {
      
      if ($this->post->cid && $this->post->content) {

         $id = (int) str_replace('cm_', '', $this->post->cid);
         $content = strip_tags($this->raw->content, '<blockquote><strong><em><span><a><b><i><p><img><br><div>');
         if (mb_strlen($content) < (int) App::get('comm_minlen') || empty($content)) { 
            die(App::printJSON('error', Lang::get('comm_minlen')));
         }

         if ($this->user->isadmin) {
            $this->sql->where('id', $id)->update('comments', ['tresc' => $content]);
            die(App::printJSON('success', Lang::get('changes_saved')));
         }
         
         $status = $this->user->logged
           ? $this->sql->where('id=? and logged=1 and user=?', [$id, $this->user->username])->update('comments',['tresc' => $content])
           : $this->sql->where('id', $id)->where('iphash', $this->user->iphash)->update('comments', ['tresc' => $content]);

         die(App::printJSON('success', Lang::get('changes_saved')));

      }

   }

  public function ListComments() {
     
     $id = (int) $this->post->id;
     $type = $this->post->item;
     $user = $this->user;

     $results = $this->sql->ObjectBuilder()->rawQuery("
        SELECT @cid := id, id, user, data, logged, tresc, (SELECT count(*) FROM comments where reply_id = @cid) AS count 
        FROM comments WHERE wpis=? AND type=? AND reply_id = 0 AND status = 1 ORDER BY data DESC
     ", [$id,$type]);
     
     if ($this->sql->count) {

        array_map(function($elem) use (&$user) {
            $elem->name = App::fakeName($elem->user);
            $elem->data = App::zmienczas($elem->data);
            $elem->purl = !$user->logged ? $elem->name
               : '<a class="comment-author" href="profile'.(SR . $elem->user).'">'.($elem->name).'</a>';
            $elem->avatar = $elem->logged ? App::getAvatar($elem->user) : App::getAvatar();
         }, $results);
        
        $tpl = new Template('loop_comments_list');
        echo $tpl->render( compact( $tpl->getTags() ) );

     }
     
  }

  public function PostComments($id, $type, $start = 0, $limit = 10) {

     $data = []; $user = $this->user;
     $base = new Template('base_comments');
     $loop = new Template('loop_comment_single');

     $d = $this->sql->rawQuery("
         SELECT * FROM comments
         WHERE wpis='{$id}' AND type='{$type}' AND status='1' and reply_id = 0
         ORDER BY id DESC LIMIT $start, $limit");

      foreach ($d as $i => $comment) {

         array_push($data, $comment);
         $rep = $this->sql->rawQuery("
            SELECT * FROM comments
            WHERE reply_id = {$comment["id"]} AND status='1'
            ORDER BY id desc LIMIT 1
         ");
         
         foreach ($rep as $i => $reply) {
            if ($comment["replies"] > 1) { $reply["last"] = $comment["replies"]; }
            array_push($data, $reply);
         }

      }

      $data = json_decode(json_encode($data));

     if ( !empty($data) ) {
        $results = $this->PrepareComments($data);
        $output  = $loop->render( compact($loop->getTags()) );
        if ($start > 0) die($output);
     } else {
        $output = ($type == 'photo') ? 'no_comments' : '<div class="no_comments">'.Lang::get('no_comments').'</div>';
     }

     $base->extend('comments', $output);
     if ($user->logged === false) {
        $nick = Cookie::get('username') ?: ''; $disabled = empty($nick) ? '' : 'disabled';
        $base->extend('nickfield', '<input name="username" value="'.$nick.'" class="textbox margin-bottom-10" type="text" placeholder="'.Lang::get('comment_nick_placeholder').'" '.$disabled.'>');
     }
     return $base->render( compact($base->getTags()) );

  }
   
   private function PrepareComments($results) {

      $user = $this->user;

      array_map(function($elem) use (&$user) {

         $buttons = '';
         $elem = $this->ExtendObject($elem, [
            'nick' => App::fakeName($elem->user),
            'data' => App::zmienczas($elem->data),
            'menu' => $user->logged && $elem->logged,
            'tresc' => strip_tags($elem->tresc, '<blockquote><strong><em><span><a><b><i><p><img><br>'),
            'avatar' => $elem->logged ? App::getAvatar($elem->user) : App::get('app_url') . '/Uploads/Avatars/default-avatar.png'
         ]);

         $author = $user->isadmin || ($elem->iphash == $user->iphash) || ($elem->logged && ($elem->user == $user->username));

         $elem = $this->ExtendObject($elem, [
            'author' => $author || $guest ? true : false, 
            'editable' => $author || $guest ? 'canedit' : ''
         ]);

         if ($elem->author) { // Is author or admin

            // Edit button: Logged users or admin
            if ($user->logged) { $buttons .= '<a class="edit"><i class="icon-edit"></i> <!Lang::comment_edit></a>'; }

            // Remove button: No replies or admin
            if ($user->isadmin || ($author && !$elem->replies)) { 
               $buttons  .= '<a class="remove" data-replid="'.$elem->reply_id.'"><i class="icon-cancel-circle"></i> <!Lang::comment_remove></a>'; 
            }

         } else {

            // Flag button: Not author 
            $elem->flag = '<a class="flag" data-type="comment" data-id="'.$elem->id.'"><i class="icon-warning"></i> <!Lang::flag></a>';

         }

         $elem = $this->ExtendObject($elem, 'author_icons', $buttons);
         $elem->isreply = ($elem->reply_id) ? 'reply' : '';
         $elem->data_reply_id = ($elem->reply_id) ? $elem->reply_id : $elem->id;

      }, $results);
      
      return $results;
      
   }
   
   private function Validate($nick, $comm) {
      
      $this->isValid('username', $nick);
      $this->isValid('comment', [$comm, $nick]);
      
      if (count($this->error) === 0) { return true; }
      else { die( App::printJSON('error', $this->getLastError()) ); }
      
   }

   private function ExtendObject($object, $key, $value = '') {
      if (is_array($key)) foreach ($key as $k => $v) $object->$k = $v;
      else $object->$key = $value;
      return $object;
   }

} // END Class