<?php

require_once 'MysqliDb.class.php';

Class Database Extends MysqliDB {
   
   private $instance;
   
   public function __construct($config) {
      $this->instance = $this;
      parent::__construct($config);
   }
   
   /**
   * Counting table rows based on where condition
   * @param  String $table     Table name in database.
   * @param  String $condition User-provided query to execute.
   * @param  String or Array $vars Single variable or variables array to bind to the SQL statement.
   */

    public function countRows($table, $condition = null, $vars = null) {
       
       $pref = "SELECT count(*) FROM $table";
       
        if ($condition and $vars) {
            
           if (gettype($vars) != 'array') {
               $params[] = $vars; 
            } else { $params = $vars; }

            return $this->rawQueryValue("$pref WHERE $condition LIMIT 1", $params); 
           
        } elseif ($condition) { return $this->rawQueryValue("$pref WHERE $condition LIMIT 1");
        } else { return $this->rawQueryValue("$pref LIMIT 1"); }
        
    }
   
   private function is_str($str) {
      return !empty($str) && preg_match('/^[\p{L}A-Za-z0-9\s-]+$/iu', $str) ? $str : false;
   }
   
   private function is_num($num) {
      return !empty($num) && is_integer($num) ? $num : false;
   }
   
   public function prepareQuery($_data, $rows = true, $fields = '*') {
      
      if (empty($_data)) return;
      
      $_postPage = basename($_SERVER['SCRIPT_FILENAME']) === 'post.php';

      $_params = array(); 
      $_sortResult = false;
      $_data = gettype($_data) == 'array' 
         ? (object) $_data : clone $_data;
      
      // Integers && booleans
      $_data->similar = isset($_data->similar) ? true : false;
      $_data->page = isset($_data->page) ? intval($_data->page) : 0;
      $_data->limit = isset($_data->limit) ? intval($_data->limit) : 8;
      $_data->status = isset($_data->status) ? intval($_data->status) : 2;
      $_data->noid = isset($_data->noid) ? $this->is_num($_data->noid) : false;
      
      // Strings
      $_data->type = isset($_data->type) ? $this->is_str($_data->type) : false;
      $_data->tagi = isset($_data->tagi) ? $this->is_str($_data->tagi) : false;
      $_data->user = isset($_data->user) ? $this->is_str($_data->user) : false;
      $_data->sort = isset($_data->sort) ? $this->is_str($_data->sort) : false;
      $_data->search = isset($_data->search) ? $this->is_str($_data->search) : false;
      $_data->subsort = isset($_data->subsort) ? $this->is_str($_data->subsort) : false;
      $_data->category = isset($_data->category) ? $this->is_str($_data->category) : false;
      $_limit = 'LIMIT '. ($_data->page * $_data->limit) .','. $_data->limit;
      
      // Setting posts type
      switch ($_data->type) {
         case 'gif': $postType = 'gif'; break;
         case 'videos': $postType = 'video'; break;
         case 'audio': $postType = 'audio'; break;
         case 'photos': $postType = 'photos'; break;
         case 'articles': $postType = 'article'; break;
         default: $postType = null;
      } 
      
      $_params[] = "status='{$_data->status}'";
      $postType && $_params[] = "type='{$postType}'";
      
        /*** Ustawiamy parametry WHERE ***/
        $_postPage && $_data->noid = intval($_GET['id']);
        $_data->user && $_params[] = "user='{$_data->user}'";
        $_data->noid && $_params[] = "id != {$_data->noid}";
        $_data->tagi && $_params[] = "tagi LIKE '%". remLine($_data->tagi) ."%'";
        $_data->category && $_params[] = "category = '". remLine($_data->category) ."'";

        if ($_data->sort && !$_postPage) {

           switch ($_data->subsort) {
              case 'likes': $_subSort = 'likes'; break;
              case 'fav': $_subSort = 'fav'; break;
              default: $_subSort = 'views';
           }

           switch ($_data->sort) {
              case 'today': $_sortResult = $this->popular("DAY", $postType, $_subSort); break;
              case 'week':  $_sortResult = $this->popular("WEEK", $postType, $_subSort); break;
              case 'month': $_sortResult = $this->popular("MONTH", $postType, $_subSort); break;
              case 'year':  $_sortResult = $this->popular("YEAR", $postType, $_subSort); break;
              case 'thisweek':  $_sortResult = $this->sqlrange('thisweek', $postType, $_subSort); break;
              case 'lastweek':  $_sortResult = $this->sqlrange('lastweek', $postType, $_subSort); break;
              case 'thismonth': $_sortResult = $this->sqlrange('thismonth', $postType, $_subSort); break;
              case 'lastmonth': $_sortResult = $this->sqlrange('lastmonth', $postType, $_subSort); break;
              case 'fav': $_sortResult = $this->popular(null, $postType, $_data->sort); break;
              case 'likes': $_sortResult = $this->popular(null, $postType, $_data->sort); break;
           }
           
        }
      
         if ($_data->search) { $_sortResult = "MATCH (tytul,opis,tagi,category) AGAINST ('{$_data->search}')"; }
         if ($_data->similar) { $_sortResult = "MATCH (tytul,opis,tagi,category) AGAINST ('{$_data->title}')"; }
         if (isset($_data->following)) { $_params[] = implode(' or ', $_data->following); }
         
         if ($_sortResult) { $_params[] = $_sortResult; $_order = ''; } else { 

            if ($_data->sort) { switch ($_data->sort) {
                case 'random': $_order = 'rand()'; break;
                case 'oldest': $_order = 'id'; break;
                default: $_order = 'id desc';
            }} else { $_order = 'id desc'; }
            
            $_order = isset($_order) ? 'ORDER BY '. $_order : '';

         }

         if (is_array($fields)) {
            $fields = implode(',', $fields);
         }

         $_params && $_cond = 'WHERE ' . implode(' AND ', $_params);
         $_rows = $this->rawQueryValue("SELECT count(*) FROM posts $_cond"); 
         $_query = "SELECT {$fields} FROM posts $_cond $_order $_limit";

         return $rows ? [
             'query' => $_query,
             'rows' => array_shift($_rows)
         ] : $_query;

   }
   
   function sqlrange($opt,$postType=false,$sortType='views'){

      $sql = $this->instance;
      $add = 'and item="post"';
      
      if ($postType){ $add .= " and item_type='$postType'"; }

      if ( $opt == 'thisweek' ){ $data = $sql->rawQuery("select pid,SUM($sortType) as suma from daily_stats where $sortType>0 AND date >= CURDATE() - INTERVAL DAYOFWEEK(CURDATE())-1 DAY $add GROUP BY pid ORDER BY suma desc,pid"); }
      if ( $opt == 'lastweek' ){ $data = $sql->rawQuery("select pid,SUM($sortType) as suma from daily_stats where $sortType>0 AND (date > DATE_SUB(DATE(NOW()), INTERVAL DAYOFWEEK(NOW())+6 DAY) AND date <= DATE_SUB(DATE(NOW()), INTERVAL DAYOFWEEK(NOW())-1 DAY)) $add GROUP BY pid ORDER BY suma desc,pid"); }
      if ( $opt == 'thismonth' ){ $data = $sql->rawQuery("select pid,SUM($sortType) as suma from daily_stats where $sortType>0 AND date >= CURDATE() - INTERVAL DAYOFMONTH(CURDATE())-1 DAY  $add GROUP BY pid ORDER BY suma desc,pid"); }
      if ( $opt == 'lastmonth' ){ $data = $sql->rawQuery("select pid,SUM($sortType) as suma from daily_stats where $sortType>0 AND date >= (CURDATE() - INTERVAL DAYOFMONTH(CURDATE())-1 DAY) - INTERVAL 1 MONTH and date < (CURDATE() - INTERVAL DAYOFMONTH(CURDATE())-1 DAY) $add GROUP BY pid ORDER BY suma desc,pid"); }

      if ($sql->count) {
         foreach ($data as $result) { $tab[] = $result['pid']; }
         return 'id IN ('.implode(',',$tab).') ORDER BY FIELD(id, '.implode(',',$tab).')'; 
      } else{ return false; }

   }
   
   function popular($range, $postType=false, $sortType='views') {

      $sql = $this->instance; $add = '';
      if ($postType){ $add = "and item_type='$postType'"; }

      if ($range) { $data = $sql->rawQuery("select pid, SUM($sortType) as suma from daily_stats where $sortType > 0 AND date BETWEEN (CURRENT_DATE() - INTERVAL 1 $range) AND CURRENT_DATE() AND item='post' $add GROUP BY pid order by suma desc,pid"); }
      else { $data = $sql->rawQuery("select pid,SUM($sortType) as suma from daily_stats where item='post' $add GROUP BY pid order by suma desc,pid"); }

      if ($sql->count) {
         foreach( $data as $result ){ $tab[] = $result['pid']; } 
         return 'id IN ('.implode(',',$tab).') ORDER BY FIELD(id, '.implode(',',$tab).')'; 
      } else { return false; }

   }
   
}