<?php

class INI {
   
   private static $root;
   private static $file;
   private static $backup;
   
   private static $config = [];
   
   private static function read() {
      
      self::$root   = __DIR__ . "/../../Libs";
      self::$file   = __DIR__ . "/../settings.ini";
      self::$backup = __DIR__ . "/Resources/settings.ini";
      
      if (empty(self::$config)) {
         
         // Restore config file if missing
         file_exists(self::$file) ?: self::restore();
         self::$config = parse_ini_file(self::$file, true);
         
         (bool) self::get('restore') ?: self::restore();
         (bool) self::get('disqus_id') ?: self::fixMissing();

      }
      
      return self::$config;
      
   }
   
   // Get all settings as key => value
   public static function getConfig($asObject = false) {

      $arr = array_reduce(self::read(), function($hand, $item) {
         foreach ($item as $key => $value) { $hand[$key] = $value; }
         return $hand; 
      }); 
      
      return $asObject ? (object) $arr : $arr;
      
   }
   
   // Get single value from config
   public static function get($key) {
      $section = self::sectByKey($key);
      return @self::$config[$section][$key];
   }

   public static function remove($key) {
      self::read();
         $section = self::sectByKey($key) ?: 'custom'; 
         unset(self::$config[$section][$key]);
      self::save();
   }
   
   // Set or change single value
   public static function set($key, $value = null, $loop = false) { 
      
      self::read();
      
      if (is_array($key)) { 
         foreach ($key as $k => $v) { self::set($k, $v, true); }
         return self::save(); 
      } else { 
         if (!empty($key)) {
            $section = self::sectByKey($key) ?: 'custom'; 
            self::$config[$section][$key] = $value; 
            if (!$loop) { self::save(); }
         }
      }
      
   }
   
   public static function save() {
      
      $output = []; foreach (self::read() as $section => $content) {
         $content = implode("\n", array_map(function($value, $key) {
            return "$key = " . (is_numeric($value) ? $value : '"'.$value.'"');
         }, array_values($content), array_keys($content)));
         $output[] = "[$section]\n$content";
      }
      
      file_put_contents(self::$file, implode("\n\n", $output), LOCK_EX);
      self::$config = []; return true;

   }
   
   // Get section name by key
   private static function sectByKey($key) {
      foreach(self::read() as $section => $array) {
         if (array_key_exists($key, $array)) { return $section; }
      }
   }
   
   private static function fixMissing() {
      if (file_exists(self::$file) && file_exists(self::$backup)) {
         $current = parse_ini_file(self::$file, false);
         $backup = parse_ini_file(self::$backup, false);
         $data = array_diff_assoc($backup, $current);
         !empty($data) && self::set($data);
      }
   }
   
   private static function restoreValues() {
      if ( $sql = Factory::get('Database') ) {
         $data = $sql->getOne('config');
         $data["restore"] = 1; array_shift($data); 
         self::set($data);
      }
   }
   
   private static function setPermissions($dir) {
      $oldmask = umask(0);
         is_writable($dir) ?: @chmod($dir, 0755);
         is_writable($dir) ?: @chmod($dir, 0777);
      umask($oldmask);
   }
   
   private static function restore() {
      self::setPermissions(self::$root);
      @copy(self::$backup, self::$file);
      self::setPermissions(self::$file);
      self::restoreValues();
   }
   
}