<?php

class Layout extends AppController {

   // Components namespace
   const PREFIX = 'Components';

   public function __construct() {
      parent::__construct();
   }

   public function get($components) {

      if (is_string($components)) {

         $params = func_get_args();
         $module = $this->_parse($params);
         $class = $module->controller;

         if (class_exists($class) && method_exists($class, $module->action)) {
            $reflectionMethod = new ReflectionMethod($class, $module->action);
            return $reflectionMethod->invokeArgs(new $class(), $module->params);
         }

         throw new LayoutException('Layout not found: ' . $class .'\\'. $module->action);

         return false;

      } elseif (is_array($components)) {

         $output = []; 

         foreach ($components as $component => $params) {
            array_unshift($params, $component);
            $name = $this->_getAlias($params);
            $output[ $name ] = call_user_func_array(['Layout','get'], $params);
         }

         return $this->_toObject($output);

      }

   } // END get

   // ------------------------------------------------
   // -------------- PRIVATE METHODS -----------------
   // ------------------------------------------------

   private function _getAlias(&$params) {
      $split = explode(' => ', $params[0]); $params[0] = $split[0];
      return $split[1] ? $split[1] : preg_replace('~([^/]+/)~','',$split[0]);
   }

   private function _toObject($array) {
      return json_decode(json_encode($array));
   }

   private function _parse($params) {
      $base = explode('\\', array_shift($params));
      return (object) [
         'controller' => self::PREFIX .'\\'. array_shift($base),
         'action' => array_shift($base), 'params' => $params ?: []
      ];
   }

}

class LayoutException extends Exception {

   public function __construct($message, $code = 0, Exception $previous = null) {
      parent::__construct($message, $code, $previous);
  }

}