<?php

Class Notes {

   protected $sql;
   protected $conf;
   protected $config;
   protected $follows;

   public function __construct() {
      
      $this->sql  = Factory::get('Database');
      $this->user = Factory::get('User')->getData();
      $this->extract( $this->user );
      $this->config = $this->getConfig();
      $this->follows = $this->getFollow();
      
   }

   // ======================================================================= //
   // ===========================  Database Methods  ======================== //
   // ======================================================================= //

   private function extract( $inst ) {
      $this->ntconf = (object) $_POST;
      $vars = array('logged', 'username');
      array_walk($inst, function($val, $key) use (&$vars) {
         if (in_array($key, $vars)) $this->$key = $val;
      });
   }
   
   public function addNote($to, $by, $type, $created = '', $parent = '', $params = '') {
      $this->sql->insert('notes', [
         'send_to' => $to, 'send_by' => $by, 'note_type' => $type, 'params' => $params,
         'created' => $created ?: microtime(true), 'logged' => $this->logged, 'parent' => $parent
      ]);
   }

   public function removeNote($type, $created) {
      $this->sql->where('note_type', $type)->where('created', $created)->delete('notes');
   }

   private function getRowsById($table, $rows, $id) {
      return $this->sql->ObjectBuilder()->where('id', $id)->getOne($table, $rows);
   }

   public function setSeen() {
      $this->sql->where('send_to', $this->username)->update('notes', ['seen' => 1]);
   }

   public function followNote($action, $who) {
      if ($action == 'follow') $this->addNote($who, $this->username, 'usfl');
      else $this->sql->where('note_type', 'usfl')->where('send_by', $this->username)->where('send_to', $who)->delete('notes');
   }

   public function commentVote($action, $option, $params = '') {
      if ($action == 'addvote') {
         $prm = json_encode(['comment_id' => $params->comment_id, 'vote_type' => $params->vote]);
         $this->addNote($option, $this->username, 'cmvt', $params->created, $params->parent_id, $prm); 
      } elseif ($action == 'updatevote') {
         $this->sql->where('note_type', 'cmvt')->where('created', $option)->update('notes', ['params' => $params]);
      } elseif ($action == 'removevote') {
         $this->removeNote('cmvt', $option);
      }
   }

   private function idFromPhoto($id) {
      return $this->sql->where('id', $id)->getValue('photos', 'pid');
   }

   public function postVoting($post, $action = '', $time = '') {

      $paramsArray = array('post_type' => $post->type, 'vote_type' => $post->vote);

      switch ($post->item) {
         case 'post' : $sqlData = $this->getRowsById('posts','user,logged',$post->id); break;
         case 'photo': $paramsArray['photo_id'] = $post->id; 
            $sqlData = $this->getRowsById('posts','user,logged', $this->idFromPhoto($post->id));
      }

      $params = json_encode(App::array2obj($paramsArray));
      $Label = $post->vote == 'fav' ? ($post->item == 'post' ? 'psfv' : 'phfv')
                                    : ($post->item == 'post' ? 'psvt' : 'phvt');

      if ($sqlData->logged && $sqlData->user != $this->username) {
         if ($action == 'update') {
            
           $this->sql->where('note_type',$Label)
              ->where('created',$time)
              ->update('notes',['params' => $params]);
           
         } else { 
            $addID = ($post->item == 'photo') ? $this->idFromPhoto($post->id) : $post->id;
            $this->addNote($sqlData->user , $this->username , $Label , $post->time , $addID , $params); 
         }
      }
   }

   public function commentNote($post) {

      $paramsArray = array(
         'comment_id' => $post->id,
         'post_type'  => $post->item
      );
      
      if ($post->item == 'photo') {
         $paramsArray['photo_id'] = $post->postid;
         $post->postid = $this->idFromPhoto($post->postid);
      }

      if (!empty($post->parent)) { 
         $paramsArray['reply_id'] = $post->parent;
         $topCom = $this->getRowsById('comments', 'user,logged', $post->parent);
      }
      
      $label = ($post->item == 'post') ? 'pscm' : 'phcm';
      $sqlData = $this->getRowsById('posts', 'user,logged', $post->postid);
      $params = json_encode( App::array2obj($paramsArray) );

      if ($sqlData->logged && $sqlData->user != $this->username) {
         if (!empty($post->parent)) {
            if ($topCom->logged && $topCom->user != $sqlData->user) {
               $this->addNote($sqlData->user , $this->username , $label , $post->time , $post->postid , $params);
            }
         } else {
            $this->addNote($sqlData->user , $this->username , $label , $post->time , $post->postid , $params);
         }
      }

      if (!empty($post->parent)) {
         if ($topCom->logged && $topCom->user != $this->username) {
            $user = $this->username ?: $post->username;
            $this->addNote($topCom->user, $user, 'cmre', $post->time, $post->postid, $params);
         }
      }
   }

   // ==================================================================== //
   // =========================  Display methods  ======================== //
   // ==================================================================== //

   // ** Header notifications count ** //

   public function countNotes() {
      
      if ($this->logged) {

         $query = "send_to='{$this->username}' AND seen='0' AND logged='1' AND (({$this->config})";
            if ($this->follows) { $query .= " OR ({$this->config} AND {$this->follows})"; }
         $query .= ')';

         if ($rows = $this->sql->countRows('notes',$query)) { 
            return "<span class=\"pe-none count\">$rows</span>"; 
         }

      }

   }

   // ******************************************************* //
   // ** Method creates partial sql query for later usage ** //
   // ***************************************************** //

   public function getConfig() {
      if ($this->logged) {
         
         // If config exists return
         if (!empty($this->config)) { return $this->config; }
         
         // Otherwise fetching config from database
         $data = $this->sql->where('user', $this->username)->getOne('notes_config');
         $array = $this->parseConfig( $data );
         
         return $array ? implode(' or ', $array) : 'note_type=""';
         
      }
   }
   
   private function parseConfig( $conf ) {
      $arr = array();
      foreach ((array) $conf as $key => $val) {
         if ( !in_array($key, ['id','user']) ) {
            if ($val) {$arr[] = "note_type='$key'";}
            $this->conf[ $key ] = $val;
         }
      } return $arr;
   }

   public function makeInfo($type, $params) {

      if ($type=='post') { $info = Lang::get('published_new_post'); $icon = 'bookmarks'; }
      elseif ($type=='cmre') { $info = Lang::get('replied_your_comment'); $icon = 'reply'; }
      elseif ($type=='psfv') { $info = Lang::get('added_post_to_fav'); $icon = 'bookmarks'; }
      elseif ($type=='phfv') { $info = Lang::get('added_your_post_to_fav'); $icon = 'bookmarks'; }
      elseif ($type=='pscm') { $info = Lang::get('commented_your_post'); $icon = 'speech-bubble'; }
      elseif ($type=='phcm') { $info = Lang::get('commented_your_image'); $icon = 'speech-bubble'; }
      elseif ($type=='usfl') { $info = Lang::get('following_you'); $icon = 'eye'; }
      elseif ($type=='psvt' and $params->vote_type=='likes') { $info = Lang::get('liked_your_post'); $icon = 'like'; }
      elseif ($type=='phvt' and $params->vote_type=='likes') { $info = Lang::get('liked_your_pic'); $icon = 'like'; }
      elseif ($type=='cmvt' and $params->vote_type=='up') { $info = Lang::get('liked_your_comment'); $icon = 'like'; }

      return array('info' => $info, 'icon' => $icon);

   }

   // ** FUNKCJA POBIERA LOGINY OBSERWOWANYCH UŻYTKOWNIKÓW I ZWRACA FRAGMENT ZAPYTANIA SQL ** //

   public function getFollow() {
      
      if (isset($this->follows)) { return $this->follows; }
      
      if (isset($this->username)) {
         $data = $this->sql->where('kto', $this->username)->get('follow', null, 'kogo');
         if ($this->sql->count) {
            foreach ($data as $user) { $arr[] = "send_by='{$user['kogo']}'"; }
            return implode(' or ', $arr);
         } 
      }
      
      return false;
      
   }

   // ** GŁÓWNE ZAPYTANIE DO BAZY DANYCH ** //
  
  private function mReplace($str, $arr) {
    foreach ($arr as $k => $v) {
      $str = str_replace($k,$v,$str);
    } return $str;
  }

   public function smallNotes($mode = 'all', $limit = 50, $page = 0, $small = false) {

      $output = ''; $print = 0;
      $groups = $this->groupNotes($limit, $page, $mode);

      if ( !empty($groups) ) {switch ($mode) {
         case 'all': $group = $groups['all']['data']; break;
         case 'comments': $group = $groups['comments']['data']; break;
         case 'votes': $group = $groups['votes']['data']; break;
         case 'favorites': $group = $groups['favorites']['data'];
      }}

      if ( $mode == 'summary' && isset($groups['byid']) ) {
         
         foreach($groups['byid'] as $postId => $val) {
            
            foreach($val as $label => $num) {

               if ($label != 'data' && $label != 'thumb' && $label != 'title') {

                  $lastLetter = substr($num, -1);

                  if ($label == 'comments') {
                     $a = Lang::get('new');
                     if ($lastLetter == 1) { $b = Lang::get('comment'); }
                     else { $b = Lang::get('comments'); }
                     $ciag .= "$num $a $b, "; $print++;
                  }
                  elseif ($label == 'votes') {
                     
                     if ($lastLetter == 1) { $a = Lang::get('new'); $b = Lang::get('like'); }
                     elseif ($lastLetter > 1 && $lastLetter < 5) { $a = Lang::get('new'); $b = Lang::get('n_likes'); }
                     else { $a = Lang::get('new'); $b = Lang::get('n_likes'); }
                     $ciag .= "$num $a $b, "; $print++;
                  }
                  elseif ($label == 'favorites') {
                     if ($lastLetter == 1) { $a = Lang::get('has_added_your_post'); }
                     elseif ($lastLetter > 1 && $lastLetter < 5) { $a = Lang::get('have_added_your_post');}
                     else { $a = Lang::get('have_added_your_post'); }
                     $b = Lang::get('to_favorites'); 
                     $ciag .= "$num $a $b, "; $print++;
                  }
               }
            }

            if ($print) {

               $title = $groups['byid'][$postId]['title'];
               $thumb = $groups['byid'][$postId]['thumb'];
               
               $output .= '
               <a href="'.App::postLink($postId, $title).'"><div class="single-note row">
               <div class="col note-icon"><img src="'.$thumb.'"></div>
               <div class="col note-inner" style="padding-top:8px;">
               <div class="col note-head">'.Lang::get('your_post').' "<span style="color:#888">'.$title.'</span>" '.Lang::get('gets').': '.substr($ciag,0,strlen($ciag)-2).'</div>
               <div class="col note-bottom" style="margin-top:3px;">'.Lang::get('goto_post').'</div>
               </div><i class="icon-'.$icon.'"></i>
               </div></a>'; unset($ciag);
            }
            
         } if (!$print) { $output = '<div class="emptynotes">'.Lang::get('nothing_found').'</div>'; }
      }
      
      elseif (isset($group) && count($group) > 0) {

         foreach($group as $note) {

            $type = $note->note_type;

            if ($note->params) { $params = $note->params; }
            if ($type=='post' and !$this->sql->countRows('posts','id=? and status > 0', $note->parent)) {continue;}

            // Notofication message
            // ************************************

            $infoData = $this->makeInfo($type,$params);
            $info = $infoData['info']; $icon = $infoData['icon'];

            // Creating links
            // ********************************

            if ($note->parent) {
               isset($linkTab) ?: $linkTab = [];
               if (!isset($linkTab[$note->parent])) {
                  $postData = $this->sql->where('id', $note->parent)->getOne('posts', ['id','tytul','type','category']);
                  $linkTab[$note->parent] = App::postLink($postData);
                  $link = $linkTab[$note->parent]; 
               } else { $link = $linkTab[$note->parent]; }
            }

            if ($type=='pscm' or $type=='cmre' or $type=='cmvt') { $link .= '#comment_' . @$params->comment_id; }
            elseif ($type=='phcm' or $type=='phvt' or $type=='phfv') { $link .= '#photo_' . @$params->photo_id; }
            elseif ($type=='usfl' or $type=='phvt' or $type=='phfv') { $link = 'profile' . SR . @$note->send_by; }
            elseif ($type=='post') { $link = $linkTab[$note->parent]; }
            
            $title = $small ? '' : (
               isset($groups['byid'][$note->parent]['title']) 
                  ? '<span style="color:red">"'. $groups['byid'][$note->parent]['title'] .'"</span>' 
                  : ''
            );
            
            $output .= '<a class="notelink" href="'.$link.'"><div class="single-note row">
            <div class="note-icon"><img src="'.App::getAvatar($note->send_by).'"></div>
            <div class="note-inner">

            <div class="note-head"><b>' . App::fakeName($note->send_by) . '</b> '.$info.' '.$title.'</div>
            <div class="note-bottom"><i class="icon-clock"></i> '.App::zmienczas($note->created).'</div>

            </div><i class="icon-'.$icon.'"></i>
            </div></a>';

         }
      } else { $output = '<div class="emptynotes">'.Lang::get('nothing_found').'</div>'; }
      return $output;
   }

   // ** GRUPOWANIE POWIADOMIEŃ ** //

   private function setArray($base, $fields) {
      foreach ($fields as $field) {
         $base[ $field ] = [];
         $base[ $field ]['data'] = [];
      } return $base;
   }
   
   public function groupNotes($limit = 50, $page = 0, $mode) {
      
      $query = $this->prepareQuery($limit, $page, true, true, $mode);
      $result = $this->sql->ObjectBuilder()->rawQuery($query);

      if ($this->sql->count) {
         
         $group = []; $group = $this->setArray($group, [
            'all','favorites','followers',
            'votes','following','comments'
         ]);
         
         foreach($result as $note) { // Dla każdej notatki
            
            $type = $note->note_type; 
            
            // Remove unnecessary vars
            unset($note->id, $note->send_to);
            isset($note->params) && $note->params = json_decode($note->params);

            if ($note->parent && !isset($group['byid'][$note->parent]['title']) ) {
               $post = $this->getRowsById('posts' ,'tytul, dir, hash', $note->parent);
               $group['byid'][$note->parent]['title'] = $post->tytul; 
               $group['byid'][$note->parent]['thumb'] = "Uploads/Media/{$post->dir}/m_{$post->hash}.jpg";

            }

            if ($note->parent && $type != 'post') { 
               $group['byid'][$note->parent]['data'][] = $note; 
            }
            
            $group['all']['data'][] = $note;

            if ($type=='usfl') {
               isset($group['followers']) ?: $group['followers'] = [];
               $group['followers']['data'][] = $note; 
            }
            elseif ($type=='psfv' or $type=='phfv') {
               isset($group['favorites']) ?: $group['favorites'] = [];
               $group['favorites']['data'][] = $note; 
               $group['byid'][$note->parent]['favorites']++;
            }
            elseif ($type=='cmvt' or $type=='phvt' or $type=='psvt') {
               isset($group['votes']) ?: $group['votes'] = [];
               $group['votes']['data'][] = $note; 
               $group['byid'][$note->parent]['votes']++;
            }
            elseif ($type=='cmre' or $type=='phcm' or $type=='pscm') {
               isset($group['comments']) ?: $group['comments'] = [];
               $group['comments']['data'][] = $note; 
               @$group['byid'][$note->parent]['comments']++;
            }
            elseif ($type=='post' and $this->sql->countRows('posts','id=? and status > 0',$note->parent)) {
               isset($group['following']) ?: $group['following'] = [];
               $group['following']['data'][] = $note;
            }
         }
         
         $group['all']['num'] = count($group['all']['data']);
         $group['votes']['num'] = count($group['votes']['data']);
         $group['favorites']['num'] = count($group['favorites']['data']);
         $group['followers']['num'] = count($group['followers']['data']);
         $group['following']['num'] = count($group['following']['data']);
         $group['comments']['num'] = count($group['comments']['data']);
         
      }
      
      return isset($group) ? $group : array();
      
   }
   
   private function byType($type) {

      $alt = array();
      $conf = (object) $this->conf;
      
      switch ($type) {
            
         case 'votes':
            if ($conf->psvt) $alt[] = 'note_type="psvt"';
            if ($conf->phvt) $alt[] = 'note_type="phvt"';
            if ($conf->phvt) $alt[] = 'note_type="cmvt"';
            break;

         case 'comments':
            if ($conf->pscm) $alt[] = 'note_type="pscm"';
            if ($conf->phcm) $alt[] = 'note_type="phcm"';
            if ($conf->cmre) $alt[] = 'note_type="cmre"';
            break;

      } 
      
      return !empty($alt) ? '('.implode(' or ', $alt).')' : false;

   }
   
   public function prepareQuery($limit = 50, $page = 0, $onlyLogged = true, $unseen = true, $mode) {

      ### $page / $limit - strona początkowa i limit powiadomień
      ### $onlyLogged - tylko powiadomienia od zalogowanych użytkowników

      if ($unseen) { $see = "seen='0' AND"; }

      $sqlQuery = "select * from notes where $see send_to='{$this->username}'";

      if ($onlyLogged) { $sqlQuery .= ' AND logged="1"'; } // Tylko od zalogowanych
      
      if ($mode == 'comments' || $mode == 'votes') {
         $byType =  $this->byType($mode);
         if ($byType) $sqlQuery .= ' AND ' . $this->byType($mode);
         else $sqlQuery .= ' AND note_type="NONE"';
      } else { 
         if ($this->config && $this->follows) { $sqlQuery .= ' AND ('.$this->config.') OR (('.$this->follows.') AND note_type="post")'; }
         elseif ($this->config && !$this->follows) { $sqlQuery .= ' AND ('.$this->config.')'; }
      }

     $sqlQuery .= ' ORDER BY id DESC'; // Sortowanie
     $sqlQuery .= ' LIMIT '.($page*$limit).",{$limit}";

     $replaced = str_replace('*', 'count(id)', $sqlQuery);
     $replaced = preg_replace('/LIMIT (\d+),(\d+)/i', 'limit 1', $replaced);
     $this->sql->lastNum = $this->sql->rawQueryValue( $replaced );
           
     return $sqlQuery;

   }

}