<?php

Class Posts {
   
   use Stats;
   
   // Instances
   
   private $sql;
   private $get;
   private $cache;
   
   public $logged;
   public $autoplay;
   public $viewmode;
   public $postlimit;
   public $head_bg;
   public $displaymode;
   
   public $rows;
   public $query;
   public $post;
   
   public function __construct($mode) {
      $this->mode = $mode;
      $this->head_bg = $this->getBg();
      $this->logged = User::Logged();
      $this->admin = User::isAdmin();
      $this->sql = Factory::get('Database');
      $this->get = App::getUrlParams();
      $this->type = $this->getType();
      $this->viewmode = $this->getViewMode();
      $this->postlimit = $this->getPostsLimit();
      $this->displaymode = $this->getDisplayMode();
      $this->autoplay = $this->getAutoplay();
      if ($this->mode) $this->setMode($this->mode);
   }
   
   public function getBg() {
      $ref = $_SERVER["REQUEST_URI"];
      $hash = substr(hash('crc32', parse_url($ref)["path"]),1);
      $file = PATH_ROOT . '/Uploads/Icons/bg-'.$hash.'.jpg';
      return file_exists($file) ? 'Uploads/Icons/bg-'.$hash.'.jpg' : 'Uploads/Icons/bg-head.jpg';
   }
   
   public function setMode($mode) {

      if (isset($this->get->id)) $this->id = (int) $this->get->id;
      if (isset($this->get->page)) $this->page = (int) $this->get->page;

      if ($mode == 'posts') {
         
         $this->get->limit = $this->postlimit;
         $_prep = $this->sql->prepareQuery($this->get);
         $this->results = $_prep['rows'];
         $this->query = $_prep['query'];
      }
      
      elseif ($mode == 'profile') {
         $this->query = "select * from posts where status > 0 and logged = 1 and user = '{$this->get->user}' ";
         $this->query .= "order by id desc limit $this->postlimit";
      }
      
      elseif ($mode == 'single') {
      
         $this->post = new stdClass();
         
         if (!User::isAdmin()) { $this->sql->where('status','0','>'); }

         $this->sql->where('id', (int) $this->get->id);
         $this->post = (object) $this->sql->getOne('posts');
         $this->post->top_comments = $this->sql->countRows('comments', 
            "wpis=? AND type='post' AND status='1' and reply_id = 0", [$this->post->id]
         );
         
         if (!$this->sql->count) App::Redirect();
         
         $this->post = $this->_preparePostData($this->post);
         $this->updateViews($this->post->id, $this->post->type);
         
         // Routing for post view
         switch ($this->post->type) { 
            case 'video': case 'audio':
            case 'gif': $this->output = $this->_prepareVideo($this->post); break;
            case 'article': $this->output = $this->_prepareArticle($this->post); break;
            case 'photos': $this->output = $this->_prepareImage($this->post); break;
         }
         
      }
      
   }
   
   private function getCategory($cat) {
      $cat = $this->sql->escape( $cat );
      $sql = $this->sql->where('name', $cat)->getOne('categories');
      return $sql ? $sql : false;
   }
   
   private function getPage() {
      if ($this->get->page) {
         return (int) $this->get->page + 1;
      } else return '';
   }
   
   private function getType() {
      
      $get = $this->get;
      
      switch (true) {
         case isset($get->tagi): return 'tags'; break;
         case isset($get->search): return 'search'; break;
         case isset($get->category): return 'category'; break;
         default: return isset($this->get->type) ? $this->get->type : 'all';
      }

   }
   
   public function getMeta() {
      
      $get = $this->get;
      
      if (isset($get->sort)) { switch (true) {
         case strstr($get->sort, 'today'): $top = 'of_the_day'; break;
         case strstr($get->sort, 'week'): $top = 'of_the_week'; break;
         case strstr($get->sort, 'month'): $top = 'of_the_month'; break;
         case strstr($get->sort, 'year'): $top = 'of_the_year '; break;
      } isset($top) && App::set('sort', Lang::get($top)); }
      
      switch (true) {
            
         case isset($get->tagi): case isset($get->search):
            $name = App::remLine($get->tagi ?: $get->search);
            App::set('name', mb_ucwords($name, "utf-8"));
            break;
            
         case isset($get->category):
            $cdata = App::remLine($get->category);
            $data = $this->getCategory( $cdata );
            if ($data) {
               App::set('name', $data['name']);
               App::set('desc', $data['desc']);
            } break;
      
      }

      return (object) [
        'title' => Lang::get('meta_title_' . $this->type),
        'under' => App::get('desc') ? App::get('desc') : Lang::get('under_' . $this->type),
        'desc' => App::get('desc') ? App::get('desc') : Lang::get('meta_desc_' . $this->type)
      ];
      
   }
   
   public function getOutput() {
      return $this->output;
   }
   
   private function _loadTemplate($filename) {
      if ($this->cache[$filename] instanceof Template) return $this->cache[$filename];
      else return new Template($filename);
   }
   
   private function _prepareImage($_post) {
      
      $_post = $this->post;
      $_pics_on_page = App::get('pics_on_page');
      
      if ($this->mode == 'single') { // Creating data for single post
         
         $_num = $this->sql->countRows('photos','pid=?',$_post->id); // How many photos?
         
         if ($_num > 1) { // Gallery
            
            $curr_page = isset($this->page) ? (int) $this->page : 0; // Current gallery page
            $pages = ($_num > $_pics_on_page) ? ceil($_num / $_pics_on_page) : 0; // Overall pages
            
            // Fetching data for images
            
            $_images = $this->sql->ObjectBuilder()
               ->where('pid',$this->id)->orderBy('position','asc')
               ->get('photos',[$curr_page*$_pics_on_page,$_pics_on_page]);
            
            $_Base = new Template('loop_post_images');
            
            if ($pages) {
               $nextPage = $curr_page+1;
               if ($nextPage < $pages) $paging = $_Base->_cache('post_gallery_next');
               elseif ($nextPage == $pages) $paging = $_Base->_cache('post_gallery_back');
            }
            
            array_map(function($elem) use (&$curr_page) {
               
               $path = '/Uploads/Media/' .$elem->dir. '/' . $elem->file;
               $elem->gifclass = $elem->type == 'gif' ? 'gif-player' : '';
               $elem->num = ($curr_page * 10 + 1);
               $elem->imgpath = App::get('app_url') . $path;
               $elem->comments = App::getComments($elem->id, 'photo');
               
               if ($elem->orientation >= 1) {$elem->class = 'wide';}
               else {$elem->class = 'height'; }
               
            }, $_images);
            
            if ($pages) $_Base->extend('gallery_paging', $paging);
            
            $_Base->extend([
               'bigshare' => $_Base->_cache('post_single_bigshare'),
               'voteline' => $_Base->_cache('post_single_voteline'),
               'voting' => $_Base->_cache('post_single_voting')
            ]);
            
            return $_Base->render( compact($_Base->getTags()) );
            
         } else { // Single photo
            
            $img = $this->sql->ObjectBuilder()->where('pid',$this->id)->getOne('photos');
            $img->gifclass = $img->type == 'gif' ? 'gif-player' : '';
            $imgpath = App::get('app_url') . '/Uploads/Media/' .$img->dir. '/' .$img->file;
            
            $_Base = new Template('post_single_image');
            $_Base->extend([
               'bigshare' => $_Base->_cache('post_single_bigshare'),
               'voteline' => $_Base->_cache('post_single_voteline'),
               'voting' => $_Base->_cache('post_single_voting')
            ]);
            return $_Base->render( compact($_Base->getTags()) );
         }
         
      } else { // gdy wyświetlamy posty dla listy wpisów
         
      }
      
   }
   
   private function _prepareArticle($_post) {
      
      $_Base = $this->_loadTemplate('post_single_article');
      $_Base->extend([
         'nextprev' => $_Base->_cache('post_single_nextprev'),
         'bigshare' => $_Base->_cache('post_single_bigshare'),
         'voteline' => $_Base->_cache('post_single_voteline'),
         'voting' => $_Base->_cache('post_single_voting')
      ]);
      return $_Base->render( compact($_Base->getTags()) );
      
   }
   
   private function _prepareVideo($_post) {

      $_Base = $this->_loadTemplate('post_single_video');
      $media = $_post->media;
      if ($_post->mode == 'local') {
         if ($_post->type == 'video' ) { $_player = $_Base->_cache('post_video_player'); }
         elseif ($_post->type == 'audio' ) { $_player = $_Base->_cache('post_audio_player'); }
         else { $_player = $_Base->_cache('post_gif_player'); }
      } else { $_player = $_Base->_cache('post_embed_player'); }
      
      $_Base->extend([
         'nextprev' => $_Base->_cache('post_single_nextprev'),
         'bigshare' => $_Base->_cache('post_single_bigshare'),
         'voteline' => $_Base->_cache('post_single_voteline'),
         'voting' => $_Base->_cache('post_single_voting')
      ]);
      
      $_Base->extend('player', $_player);
      return $_Base->render( compact($_Base->getTags()) );
      
   }
  
  private function getLikesPrc($data) {
    return ($data->likes > 0) 
      ? intval( ($data->likes / ($data->likes + $data->dislikes)) * 100 )
      : 0;
      
    $_post->prc = ( $_post->likes / ( $_post->likes + $_post->dislikes ) ) * 100;
  }
   
   private function getCatLink($name) {
      if ($name) {
         return '<a href="category' . SR . App::addLine($name) . '">' . $name . '</a>';
      } else { return Lang::get('uncategorized'); }
   }
   
   private function _preparePostData($_post) {
      
      $_post->id = (int) $_post->id;
      $_post->name = App::fakeName($_post->user);
      $_post->czas = App::unixTimeArray($_post->czas);
      $_post->catlink = $this->getCatLink($_post->category);
      $_post->czas['ago'] = App::zmienczas($_post->czas['unix']);
      $_post->profile = $_post->logged ? 'profile' . SR . $_post->user : '#';
      $_post->url_relative = App::postLink($_post);
      $_post->url_http = App::Config('app_url') .'/'. $_post->url_relative;
      $_post->avatar = $_post->logged ? App::getAvatar($_post->user) : App::getAvatar();
      $_post->taglist = App::tagList($_post->tagi);
      $_post->imgdata = $this->_postImage($_post);
      $_post->score = $_post->likes - $_post->dislikes;
      $_post->path = 'Uploads/Media/' . $_post->dir . '/' . $_post->hash;
      $_post->opis = strip_tags($_post->opis,"<ul><ol><dl><li><h1><h2><h3><h4><h5><h6><table><tr><td><th><thead><video><source><audio><iframe><blockquote><span><a><p><img><br>");
     
      $_post->prc = $this->getLikesPrc($_post);
      
      if ($_post->mediadata) {
         $_post->media = json_decode($_post->mediadata);
         $_post->extension = str2var($_post, 'media->mime') ? App::mime2ext($_post->media->mime) : 'mp4';
         $_post->media->file = str2var($_post, 'media->file') ?: str2var($_post, 'media->media_mp4');
         if ($_post->type == "video" && $_post->mode == "local") 
            $_post->vurl = $_post->media->file ?: $_post->path . '.' . $_post->extension;
      }
      
      if ($this->mode != 'posts') $_post->more = App::more_posts($_post->id);
      
         if ($_post->type == 'gif') $_post->listmode = 'gif';
         elseif ($_post->mode == 'embed') $_post->listmode = 'embed';
         elseif ($_post->mode == 'video') $_post->listmode = 'video';
         elseif ($_post->mode == 'audio') $_post->listmode = 'audio';
         else $_post->listmode = $_post->type;
      
      $_post->meta = new stdClass();
      $_post->meta->tags = $_post->tagi;
      $_post->meta->title = $_post->tytul .' - '. App::Config('app_name');
      $_post->meta->t_ent = htmlentities($_post->tytul, ENT_QUOTES);
      $_post->meta->cache = str2var($this->get, 'cache');
      $_post->meta->cache && Session::set('cache', true);
      $_post->meta->desc = htmlspecialchars(mb_substr(strip_tags($_post->opis),0,150,'utf-8'));
      $_post->meta->thumb = App::Config('app_url') . '/Uploads/Media/' . $_post->dir . '/' . $_post->hash.'.jpg';
      $_post->video = App::getFile() == "post.php" ? str2var($_post, 'vurl') : 'Uploads/mp4.mp4';
      
      return $_post;
      
   }
  
  public function bottomList($title) {
      
      $title =  $this->sql->escape(trim(preg_replace('/[^a-zA-Z0-9 ]/','',$title)));
      $query = $this->sql->prepareQuery(['limit' => 8, 'similar' => true, 'title' => $title]);
      $results = $this->sql->ObjectBuilder()->rawQuery($query['query']);

      if ($query['rows'] < 4) {
        $query = $this->sql->prepareQuery(['limit' => 8, 'sort' => 'random']);
        $results = $this->sql->ObjectBuilder()->rawQuery($query['query']);
      }
     
      array_map(function($elem) {
         $elem->name = App::fakeName($elem->user);
         $elem->time = App::zmienczas($elem->czas);
         $elem->avatar = App::getAvatar($elem->user);
         $elem->link = App::postLink($elem); 
         return $elem;
      }, $results);

      $item = new Template('loop_btmlist', ['scope' => false]);
      return $item->render( compact($item->getTags()) );

   }
   
   public function redirect() {

      $requestUri = explode(',',$_SERVER['REQUEST_URI']);
      $queryCheck = strpos($requestUri[0],'?') ? strpos($requestUri[0],'?')-1 : strlen($requestUri[0]);
      $redir = substr($requestUri[0],1,$queryCheck);
      
      if( $this->page && $this->rows <= ($this->page * $this->postlimit) ) header("Location: $redir",true,301);
      if( $this->page < 0 ) App::Redirect();
   }
   
   private function getAutoplay() {
      return Cookie::get('autoplay') ? 'checked' : '';
   }
   
   private function getDisplayMode() {
      return (Cookie::get('pagemode') == 'pagination') ? 'pagination' : 'infinite';
   }
   
   private function getPostsLimit() {
      switch ($this->mode) {
         case 'profile': return $this->viewmode == 'list' ? App::Config('prof_num_lists') : App::Config('prof_num_icons'); break;
         default: return $this->viewmode == 'list' ? App::Config('page_num_lists') : App::Config('page_num_icons'); break;
      }
   }
   
   private function getViewMode() {
      return Cookie::get('viewmode') ?: App::get('page_view_mode');
   }
   
   public function jsParams() {
      if (is_object($this->get)) { 
         foreach($this->get as $key => $val) $getArray[] = "$key: '$val'";
         return implode(', ',$getArray);
      }
   }
   
   public function _postImage($_post) {

      $_img = new stdClass();
      $_source = App::getFromJson($_post->mediadata,'source');

      $_img->path  = 'Uploads/Media/'.$_post->dir.'/';
      $_img->image = $_img->path . $_post->hash.'.jpg';
      $_img->thumb = $_img->path .'m_'. $_post->hash.'.jpg';
      
      $_img->image_http = App::Config('app_url') .'/'. $_img->path . $_post->hash.'.jpg';
      $_img->thumb_http = App::Config('app_url') .'/'. $_img->path .'m_'. $_post->hash.'.jpg';
      
      $_img->icon = App::getBrand($_post->type, $_post->mode, $_source, true);
      

      return $_img;
      
   }
   
   public function noResults() {
      if ($this->page) header("Location: all");
      return $this->mode == 'profile' 
         ? '<div class="nothing_found"><div>'.Lang::get('no_posts').'</div></div>'
         : '<div class="nothing_found"><div>'.Lang::get('nothing_found').'</div></div>';
   }
   
   public function listView($query = false) {
      
      if (!$query) $results = $this->sql->ObjectBuilder()->rawQuery($this->query);
      else $results = $this->sql->ObjectBuilder()->rawQuery($query);
      
      $this->rows = $this->sql->count;
      
      if ($this->rows) {
         
         $_tpl = new Template('postlist_full_single');
         $_vars = $_tpl->getTags();
         
         array_map(function($elem) {
            $elem = $this->_preparePostData($elem);
            if ($elem->type == 'photos') {
               $_img = $this->sql->where('pid', $elem->id)->getOne('photos', 'orientation');
               $elem->imgclass = ($_img['orientation'] >= 1) ? 'img-wide' : 'img-height';
               $elem->picnum = $this->sql->countRows('photos', "pid={$elem->id}");
            }
         }, $results);
         
         return $_tpl->render( compact($_vars) );
         
      } 
      
      return $this->noResults();
      
   }
   
   public function iconView($query = false) {

      if (!$query) $results = $this->sql->ObjectBuilder()->rawQuery($this->query);
      else $results = $this->sql->ObjectBuilder()->rawQuery($query);
      
      $this->rows = $this->sql->count;
      
      if ($this->rows) {
         
         $i = 1; $_tpl = new Template('postlist_icon_single');
         
         array_map(function($elem) use (&$i) {
            $elem = $this->_preparePostData($elem);
         }, $results);
         
         $output = $_tpl->render(compact($_tpl->getTags()));
         return $output;
         
      } 
      
      return $this->noResults();

   }
   
   public function getProfilePosts(){
      $posts = ($this->getViewMode() == 'list') ? $this->listView() : $this->iconView();
      return $posts ? $posts : '<div id="nopost"><i class="icon-search"></i>'.Lang::get('postlist_empty').'</div>';
   }
   
}