<?php

defined('APP_VERSION') || die("This file can't be accessed directly");

class ProfileEdit extends PanelsEdit {

   protected $Messages;
   
   public function __construct() {

      parent::__construct();
      
      $this->Messages = Factory::get('Messages');

      switch ($this->post->type) {
         case 'rm_msg': $this->RemoveMessage(); break;
         case 'ping': $this->GetMessageNum(); break;
         case 'sendmsg': $this->SendMessage(); break;
         case 'usr_list': $this->UsersList(); break;
         case 'note_cfg': $this->Notifications(); break;
         case 'changepass': $this->ChangePass(); break;
         case 'editprofile': $this->EditProfile(); break;
         case 'publish_draft': $this->PublishDraft(); break;
      }

   }

   private function PublishDraft() {

      $id = (int) $this->post->id;
      $status = $this->getPublishStatus();
      $this->sql->where('id', $id)->where('user', $this->user)
                ->update('posts', ['status' => $status->id]);

      echo $status->icon;

   }
   
   private function GetMessageNum() {
      echo $this->Messages->countMessages();
   }
   
   private function RemoveMessage() {

      $id = (int) $this->post->id;
      $self = (bool) $this->post->self;
      $mod = explode('_', $this->post->mode);

      switch ( $mod[0] ) {
         case 'trash': $this->Messages->trash($id, $self); break;
         case 'delete': $this->Messages->delete($id, $self); break;
      }

      echo 1;

   }
   
   private function SendMessage() {
      
      // Local error handlers
      $info = function($type, $code) {die(json_encode([$type => true,'message' => $code]));};
      $check = function($topic, $message, $user) use (&$info) {

         $t_len = strlen($topic); $m_len = strlen($message);
         if ($user == User::get('username'))  $info('error', Lang::get('msg_self'));
         elseif ($t_len < 5 || $t_len > 60) $info('error', Lang::get('msg_topic_len'));
         elseif ($m_len < 10) $info('error', Lang::get('msg_too_short'));

      };

      $tags  = 'iframe,blockquote,strong,ul,ol,li,a,p,span,hr,u,i,b,';
      $tags .= 'img,pre,h1,h2,h3,h4,h5,h6,em,br,table,tr,td,tbody,div';
      $stay = '<' . implode('><', explode(',', $tags)) . '>';

      $user = htmlentities($this->post->to);
      $topic = htmlentities($this->post->subject);
      $message = strip_tags($_POST['message'], $stay);

      User::userExists($user) ?: $info('error', Lang::get('msg_user_nf'));
      $check($topic, $message, $user);

      if ($this->Messages->send($user, $topic, $message)) {
         $info('success', Lang::get('msg_sent'));
      }
         
   }
   
   private function UsersList() {
      $user = preg_replace('/[^a-z0-9_]/i','', $this->post->val);
      $users = $this->sql->where('username', "%{$user}%",'like')->get('login',10,'username');
      if ($users) { foreach($users as $usr) { $out[] = $usr["username"]; }
      echo json_encode($out); };
   }
   
   private function EditProfile() {

      $data = array();
      $post = $this->post;
      
      $data['user_email'] = Filter::Email($post->email) ? $post->email : '';
      $data['user_blog']  = Filter::Url($post->blogurl) ? $post->blogurl : '';
      $data['user_name']  = $this->isValid('fakename', $post->nazwa) ?: '';
      $data['user_bio']   = (strlen($post->bio) <= 400) ? $this->sql->escape($post->bio) : '';

      $data['user_fburl'] = $this->socialLinks('facebook', $post->fburl);
      $data['user_twurl'] = $this->socialLinks('twitter', $post->twurl);
      $data['user_gpurl'] = $this->socialLinks('google', $post->gpurl);
      $data['user_pturl'] = $this->socialLinks('pinterest', $post->pturl);
      $data['user_yturl'] = $this->socialLinks('youtube', $post->yturl);

      echo $this->sql->where('username', $this->user)->update('login', $data) ? 1 : 'error';

   }
   
   private function socialLinks($site, $value) {
      
      $social = [
         'facebook' => 'facebook.com', 'twitter' => 'twitter.com',
         'google' => 'plus.google.com', 'pinterest' => 'pinterest.com',
         'youtube' => 'youtube.com'
      ];
      
      if ($social[$site]) {
         $valid = Filter::Url($value) && stristr($value, $social[$site]);
         if (!$valid && !empty($value)) { 
            die( Lang::get('invalid_profile', ['site' => ucfirst($site)]) ); }
         return $valid ? $value : '';
      }
      
      return '';
      
   }
   
   private function ChangePass() {
      
      $post = $this->post;
         
      $this->isValid('password', $post->pass1) ?: $this->Message('pass_invalid', 1);
      $this->isValid('pass2pass', [$post->pass1,$post->pass2]) 
         ?: $this->Message('pass_notmatch', true);

      $basePass = $this->getPassword($this->user);
      if (password_verify($post->oldpass, $basePass)) {

         $passHash = password_hash($post->pass1, PASSWORD_DEFAULT);
         $update = $this->sql->where('username', $this->user)->update('login', [ 'password' => $passHash ]);

         echo ($update) ? 1 : $this->Message('error_occurred');

      } else { $this->Message('old_pass_invalid', true); }

   }
   
   private function Notifications() {
      $fields = array('post','psvt','pscm','psfv','phvt','phcm','phfv','cmre','cmvt','usfl');
      if (in_array($this->post->opt, $fields)) {
         $option = $this->post->st == 'true' ? 1 : 0;
         $r = $this->sql
            ->where('user', $this->user)
            ->update('notes_config', [$this->post->opt => $option]);
      }
   }
   
   private function getPassword($user) {
      return $this->sql->where('username', $user)
         ->getValue('login', 'password');
   }
   
}