<?php

class Routing {
    
    private $controller = null;
    private $action = null;
    private $params = [];

    /*
     * "Start" the application:
     * Analyze the URL and calls the according controller/method
     */

    public function __construct() {
        
        $this->splitUrl(); // Splitting the URL
        isset($this->controller) ?: App::redirect();

        $class = "Controllers\\{$this->controller}";

        if ( class_exists($class) ) {

            $class = new $class();
            $action = $this->action ?: 'index';

            if (method_exists($class, $action)) {

                $reflectionMethod = new ReflectionMethod($class, $action);
                $reflectionMethod->invokeArgs($class, $this->params);

            } else {
                
                // Calling index method or redirect
                method_exists($class, 'index')
                    ? $class->index() : App::Redirect();
                
            }
            
        } elseif ($match = glob("Template/Views/{$this->controller}*")) {

            $path = pathinfo($match[0]);

            switch ( $path['extension'] ) {
                case 'php': include $match[0]; break;
                case 'tpl': (new Template)->loadView($match[0])->display();
            }

        } else {

            $sql = Factory::get('Database');
            $slug = App::remLine( $this->controller );

            $params = [
                'custom_page', 
                'title=? or name=?', 
                [$slug, $this->controller]
            ];

            // If custom page exists
            if (call_user_func_array([$sql, 'countRows'], $params)) {

                $_data = $sql
                    ->where('title', $slug)
                    ->orWhere('name', $this->controller)
                    ->getOne("custom_page", ["title","content"]);

                (new Template('base_info'))->extend([
                    'title' => $_data['title'],
                    'body' => stripcslashes($_data['content'])
                ])->display();

            } else { App::Redirect(); }

        }
        
    }

    private function splitUrl() {
    
        if (isset($_GET['url'])) {

            // split URL
            $url = trim($_GET['url'], '/');
            $url = filter_var($url, FILTER_SANITIZE_URL);
            $url = explode('/', $url);

            // Put URL parts into according properties
            $this->controller = isset($url[0]) ? $url[0] : null;
            $this->action = isset($url[1]) ? $url[1] : null;

            // Remove controller and action from the split URL
            unset($url[0], $url[1]);

            // Rebase array keys and store the URL params
            $this->params = array_values($url);

        }
    }
}