<?php

defined('APP_VERSION') || die("This file can't be accessed directly");

Class TemplateException extends Exception {

   public function __construct($message, $code = 0, Exception $previous = null) {
      parent::__construct($message, $code, $previous);
      $this->Display();
    }

    public function Display() {
      $function = $this->_createDiv();
      $error = $this->_parseTrace();
      $function($error);
    }

    private function _createDiv() {

      $this->style = '
        p { margin:5px 0 5px 10px; color:cornsilk }
        .err{ background: #3E4A4A; color: #fff; margin: 10px; padding:15px }
        .err-head { padding-bottom:8px; margin-bottom:8px; border-bottom:1px solid #fff }
      ';

      return function($error) {
        echo '<style>'.$this->style.'</style>';
        echo '<div class="err">'.$error.'</div>';
      };

    }

    public function _parseTrace() {

      $root = PATH_ROOT .'/';

      $trace = array_map(function($e) {
        $output  = "&#8627; {$e['file']}";
        $output .= "({$e['line']}): {$e['class']}{$e['type']}{$e['function']}";
        return $output .= '(' .implode(',',$e['args']) . ')';
      }, $this->getTrace());

      $message = '<div class="err-head">' . 'TemplateException: ' . $this->message . '</div>';
      $message .= '<p>'.str_replace($root, '', implode('</p><p>', $trace)).'</p>';
      return $message;

    }

} // END Class

Class Template {

   const RX_EXTEND = '/\[@extend\s(.*?)\]/s';

	private $template;
	private $cache = [];

  private $config = [
    'scope' => true,
    'tpl_dir' => THEME
  ];

  public function __construct($file = null, array $options = null) {
    if ($file) $this->template = $this->loadFile($file);
    if ($options) $this->config = array_merge($this->config, $options);
    return $this;
  }

    public function loadFile($file) {
        $file = $this->config['tpl_dir'] .'/'. $file . '.tpl';
        if (file_exists($file)) return file_get_contents($file);
        else if ($__tpl__ = $this->searchFile($file)) return $__tpl__;
        else throw new TemplateException("Template file not found: {$file}.tpl");
    }

    public function loadView($file) {
        $this->template = file_get_contents($file);
        return $this;
    }

  public function searchFile($__tpl__, $path) {
    $dir = $path ?: $this->config['tpl_dir'];
    foreach (glob($dir . '/*') as $file) {
      if (is_dir($file) && $output = $this->searchFile($__tpl__, $file)) return $output;
      else if (basename($file) == $__tpl__ . '.tpl') return file_get_contents($file);
    } return false;
  }

    public function extend($elem, $data = null) {

      if (!$this->template) throw new TemplateException("Template not set. Can\'t append $elem");

      if ($elem) {
        if (is_array($elem)) foreach ($elem as $key => $val) $this->_swapAppendTags($key, $val);
        else $this->_swapAppendTags($elem, $data);
      } else throw new TemplateException('Can\'t append empty '.$elem.' to base template');

      return $this;
  }

  private function _swapAppendTags($elem, $data) {
      if ($data) $this->template = preg_replace('/\[\@append '.$elem.'\]/i', $data, $this->template);
      else throw new TemplateException('Can\'t append empty '.$elem.' to base template element');
   }

   // =============================================
   // ============= Display Functions =============
   // =============================================

   public function display($_vars = null) {
      echo $this->_prepare(null, $_vars);
   }

   public function render($_vars = null) {
      return $this->_prepare(null, $_vars);
   }

   public function loadAndRender($file, $_vars = null) {
      return $this->_prepare($this->_cache($file), $_vars);
   }

   // ==============================================
   // ============= Swapping Functions =============
   // ==============================================

   private function _prepare($__tpl__, $_vars = null) {

      if ($_vars) $_vars = $this->clearVars( $_vars );

      $template = $__tpl__ ? $__tpl__ : $this->template;
      $tags = $this->_findTags($template);
      $template = preg_replace('/<!Path>/', App::get('app_path'), $template);
      $template = $this->_swapConstants($template);
      $template = preg_replace('/\[@ignore\](.*?)\[@endignore\]/is', '', $template);

      if (isset($tags->include)) $template = $this->_swapIncludeTags($template);
      if (isset($tags->foreach)) $template = $this->_swapLoops($template, $_vars);
      if (isset($tags->extend)) $template = $this->_swapExtend($template);
      if (isset($tags->if)) $template = $this->_swapIfTags($template, $_vars);
      if (isset($tags->ifnot)) $template = $this->_swapIfNotTags($template, $_vars);
      if (isset($tags->variables)) $template = $this->_swapVariables($template, $_vars);
      if (isset($tags->config)) $template = $this->_swapConfig($template, $_vars);
      if (isset($tags->can) || isset($tags->cant)) $template = $this->_swapCan($template, $_vars);
      if (isset($tags->advanced)) $template = $this->_swapAdvanced($template, $_vars);

      return preg_replace('/\[[\$|@|%](.*?)\]/','',$template);

   }

  private function _swapExtend($__tpl__) {

    preg_match_all(self::RX_EXTEND, $__tpl__, $matches);
    foreach ($matches[1] as $i => $_tpl) { if ($_tpl) {

         $loaded = $this->_cache($_tpl);
         $__tpl__ = str_replace($matches[0][$i], $loaded, $__tpl__);

         if (strpos($__tpl__, '@extend') !== false) {
            return $this->_swapExtend($__tpl__);
         }

    }}

    return $__tpl__;

  }

   private function _findTags(&$__tpl__) {
      $tags = new stdClass;
      $tags->variables = (strpos($__tpl__, '[$')) ? true : false;
      $tags->config = (strpos($__tpl__, '<!Conf')) ? true : false;
      $tags->advanced = (strpos($__tpl__, '[%')) || strpos($__tpl__, '<!Lang') ? true : false;
      foreach (['@if ','@ifnot ','@once','@foreach ','@extend ','@include ','@can','@cant'] as $tag) {
         $tags->{trim(ltrim($tag,'@'))} = (strpos($__tpl__, $tag)) ? true : false;
      } return $tags;
   }

   private function _injectScope() {
      $vars = ['_GET','_POST','_SERVER','_COOKIE','_FILES','_SESSION','GLOBALS','sql']; $_vars = [];
      foreach ($GLOBALS as $name => $variable) if (!in_array($name, $vars)) $_vars[$name] = $variable;
      return $_vars;
   }

   private function _swapIncludeTags($__tpl__) {

      extract( $this->_injectScope() );
      preg_match_all('/\[\@include (.*?)\]/i', $__tpl__, $tags);

      if ($tags[1]) { foreach ($tags[1] as $i => $_inc) {
         ob_start(); include($_inc); $file = ob_get_clean();
         $__tpl__ = str_replace($tags[0][$i], $file, $__tpl__);
      }}

      return $__tpl__;

   }

   private function _swapIfNotTags($__tpl__, $_vars = null) {

      if ($_vars) extract($_vars); $cfg = $this->config;
      if ($cfg['scope']) extract($this->_injectScope());

      $_logged = User::logged();
      $parsed = $this->_parseTemplate($__tpl__);

      if (isset($parsed['ifnot'])) { foreach ($parsed['ifnot'] as $_if) {

         if ($_if->left->vartype == 'variable') { $_var_l = ${$_if->left->name}; }
         elseif ($_if->left->vartype == 'object') { $_var_l = $this->str2var(${$_if->left->object}, $_if->left->name); }

         $_swap = (!$_var_l) ? true : false;
         $__tpl__ = str_replace($_if->tag, ($_swap ? $_if->body : ''), $__tpl__);

      }}

      return $__tpl__;

   }

   private function _swapCan($__tpl__, $_vars = null) {

      if ($_vars) extract($_vars); $cfg = $this->config;
      if ($cfg['scope']) extract($this->_injectScope());

      $_logged = User::logged();
      $parsed = $this->_parseTemplate($__tpl__);
      if (isset($parsed['can'])) { foreach ($parsed['can'] as $_if) {

         $_can = User::can($_if->left->name);

         $_swap = $_can ? true : false;
         $__tpl__ = str_replace($_if->tag, ($_swap ? $_if->body : ''), $__tpl__);

      }}

      if (isset($parsed['cant'])) { foreach ($parsed['cant'] as $_if) {

         $_can = !User::can($_if->left->name);

         $_swap = $_can ? true : false;
         $__tpl__ = str_replace($_if->tag, ($_swap ? $_if->body : ''), $__tpl__);

      }}

      return $__tpl__;

   }

   private function getParsed($__tpl__, $tag) {
      $parsed = $this->_parseTemplate($__tpl__);
      if ( isset($parsed[$tag]) ) { return $parsed[$tag]; }
      return null;
   }

   private function _swapIfTags($__tpl__, $_vars = null) {

      if ($_vars) extract($_vars); $cfg = $this->config;
      if ($cfg['scope']) extract($this->_injectScope());

      $_logged = User::logged();
      $iftags = $this->getParsed($__tpl__, 'if');

      if ($iftags) { foreach ($iftags as $_if) {

        if ($_if->left->vartype == 'variable') { $_var_l = isset(${$_if->left->name}) ? ${$_if->left->name} : null; }
        elseif ($_if->left->vartype == 'object') { $_var_l = $this->str2var(${$_if->left->object}, $_if->left->name); }

        if (isset($_if->right->vartype)) {
           switch ($_if->right->vartype) {
              case 'variable': $_var_r = ${$_if->right->name}; break;
              case 'object': $_var_r = $this->str2var(${$_if->right->object}, $_if->right->name); break;
              case 'value': $_var_r = $_if->right->value; break;
           }
        }

         $_swap = ($_if->count == 1 && $_var_l)
            ? true : ($_if->count == 3 && $this->compare($_var_l, $_if->sign, $_var_r))
            ? true : false;

         if ($_swap && isset($_if->else)) {
            $__tpl__ = str_replace('[@else]' . $_if->else . '[@endelse]','',$__tpl__);
         }

         $__tpl__ = str_replace($_if->tag, ($_swap ? $_if->body : ''), $__tpl__);

      }}

      return $__tpl__;

   }

   private function compare($var, $sign, $comp) {
      switch ($sign) {
         case '&&': return $var && $comp;
         case '||': return $var || $comp;
         case '==': return $var == $comp;
         case '>' : return $var > $comp;
         case '>=' : return $var >= $comp;
         case '<' : return $var < $comp;
         case '<=' : return $var <= $comp;
         case '!=': return $var != $comp;
      }
   }

   private function clearVars($_vars) {
      $arr = ['_SERVER','_ENV','GLOBALS','_SESSION','_COOKIE','_GET','_POST','_FILES','_REQUEST'];
      foreach ($_vars as $name => $value) { if (in_array($name, $arr)) { unset($_vars[$name]); } }
      return $_vars;
   }

   private function _swapVariables($template, $_vars = null) {

      $_logged = User::logged();
      $_random = rand(1,99999);

      if ($_vars) extract($_vars); $cfg = $this->config;
      if ($cfg['scope']) extract($this->_injectScope());

      if (preg_match_all('/\[\$(.*?)\]/i', $template, $_dollar_tags)) {

         foreach ($_dollar_tags[1] as $i => $_varname) {

            $_varname = $this->_normalize($_varname);
            $_parts = explode('#', $_varname);
            $_pattern = '[$'.$_varname.']';

            $_var = $_parts[0];
            $_type = $this->_varType($_var);

            if ($_type == 'object' || $_type == 'array') {
               $_temp = explode('->', $_var);
               $_final = $this->str2var(${$_temp[0]},$_var);
            }
            else { $_final = isset(${$_var}) ? ${$_var} : ''; }

            if (!empty($_parts[1])) {
               $_func = explode('|', $_parts[1]);
               foreach($_func as $func) if (function_exists($func)) $_final = $func($_final);
            }

            $template = str_replace($_pattern, $_final, $template);

         }
      }

      return $template;

   }

   private function _swapConstants($template) {

      if (preg_match_all('/\[#(.*?)#\]/i', $template, $_const)) {

         foreach ($_const[1] as $i => $_varname) {

            $_pattern = $_const[0][$i];
            $_final = constant($_varname);

            if ($_final) {
               $template = str_replace($_pattern, $_final, $template);
            }

         }
      }

      return $template;

   }

   private function _swapConfig($template, $_vars = null) {

      if (preg_match_all('/<!Conf::(.*?)>/i', $template, $_conf_var)) {

         foreach ($_conf_var[1] as $i => $_varname) {

            $_pattern = $_conf_var[0][$i];
            $_final = App::get($_varname);

            if ($_final) {
               $template = str_replace($_pattern, $_final, $template);
            }

         }
      }

      return $template;

   }

   private function _swapAdvanced($template, $_vars = null) {

      if ($_vars) extract($_vars); $cfg = $this->config;
      if ($cfg['scope']) extract($this->_injectScope());

      preg_match_all('/<!Lang::(.*?)>/', $template, $lng);
      foreach ($lng[1] as $i => $_varname) {
         $_pattern = '<!Lang::'.$_varname.'>';
         $_varname = '[%Lang::get(\''.$_varname.'\')]';
         $template = str_replace($_pattern, $_varname, $template);
      }

      if (preg_match_all('/\[%(.*?)\]/', $template, $_procent_tags)) {

         foreach ($_procent_tags[1] as $i => $_varname) {

            $_pattern = '[%'.$_varname.']';

            if ($_type = $this->_varType($_varname)) {

               $_final = '';

               if ($_type == 'method') {

                  preg_match('/^(\w+)->(\w+)\((.*?)\)$/', $_varname, $_parts);
                  $_class = get_class(${$_parts[1]});

                  if (class_exists($_class) && method_exists($_class, $_parts[2])) {
                     if (empty($_parts[3])) $_final = call_user_func([${$_parts[1]}, $_parts[2]]);
                     else $_final = call_user_func_array([${$_parts[1]},$_parts[2]], $this->_clearParams($_parts[3]));
                  } else { $template = str_replace($_pattern,'',$template); }
               }

               elseif ($_type == 'static_property') {
                  $_parts = explode('::', $_varname);
                  if (class_exists($_parts[0]) && property_exists($_parts[0], ltrim($_parts[1],'$'))) {
                     $_Reflector = new ReflectionClass($_parts[0]);
                     try { $_final = $_Reflector->getStaticPropertyValue( ltrim($_parts[1],'$') ); }
                     catch(Exception $e){ $template = str_replace($_pattern,'',$template); }
                  } else { $template = str_replace($_pattern,'',$template); }
               }

               elseif ($_type == 'static_method') {
                  preg_match('/^(\w+)::(\w+)\((.*?)\)$/is', $_varname, $_parts);
                  if (class_exists($_parts[1]) && is_callable($_parts[1], $_parts[2])) {
                     if (empty($_parts[3])) $_final = call_user_func("$_parts[1]::$_parts[2]");
                     else $_final = call_user_func_array("$_parts[1]::$_parts[2]", $this->_clearParams($_parts[3]));
                  } else { $template = str_replace($_pattern,'',$template); }
               }

               /* Swapping function tags [%function()] */

               elseif ($_type == 'function') {
                  preg_match('/^(\w+)\((.*?)\)$/', $_varname, $_parts);
                  if (function_exists($_parts[1])) {
                     if (empty($_parts[2])) $_final = call_user_func($_parts[1]);
                     else $_final = call_user_func_array($_parts[1], $this->_clearParams($_parts[2]));
                  } else { $template = str_replace($_pattern,'',$template); }
               }

               $template = str_replace($_pattern, $_final, $template);

            }

         }

      }

      return $template;

   }

   private function _swapLoops($template, $_vars = null) {

      if ($_vars) extract($_vars); $cfg = $this->config;
      if ($cfg['scope']) extract($this->_injectScope());

      $isLoop = preg_match_all('/\[@foreach \$(.*?) as \$(.*?)\](.*?)\[@endforeach\]/s', $template, $loops);

      if ($isLoop) {

         $oneSwapped = false;

         for ($i=0; $i < sizeof($loops[0]); $i++) { // Dla każdej pętli z szablonu

            $output = '';

            $loop = (object) [
               'inner' => $loops[3][$i], 'outer' => $loops[0][$i],
               'loop_for' => $loops[1][$i], 'loop_as' => $loops[2][$i]
            ];


            // Fetching tags from loop template
            $_loop_tags = $this->_parseTemplate($loop->inner);

            if (strpos($loop->loop_for, '->')) {
               $_exp = explode('->', $loop->loop_for);
               $array = ${$_exp[0]}->{$_exp[1]};
            } else { $array = ${$loop->loop_for}; }

            foreach ($array as $ii => $row) {

               $_temp = $loop->inner;

               // Replacing [@if ...] tags

               if ( isset($_loop_tags['if']) ) {
                  foreach ($_loop_tags['if'] as $_if) {

							if (strpos($_if->left->name, '->') === false) {
                        $_var = ${$_if->left->name};
							} else {
								$_var = $this->str2var($row, $_if->left->name);
                     }

                     $_swap = ($_if->count == 1 && $_var)
                        ? true : ($_if->count == 3 && $this->compare($_var, $_if->sign, $_if->right->value))
                        ? true : false;

                     if ($_swap && isset($_if->else)) { $_temp = str_replace('[@else]' . $_if->else . '[@endelse]','',$_temp); }
                     // otherwise else tags will be removed by _parseTemplate method, content will stay
                     $_temp = str_replace($_if->tag, ($_swap ? $_if->body : ''), $_temp);

                  }
               }

               if (isset($_loop_tags['ifnot']) && $_loop_tags['ifnot']) {
                  foreach ($_loop_tags['ifnot'] as $_ifnot) {
                     $_var = $this->str2var($row, $_ifnot->left->name);
                     $_swap = (!$_var) ? true : false;
                     $_temp = str_replace($_ifnot->tag, ($_swap ? $_ifnot->body : ''), $_temp);
                  }
               }

               // Replacing [@once ...] tags

               if ( !$oneSwapped && isset($_loop_tags['once']) ) {
                  foreach($_loop_tags['once'] as $_once) {

                     if (isset($_once->count)) {
                        $_var = $this->str2var(${$_once->left->object}, $_once->left->name);
                        if ($_var) $replace = true;
                     } else $replace = true;

                     $temp = isset($replace)
                        ? str_replace($_once->tag, $_once->body, $_temp)
                        : str_replace($_once->tag, '', $_temp);

                  } $oneSwapped = true;
               } else $_temp = preg_replace('/\[@once(.*?)\](.*?)\[@endonce\]/s','',$_temp);


               if ( isset($_loop_tags['extend']) ) {
                  foreach ($_loop_tags['extend'] as $_extend) {

                     $_tpl = $this->_cache($_extend->tpl);

                     $_cmp = isset($_extend->right->value) ? $_extend->right->value : null;
                     $_var = isset($_extend->left) ? $this->str2var($row, $_extend->left->name) : null;

                     $_replace = ($_extend->count == 1)
                        ? true : ($_extend->count == 2 && $_var)
                        ? true : ($_extend->count == 4 && $_var == $_cmp)
                        ? true : false;

                     $_temp = str_replace($_extend->tag, ($_replace ? $_tpl : ''), $_temp);

                  }
               }

               $toSwap = preg_match_all('/\[\$(.*?)\]/i', $_temp, $param); // Finding variables in body

               if ($toSwap) {

                  foreach( $param[1] as $elem ) {

                     $_parts = explode('#', $elem);
                     $_varparts = explode('->', $_parts[0]);

                     if ($_varparts[0] === $loop->loop_as) {

                        $_swap = $this->str2var($row, $_parts[0]);

                        if ( isset($_parts[1]) ) {
                           $_funcs = explode('|', $_parts[1]);
                           foreach($_funcs as $_func) {
                              if (function_exists($_func)) $_swap = $_func($_swap);
                           }
                        }

                        if ($_varparts[1]) $_temp = str_replace('[$'.$elem.']', $_swap, $_temp);
                        $_temp = str_replace('[$ii]', $ii, $_temp);
                        $_temp = str_replace('[$random]', rand(100,999), $_temp);

                     }
                  }

               }

               $output .= str_replace($loop->outer, $_temp, $loop->outer);

            }

            $template = str_replace($loop->outer, $output, $template);
            unset($output);
         } // END $loops

      } return $template;

   } // END _swapLoops

   // =============================================
   // ============= Helpers Functions =============
   // =============================================

   private function _clearParams($params) {
      return array_map(function($e){
         return preg_replace('/[\'|"]/','',$e);
      }, explode(',', $params));
   }

   public function _cache($file) {
      $__tpl__ = &$this->cache['files'][$file];
      if (!isset($__tpl__)) $__tpl__ = $this->loadFile($file);
      return $__tpl__;
   }

   public function loadTemplate($file) {
      $this->template = $this->loadFile($file);
   }

   public function getTags() {
      return $this->_getVars($this->template);
   }

   public function showTemplate() {
      return $this->template;
   }

   private function _getVars($template) {
      if ($template) {
         preg_match_all('/\$[(\w+)->]+/', $template, $params);
         return array_unique(
           array_map(function($var) {
             $e = explode('->', $var);
             return ltrim($e[0],'$');
           }, $params[0])
         );
      } else {
         throw new TemplateException('Can\'t fetch variables from empty template.');
      }
   }

   private function str2var($var, $strvar, $depth = 1) {

    if (empty($strvar)) { return $var; }
		if ($this->validType($var) && strpos($strvar, '->')) {

			$prop = explode('->', $strvar);
			$ii = isset($prop[ $depth ]) ? $depth : 0;

			$var = (gettype($var) == 'object')
				? (isset($var->{$prop[$ii]}) ? $var->{$prop[$ii]} : null)
				: (isset($var[$prop[$ii]]) ? $var[$prop[$ii]] : null);

			if ( $this->validType($var) ) {
				return $this->str2var($var, $strvar, $depth + 1);
			}

			if ($depth > 1 && sizeof($prop) == $depth) {
				return true;
			}

		}

		return $var;

   }

   private function validType($var) {
      return in_array(gettype($var), ['array','object']);
   }

   private function _normalize($var) {
      $var = array_filter(explode(' ', $var));
      return implode('', $var);
   }

   private function _parseTemplate($str) {

      $out = array();
      preg_match_all('/\[@(ifnot|if|once|extend|cant|can)\s?(.*?)\](.*?)\[@end(.*?)\]/s', $str, $matches);

      for ( $i=0 ; $i<sizeof($matches[0]) ; $i++ ) {

         $elem = new stdClass;
         $elem->tag = $matches[0][$i];
         $elem->type = $matches[1][$i];
         $elem->body = $matches[3][$i] ? $matches[3][$i] : null;


         if ($matches[2][$i]) {

            $params = explode(' ', $matches[2][$i]);

            $params = ($params[0] == 'if') ? array_splice($params, 1) : $params;
            $elem->count = sizeof($params);
            switch ($elem->type) {

               case 'if':
               case 'can':
               case 'cant':
               case 'ifnot':
               case 'once':
               case 'extend':

                  // Searching for else
                  $var = preg_quote($matches[2][$i]);
                  preg_match('/\[@if\s'.$var.'\](?:(.*?)\[@endif\]).*?\[@else\](.*?)\[@endelse\]/s', $str, $els);

                  if ($elem->type == 'extend') $elem->tpl = array_pop($params);
                  if ($params) {

                     $stmt = new stdClass;
                     $stmt->left = $params[0];
                     isset($params[2]) && $stmt->right = $params[2];
                     isset($els[2]) && $elem->else = $els[2];

                    if (isset($stmt->left)) {
                        $elem->left = new stdClass;
                        $elem->left->elem = $stmt->left;
                        $elem->left->name = ltrim($stmt->left, '$');
                        $elem->left = (object) array_merge(
                           (array) $elem->left,
                           (array) $this->_parseVar($elem->left->elem)
                        );
                     }

                     if (isset($stmt->right)) {
                        $elem->right = new stdClass;
                        $elem->sign = $params[1];
                        $elem->right->elem = $stmt->right;
                        if (($stmt->right[0] == '$')) {
                           $elem->right->name = ltrim($stmt->right, '$');
                           $elem->right = (object) array_merge(
                              (array) $elem->right,
                              (array) $this->_parseVar($elem->right->elem)
                           );
                        } else {
                          $elem->right->name = ltrim($stmt->right, '$');
                          $elem->right = (object) array_merge(
                              (array) $elem->right,
                              (array) $this->_parseVar($stmt->right)
                           );
                          $elem->right->value = $stmt->right;
                        }
                     }

                  } // IF params

                  break;

            } // END switch

         }

         $out[$elem->type][] = $elem;

      }

      return $out;

   } // END _parse

  private function _parseVar($var) {

     $type = $this->_varType($var);
     $elem = new stdClass;
     $elem->vartype = $type;

     if ($type == 'object') {
        $parts = explode('->', $var);
        $elem->object = ltrim(array_shift($parts), '$');
        $elem->property = implode('->', $parts);
     }
    return $elem;

  }

  private function _varType($var) {

      $is_var = ($var[0] === '$');
      $_var = $is_var ? ltrim($var, '$') : $var;

      if (preg_match('/(\w+)(->\w+){1,}/s', $_var)) return 'object';
      elseif (preg_match('/^(\w+)->(\w+)\((.*?)\)$/', $_var)) return 'method';
      elseif (preg_match('/^(\w+)::\$(\w+)$/', $_var)) return 'static_property';
      elseif (preg_match('/^(\w+)::(\w+)\((.*?)\)$/', $_var)) return 'static_method';
      elseif (preg_match('/^(\w+)\.(\w+)$/', $_var)) return 'array';
      elseif (preg_match('/^(\w+)\((.*?)\)$/', $_var)) return 'function';
      else return $is_var ? 'variable' : 'value';
   }

} // END Class