<?php

class User {

   private $sql;
   private $auth;
   private $user;
   private static $instance;
   
   public $permissions;
   
   public function __construct() {

     self::$instance = $this;

     $this->auth = Factory::get('Auth');
     $this->sql  = Factory::get('Database');
     $this->user = $this->parseUserData();
     
     $bot = new CrawlerDetect\CrawlerDetect;
     $this->user->isbot = $bot->isCrawler();
     $this->permissions = new Permissions($this->sql, $this->user);
     $this->auth->authenticate($this->user);

   }

   private function parseUserData() {

      $sess_id    = session_id();
      $user_ip    = $_SERVER['REMOTE_ADDR'];
      $user_agent = $_SERVER['HTTP_USER_AGENT'];
      $user_hash  = $sess_id . $user_ip;

      $user = [
         'user_id' => 0,
         'group_id' => 1,
         'logged' => false,
         'isadmin'  => false,
         'agent'  => $user_agent,
         'ip' => $user_ip,
         'hash'   => hash('crc32b', $user_hash),
         'iphash' => hash('crc32b', $user_ip)
      ];

      if ($guid = $this->auth->is_Logged()) {
         $user = (object) array_merge($user, $guid, ['logged' => true]);
         $this->auth->is_Admin( $user ) && $user->isadmin = true;
      }

      return (object) $user;

   }

   public function __get($key) {
       return $this->user->$key;
   }

   public function getGroupnameById($id) {
      switch ($id) {
         case 1: return 'Guest';
         case 2: return 'Member';
         case 3: return 'Moderator';
         case 4: return 'Administrator';
      }
   }

   public function getData() {
      return $this->user;
   }
   
   public function userSocial($obj) {
      $out = '';
      if ($obj['user_fburl']) { $out .= '<a target="_blank" href="'.($obj['user_fburl']).'"><i class="icon-facebook"></i></a>'; } else { $out .= '<i class="icon-facebook"></i>'; }
      if ($obj['user_twurl']) { $out .= '<a target="_blank" href="'.($obj['user_twurl']).'"><i class="icon-twitter"></i></a>'; } else { $out .= '<i class="icon-twitter"></i>'; }
      if ($obj['user_gpurl']) { $out .= '<a target="_blank" href="'.($obj['user_gpurl']).'"><i class="icon-google-plus"></i></a>'; } else { $out .= '<i class="icon-google-plus"></i>'; }
      if ($obj['user_pturl']) { $out .= '<a target="_blank" href="'.($obj['user_pturl']).'"><i class="icon-pinterest"></i></a>'; } else { $out .= '<i class="icon-pinterest"></i>'; }
      if ($obj['user_yturl']) { $out .= '<a target="_blank" href="'.($obj['user_yturl']).'"><i class="icon-youtube"></i></a>'; } else { $out .= '<i class="icon-youtube"></i>'; }

      return $out;

   }
   
   private function getPoints($obj) {
      return round(
         $obj['comments'] * App::get('points_per_comment')
       + $obj['sumlikes'] * App::get('points_per_likes')
       + $obj['sumfav']   * App::get('points_per_favs')
       + $obj['posts']    * App::get('points_per_post')
       + $obj['views']    * App::get('points_per_view')
      );
   }
   
   private function parseTime($obj, $mode) {
      return ($mode == 'regdate') 
         ? App::zmienczas($obj['regdate'])
         : App::zmienczas($obj['lastlogin'], true);
   }
   
   private function userBio($obj) {
      return $obj['user_bio']
         ? '<div class="isdesc">'.$obj['user_bio'].'</div>'
         : '<div class="nodesc">'.Lang::get('no_description').'</div>';
   }
   
   private function getFollowStatus($user) {
      $st = $this->sql->countRows('follow', 'kto=? and kogo=?', [self::get('username'), $user]);
      return (!$st or !self::Logged()) 
         ? '<a id="follow" class="btn darkgrey" data-who="'.($user).'" href="#"><i class="icon-plus"></i> ' .Lang::get('follow'). '</a>'
         : '<a id="follow" class="btn darkgrey" data-who="'.($user).'" href="#"><i class="icon-cross"></i> ' .Lang::get('unfollow'). '</a>';
   }
   
   private function getStatus($user) {
      if ($user["ban"]) return 'Banned';
      if ($user["activated"]) return 'Activated';
      return 'Not activated';
   }
   
   public function getSubscribers($user) {
      $out = ''; $users = $this->sql->where('kogo', $user)->get('follow',10,'kto');
      foreach ($users as $u) {
         $avatar = App::getAvatar($u['kto']);
         $out .= '<a href="profile'. SR . $u['kto'] .'"><img src="'.$avatar.'"></a>';
      }; return $out ?: '<div class="no_followers">'.Lang::get('no_followers').'</div>';
   }
   
   public function getProfileBg($user) {
      if (file_exists(PATH_ROOT . '/Uploads/Avatars/'.$user.'-bg.jpg')) {
         return 'Uploads/Avatars/'.$user.'-bg.jpg';
      }
      return 'Uploads/Avatars/profile-bg.jpg';
   }
   
   public function userData($username) {

      if (!$this->is_User($username)) {
         return false;
      }

      $db = $this->sqlData($username);
      $stats = $this->getStats($username);
      
      if ($this->sql->count) {

         $data = [
            'profile_bg' => $this->getProfileBg($username),
            'acc_status' => $this->getStatus($db),
            'subscribers' => $this->getSubscribers($username),
            'social' => $this->userSocial($db), 'points' => $this->getPoints($stats),
            'avatar' => App::getAvatar($db['username']), 'desc' => $this->userBio($db),
            'username' => $db['username'], 'name' => mb_ucfirst($db['user_name'] ?: $db["username"]), 
            'flbtn' => $this->getFollowStatus($db['username']), 'regdate' => $this->parseTime($db,'regdate'),
            'lastlogin' => $db['lastlogin'] ? $this->parseTime($db, 'status') : 'Never', 'views' => $stats['views'], 
            'row' => ['lastlogin' => $db['lastlogin'], 'regdate' => $db['regdate']],
         ];
         
         return (object) array_merge(
            (array) $db, 
            (array) $stats, 
            (array) $data
         );

      }

      return false;

   }
   
   public function sqlData($user = null, $asObject = false) {
      $user = $user ?: Session::get('username');
      $data = $this->sql->rawQueryOne("
         SELECT id as uid, username, email, lastlogin, ban, activated, regdate, user_bio, user_name, user_email,
         user_blog, user_fburl, user_twurl, user_gpurl, user_pturl, user_yturl
         FROM login WHERE username = ?", [$user]);
      return $asObject ? (object) $data : $data;
   }
   
   public function getStats($user = null, $asObject = false) {

      $user = $user ?: $this->user->username;
		for ($i = 0; $i < 10; $i++) { $bind[] = $user; }

      $data = $this->sql->rawQueryOne("SELECT
         (SELECT SUM(views) FROM posts where user=? and logged = 1) AS views,                  # User's posts views
         (SELECT SUM(likes) FROM posts where user=? and logged = 1) AS sumlikes,               # Upvotes received
         (SELECT SUM(fav) FROM posts where user=? and logged = 1) AS sumfav,                   # Favs received
         (SELECT COUNT(id) FROM follow WHERE kto=?) AS follow,                                 # Following count
         (SELECT COUNT(id) FROM follow WHERE kogo=?) AS followby,                              # Followed by count
         (SELECT COUNT(id) FROM comments WHERE user=?) AS comments,                            # Added comments
         (SELECT COUNT(id) FROM favorites WHERE username=? and action='fav') AS favs,          # Posts added to fav
         (SELECT COUNT(id) FROM favorites WHERE username=? and action='likes') AS likes,       # Liked posts
         (SELECT COUNT(id) FROM posts WHERE user=? and status=2) AS posts,                     # Liked posts
			(SELECT COUNT(id) FROM posts WHERE user=? and status=2) AS posts LIMIT 1", 
		$bind);

		return $asObject ? (object) $data : $data;

   }

   public function is_User($user) {
      return $this->sql->where('username', $user)
            ->getValue('login','count(id)');
   }

   // Routing static requests
   public static function __callStatic($func, $args) {

      $self = self::$instance;
      $func = strtolower($func);

      switch ($func) {
         case 'get':        return $self->user->{$args[0]};
         case 'can':        return $self->permissions->get($args[0]);
         case 'cant':       return !$self->permissions->get($args[0]);
         case 'logout':     return $self->auth->Logout();
         case 'userexists': return $self->is_User($args[0]);
         default:           return $self->user->{$func};
      }

	}
   
}