<?php

class UserPanel extends Panels {
  
   private $data;
   private $user;
   
   public function __construct() {
      parent::__construct();
      $this->user = Factory::get('User');
      $this->data = new stdClass();
    }
   
   public function GetPageData() {
      switch ($this->page) {
         case 'inbox': case 'msg_send': return $this->Inbox();
         case 'notescfg': return $this->NotificationsConfig();
         case 'notes': return $this->Notifications();
         case 'changepass': return $this->ChangePassForm();
         case 'edit': return $this->AccountSettings();
         case 'favorites': return $this->_get_favorites();
         case 'posts': return (isset($this->get->edit)) 
            ? $this->EditPost() : $this->GetPostsList();
      }
   }
   
   private function inbox() {
      
      $_opt = $this->get; 
      $_data = $this->data;
      $_msg = Factory::get('Messages');
      
      $_data->title = Lang::get('messages');
      $_data->template = 'Panel/' . $this->page;

      if ($this->page == "inbox") {

         $messages = $_msg->getMessages($_opt->view, $this->get->page);
         $total = $this->sql->totalPages ?: 1;
         
         $var = [
            'href' => 'profil?inbox&view=' . $_opt->view,
            'next' => (int) $this->get->page + 1,
            'prev' => (int) $this->get->page - 1,
            'current' => abs($this->get->page + 1),
            'page' => (int) $this->get->page
          ];

          $tpl = new Template('panel_paging', ['scope' => false]);
          $_data->paging = $tpl->render( compact($tpl->getTags()) );

         array_map(function($e) use (&$_opt) {
            $e->time = App::zmienczas($e->time);
            $e->message = strip_tags($e->message);
            $e->icon = $e->rcv ? 'folder-open-o' : 'folder-o';
            $e->self = intval($e->sender == User::get("username"));
            $e->mode = $_opt->view == "trash" ? 'delete' : 'trash';
            $e->avatar = App::getAvatar($_opt->view != "sended" ? $e->sender : $e->receiver);
            $e->message = strlen($e->message) > 180 ? substr($e->message, 0, 180) . '...' : $e->message;

         }, $messages);
         
         $sd = $_opt->view == "sended";
         $tpl = new Template($sd ? 'Panel/msg_sent_loop' : 'Panel/msg_loop');
         $out = $tpl->render( compact($tpl->getTags()) );
         
         $_data->items = $out ?: '<div class="msg-none">'.Lang::get('msg_none').'</div>';
         
      } elseif ($this->page == "msg_send") {
         
         $_data->title = Lang::get('msg_send');
         isset($_opt->to) && $_data->receiver = Filter::_String($_opt->to);
         isset($_opt->id) && $_data->rdata = $_msg->getMessage($_opt->id);
         
         if ( isset($_data->rdata) ) {
            $_msg->setSeen($_data->rdata->id);
            $_data->self = intval($_opt->to == User::get("username"));
            $_data->rdata->re_topic = 'Re: ' . mb_ucfirst($_data->rdata->topic, "utf-8");
            $rp_tpl = new Template('Panel/reply_msg');
            $_data->reply = $rp_tpl->render( compact($rp_tpl->getTags()) );
         }
         
      }
      
      return $_data;
      
   }
   
   private function EditPost() {
      
      $id = (int) $_GET['edit'];
      
      $_data = $this->data;
      $_data->catlist = App::getCategories('form');
      $_data->template = 'Panel/post_edit';
      $_data->post = $this->getPost($id);
      $_data->title = Lang::get('editing_post', ['title' => htmlentities($_data->post->tytul)]);
      $_data->category = $_data->post->category ?: Lang::get('catselect');
      $_data->post->czas = App::zmienczas($_data->post->czas);
      
      if (!$this->sql->count) App::Redirect();
      
      return $_data;
      
   }
  
  protected function getPost($id) {
      return $this->sql->ObjectBuilder()
         ->where('user=? and id=?', [$this->nick, $id])->getOne('posts');
   }
  
  public function GetUserData() {
      $_data = $this->data;
      $_user = $this->sql->where('username', $this->nick)->getOne('login','username as user, user_name as nick');
      $_data = (object) array_merge($_user, $this->user->getStats()); $_data->avatar = App::getAvatar($this->nick);
      return $_data;
   }
   
   private function GetPostsList() {
     return $this->PostList();
   }
   
   private function _get_favorites() {
      
      $get = $this->get;
      
      $_data = $this->data;
      $_data->title = Lang::get('favorites');
      
      ($get->mode == 'img') ? ($tab = 'photos') && ($item = 'photo') : ($tab = 'posts') && ($item = 'post');
      ($get->sort == 'fav') ? ($action = 'fav') : ($action = 'likes');

      ($item == 'photo') ? ( ($action=='fav') ? $_data->a2 = 'active' : $_data->a1 = 'active' ) : 
                           ( ($action=='fav') ? $_data->a4 = 'active' : $_data->a3 = 'active' );

      $results = $this->sql->orderBy('pid','desc')
         ->where('username', $this->nick)->where('item',$item)
         ->where('action',$action)->get('favorites');

      if ($results) {
         foreach ($results as $out) { 
            $favTab[] = $out['pid']; 
         } 
         $ids = implode(',',$favTab);
      }

      // GDY BRAK WYNIKÓW

      if (!empty($ids)) {
         
         $_data->items = '';
         $imgTpl = new Template('Panel/panel_fav_img');
         $postTpl = new Template('Panel/panel_fav_post');
         
         $results = $this->sql->ObjectBuilder()->rawQuery("select * from $tab where id IN ($ids)");
         foreach ($results as $pic) {
            if ($get->mode=='img') {
               $imgData = App::postImage($pic->pid);
               $_data->items .= $imgTpl->render( compact($imgTpl->getTags()), false );
            } else {
               $imgData = App::postImage($pic); $czas = App::unixTimeArray($pic->czas);
               $postLink = App::postLink($pic);
               $_data->items .= $postTpl->render( compact($postTpl->getTags()), false );
            }
         }
      } else {
         $_data->empty = 'empty-list';
         $_data->items = '<div id="empty">'.Lang::get('nothing_found').'</div>';
      }
      
      $_data->template = 'Panel/panel_fav';
      
      return $_data;
   }
   
   private function Notifications() {

      $get = $this->get;
      $Notes = Factory::get('Notes');
      
      $_data = (object) [
         'template' => 'Panel/panel_notes',
         'title' => Lang::get('notifications')
      ];
      
      if (isset($get->mode) && $get->mode == 'setseen') { 
         $Notes->setSeen(); header('Location: profil?notes'); 
      }

      if (isset($get->view)) { switch ($get->view) {
         case '': $_data->a1 = 'active'; break;
         case 'all': $_data->a2 = 'active'; break;
         case 'votes': $_data->a3 = 'active'; break;
         case 'comments': $_data->a4 = 'active'; break;
      }}
      
      $page = (int) (isset($get->page) ? $get->page : 0);
      $on_page = 20; $currPage = $page + 1;

      $_data->smallnotes = isset($get->view)
         ? $Notes->smallNotes($get->view, $on_page, $page)
         : $Notes->smallNotes('summary', $on_page, $page);

     $pages = ceil($this->sql->lastNum / $on_page) ?: 1;
     
     $_data->paging = isset($get->page)
        ? '<a class="page btn btn-small blue dcell w10p" style="margin:2px;" href="profil?notes&page='.($this->get->page-1).'">'.Lang::get('prev_page').'</a>'
        : '<div class="dcell w10p"></div>';
     $_data->paging .= '<span style="line-height:30px">'.Lang::get('page').' ' . $currPage . ' '.Lang::get('of').' ' . $pages . '</span>';
     $_data->paging .= ($currPage < $pages)
        ? '<a class="page btn btn-small blue dcell w10p" style="margin:2px;" href="profil?notes&page='.($this->get->page+1).'">'.Lang::get('next_page').'</a>'
        : '<div class="dcell w10p"></div>';
     
      return $_data;

   }
   
   private function AccountSettings() {
      $_data = $this->data;
      $_data = $this->sql->ObjectBuilder()->where('username', $this->nick)
         ->getOne('login','user_name,user_email,user_blog,user_bio,
           user_fburl,user_twurl,user_gpurl,user_pturl,user_yturl');
      $_data->title = Lang::get('menu_4');
      $_data->template = 'Panel/panel_edit';
      return $_data;
   }
   
   private function ChangePassForm() {
      $_data = $this->data;
      $_data->title = Lang::get('change_pass_head');
      $_data->template = 'Panel/panel_passform';
      return $_data;
   }
      
   private function NotificationsConfig() {
      
      $_data = $this->data;
      
      $isConfig = $this->sql->countRows('notes_config','user=?', [$this->nick]);
      if (!$isConfig) { $this->sql->insert('notes_config', ['user' => $this->nick]); }
      
      $_cfg = $this->sql->where('user', $this->nick)->getOne('notes_config');
      foreach ($_cfg as $_opt => $_val) { $_data->$_opt = $_val ? 'checked' : ''; }
      
      $_data->template = 'Panel/panel_notecfg'; 
      $_data->title = Lang::get('notes_settings');
      
      return $_data;
      
   }
   
   private function _setNotesConfig() {
      $this->sql->insert('notes_config',['user' => $this->nick]); 
      $this->notescfg = $this->sql->where('user', $this->nick)->getOne('notes_config');
   }
   
}