<?php

class Utils extends AppController {

   public function __construct() {
      parent::__construct();
   }

   public function array2obj($array, $json = false) {
      $object = (object) array_map( function($value) {
         return trim(strip_tags($value));
      }, $array);
      return $json ? json_encode($object) : $object;
   }

   public function isAJAX() {
      $req = $_SERVER['HTTP_X_REQUESTED_WITH'];
      return (!empty($req) && strtolower($req) == 'xmlhttprequest');
   }

   public function object_map($fnc, $object) {
      $vars = get_object_vars( $object );
      foreach ($vars as $name => $val) {
         $object->$name = $fnc( $val );
      } return $object;
   }

   public function getPages($render = 'sidemenu', $str = '') {

      $result = $this->sql->get('custom_page', null, 'id,name,title');

      foreach ($result as $page) {
         if ($render == 'sidemenu') {
            $title = $this->urlPrepare($page['title']);
            $str .= '<li><a href="page/' . $title . '">
            <span class="sidenav-link-icon"><i class="icon-eye"></i></span>
            <span class="sidenav-link-title">' . mb_ucfirst($page['title']) . '</span>
            </a></li>';
         }
      }

      return $str;

   }

   public function printJSON($status, $message = null) {
      if (is_array($status)) $json = $status;
      else $json = ['status' => $status, 'message' => $message];
      return json_encode($json, JSON_UNESCAPED_UNICODE);
   }

   public function getFile() {
      return basename($_SERVER['SCRIPT_FILENAME']);
   }

   public function getBrand($type,$mode,$site,$v2 = false) {

      switch ($site) {
         case 'vimeo': return $v2 ? 'vimeo1' : 'vimeo2';
         case 'dailymotion': return 'dailymotion2';
         case 'ninegag': return '9gag';
         case 'metacafe': return 'metacafe';
         case 'youtube': return $v2 ? 'youtube2' : 'youtube';
         case 'facebook': return $v2 ? 'facebook' : 'facebook2';
         case 'vk': return 'vk';
         case 'instagram': return 'instagram';
         case 'soundcloud': case 'mp3': return 'headphones';
      }

      if ($type == 'article') return 'newspaper';
      if ($type == "photos") return 'images';
      if ($type == "video" && $v2) return 'film';

   }

  public function topbarStatic() {
      $page = ['upload.php','panel.php'];
      if (in_array($this->getFile(), $page)) {
         echo 'static';
      }
   }

   public function fakeName($user) {
      if (User::Logged()) { 
         $result = $this->sql->where('username', $user)->getOne('login','user_name');
         return $result['user_name'] ?: ucfirst($user);
      } return ucfirst($user);
   }

   public function getUrlParams() {

       $_get = array();
       $rUri = $_SERVER['REQUEST_URI'];
       $rQue = $_SERVER['QUERY_STRING'];

       $prm = $rQue . '&' . substr($rUri, strpos($rUri, '?') + 1, strlen($rUri));

         if ($prm && strstr($prm, '=')) {

            $_exParams = array_unique(explode('&', $prm));

            foreach ($_exParams as $_pairs) {

              $_exData = explode('=',$_pairs);

               if (sizeof($_exData) == 2) { $_get[$_exData[0]] = $_exData[1]; }
               elseif(sizeof($_exData) > 2) {
                  $base = $_exData[0];
                  array_shift($_exData);
                  $_get[$base] = join('=',$_exData);
               }

            }

           return $this->array2obj($_get);

         }

     return new stdClass;

   }

   public function getCatIcon($id) {

      $path = 'Uploads/Icons';
      $image = PATH_ROOT . "/$path/cat-$id.png";
      return file_exists($image) ? "$path/cat-$id.png" : $path . '/rsz_empty.jpg';

   }

   public function getCategories($render = 'sidemenu', $icons = false) {
      $str = ''; $result = $this->sql->get('categories');
      foreach ($result as $cat) {

         $line = $this->addLine($cat['name']);
         $name = mb_ucfirst($cat['name']);

         switch ($render) {
            case 'form': $str .= "<li>$name</li>"; break;
            case 'sidemenu': $str .= "<li><a href=\"category" . SR . $line . "\">$name</a></li>"; break;
            case 'icons': $str .= "<li><a href=\"category" . SR . $line . "\"><img src='".$this->getCatIcon($cat["id"])."'> $name</a></li>";
         }

      }; return $str;
   }

   public function parseCategory($name) {
      return empty($name) || !is_string($name) ? "" : [
			'name' => $name, 'ucname' => mb_ucwords($name),
			'url_relative' => 'category' . SR . App::addLine($name),
			'url_http' => App::get('app_url') . '/category' . SR . App::addLine($name)
      ];
   }

	public function removeDir($path) {

		if (file_exists($path) && is_dir($path)) {

			$currentMask = umask(0);
			$dir = new DirectoryIterator($path);

			foreach ($dir as $item) {
				if ($item->isFile()) {
					$file = $item->getRealPath();
					@chmod($file, 0666); @unlink($file);
				} elseif (!$item->isDot() && $item->isDir()) {
					$this->removeDir($item->getRealPath());
				}
			}

			@chmod($path, 0777); @rmdir($path);
			umask($currentMask);

		}

	}

   public function hideIfLogged () {
      if (User::Logged()) return 'hide';
   }

   public function createTempdir() {
     if (!is_dir(PATH_TEMP)) {
		   $oldmask = umask(0);
		   mkdir(PATH_TEMP, 0777, true);
		   umask($oldmask);
     }
   }

   public function getMediaData($url) {
      $this->createTempdir();
      $data = $this->mediaData($url);
      file_put_contents(PATH_TEMP.'/mediadata.json', $data);
      chmod_u(PATH_TEMP.'/mediadata.json', 0666);
      return $data;
   }

   public function replaceAccentedChars($str) {

       $patterns = array(
           /* Lowercase */
           /* a  */ '/[\x{00E0}\x{00E1}\x{00E2}\x{00E3}\x{00E4}\x{00E5}\x{0101}\x{0103}\x{0105}\x{0430}\x{00C0}-\x{00C3}\x{1EA0}-\x{1EB7}]/u',
           /* b  */ '/[\x{0431}]/u',
           /* c  */ '/[\x{00E7}\x{0107}\x{0109}\x{010D}\x{0446}]/u',
           /* d  */ '/[\x{010F}\x{0111}\x{0434}\x{0110}\x{00F0}]/u',
           /* e  */ '/[\x{00E8}\x{00E9}\x{00EA}\x{00EB}\x{0113}\x{0115}\x{0117}\x{0119}\x{011B}\x{0435}\x{044D}\x{00C8}-\x{00CA}\x{1EB8}-\x{1EC7}]/u',
           /* f  */ '/[\x{0444}]/u',
           /* g  */ '/[\x{011F}\x{0121}\x{0123}\x{0433}\x{0491}]/u',
           /* h  */ '/[\x{0125}\x{0127}]/u',
           /* i  */ '/[\x{00EC}\x{00ED}\x{00EE}\x{00EF}\x{0129}\x{012B}\x{012D}\x{012F}\x{0131}\x{0438}\x{0456}\x{00CC}\x{00CD}\x{1EC8}-\x{1ECB}\x{0128}]/u',
           /* j  */ '/[\x{0135}\x{0439}]/u',
           /* k  */ '/[\x{0137}\x{0138}\x{043A}]/u',
           /* l  */ '/[\x{013A}\x{013C}\x{013E}\x{0140}\x{0142}\x{043B}]/u',
           /* m  */ '/[\x{043C}]/u',
           /* n  */ '/[\x{00F1}\x{0144}\x{0146}\x{0148}\x{0149}\x{014B}\x{043D}]/u',
           /* o  */ '/[\x{00F2}\x{00F3}\x{00F4}\x{00F5}\x{00F6}\x{00F8}\x{014D}\x{014F}\x{0151}\x{043E}\x{00D2}-\x{00D5}\x{01A0}\x{01A1}\x{1ECC}-\x{1EE3}]/u',
           /* p  */ '/[\x{043F}]/u',
           /* r  */ '/[\x{0155}\x{0157}\x{0159}\x{0440}]/u',
           /* s  */ '/[\x{015B}\x{015D}\x{015F}\x{0161}\x{0441}]/u',
           /* ss */ '/[\x{00DF}]/u',
           /* t  */ '/[\x{0163}\x{0165}\x{0167}\x{0442}]/u',
           /* u  */ '/[\x{00F9}\x{00FA}\x{00FB}\x{00FC}\x{0169}\x{016B}\x{016D}\x{016F}\x{0171}\x{0173}\x{0443}\x{00D9}-\x{00DA}\x{0168}\x{01AF}\x{01B0}\x{1EE4}-\x{1EF1}]/u',
           /* v  */ '/[\x{0432}]/u',
           /* w  */ '/[\x{0175}]/u',
           /* y  */ '/[\x{00FF}\x{0177}\x{00FD}\x{044B}\x{1EF2}-\x{1EF9}\x{00DD}]/u',
           /* z  */ '/[\x{017A}\x{017C}\x{017E}\x{0437}]/u',
           /* ae */ '/[\x{00E6}]/u',
           /* ch */ '/[\x{0447}]/u',
           /* kh */ '/[\x{0445}]/u',
           /* oe */ '/[\x{0153}]/u',
           /* sh */ '/[\x{0448}]/u',
           /* shh*/ '/[\x{0449}]/u',
           /* ya */ '/[\x{044F}]/u',
           /* ye */ '/[\x{0454}]/u',
           /* yi */ '/[\x{0457}]/u',
           /* yo */ '/[\x{0451}]/u',
           /* yu */ '/[\x{044E}]/u',
           /* zh */ '/[\x{0436}]/u',
           /* Uppercase */
           /* A  */ '/[\x{0100}\x{0102}\x{0104}\x{00C0}\x{00C1}\x{00C2}\x{00C3}\x{00C4}\x{00C5}\x{0410}]/u',
           /* B  */ '/[\x{0411}]/u',
           /* C  */ '/[\x{00C7}\x{0106}\x{0108}\x{010A}\x{010C}\x{0426}]/u',
           /* D  */ '/[\x{010E}\x{0110}\x{0414}\x{00D0}]/u',
           /* E  */ '/[\x{00C8}\x{00C9}\x{00CA}\x{00CB}\x{0112}\x{0114}\x{0116}\x{0118}\x{011A}\x{0415}\x{042D}]/u',
           /* F  */ '/[\x{0424}]/u',
           /* G  */ '/[\x{011C}\x{011E}\x{0120}\x{0122}\x{0413}\x{0490}]/u',
           /* H  */ '/[\x{0124}\x{0126}]/u',
           /* I  */ '/[\x{0128}\x{012A}\x{012C}\x{012E}\x{0130}\x{0418}\x{0406}]/u',
           /* J  */ '/[\x{0134}\x{0419}]/u',
           /* K  */ '/[\x{0136}\x{041A}]/u',
           /* L  */ '/[\x{0139}\x{013B}\x{013D}\x{0139}\x{0141}\x{041B}]/u',
           /* M  */ '/[\x{041C}]/u',
           /* N  */ '/[\x{00D1}\x{0143}\x{0145}\x{0147}\x{014A}\x{041D}]/u',
           /* O  */ '/[\x{00D3}\x{014C}\x{014E}\x{0150}\x{041E}\x{00D6}]/u',
           /* P  */ '/[\x{041F}]/u',
           /* R  */ '/[\x{0154}\x{0156}\x{0158}\x{0420}]/u',
           /* S  */ '/[\x{015A}\x{015C}\x{015E}\x{0160}\x{0421}]/u',
           /* T  */ '/[\x{0162}\x{0164}\x{0166}\x{0422}]/u',
           /* U  */ '/[\x{00D9}\x{00DA}\x{00DB}\x{00DC}\x{0168}\x{016A}\x{016C}\x{016E}\x{0170}\x{0172}\x{0423}]/u',
           /* V  */ '/[\x{0412}]/u',
           /* W  */ '/[\x{0174}]/u',
           /* Y  */ '/[\x{0176}\x{042B}]/u',
           /* Z  */ '/[\x{0179}\x{017B}\x{017D}\x{0417}]/u',
           /* AE */ '/[\x{00C6}]/u',
           /* CH */ '/[\x{0427}]/u',
           /* KH */ '/[\x{0425}]/u',
           /* OE */ '/[\x{0152}]/u',
           /* SH */ '/[\x{0428}]/u',
           /* SHH*/ '/[\x{0429}]/u',
           /* YA */ '/[\x{042F}]/u',
           /* YE */ '/[\x{0404}]/u',
           /* YI */ '/[\x{0407}]/u',
           /* YO */ '/[\x{0401}]/u',
           /* YU */ '/[\x{042E}]/u',
           /* ZH */ '/[\x{0416}]/u');
           // ö to oe
           // å to aa
           // ä to ae

       $replacements = array(
               'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'r', 's', 'ss', 't', 'u', 'v', 'w', 'y', 'z', 'ae', 'ch', 'kh', 'oe', 'sh', 'shh', 'ya', 'ye', 'yi', 'yo', 'yu', 'zh',
               'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'R', 'S', 'T', 'U', 'V', 'W', 'Y', 'Z', 'AE', 'CH', 'KH', 'OE', 'SH', 'SHH', 'YA', 'YE', 'YI', 'YO', 'YU', 'ZH'
           );

       return preg_replace($patterns, $replacements, $str);

   }

   public function urlPrepare($string) {
      $string = $this->replaceAccentedChars($string);
      $string = mb_strtolower(preg_replace("/(\s|-){1,}/",' ',$string),"utf-8");
      $string = preg_replace('/[^\p{L}a-z0-9 ]/iu','',$string);
      return str_replace(' ','-',$string);

   }

   public function sqlEsc($var){
       return $this->sqlPrepare($var);
   }

   public function addLine($str) {
      $str = mb_strtolower(trim($str), 'utf-8');
      return rawurlencode(str_replace(' ', '-', $str));
   }

   public function remLine($str) {
      return str_replace('-',' ',$str);
   }

   public function sqlPrepare($var) {
      return $this->sql->escape($var);
   }

   public function getSlider() {

      if ($pop = $this->sql->popular("WEEK")) {
         $pop = 'and ' . $pop;
      }

      $query = "select * from posts where status='2' order by rand() limit 0,15";
      $result = $this->sql->ObjectBuilder()->rawQuery($query);
      $str = '';
      foreach($result as $item) {
         $image = $this->postImage($item);
          $url = $this->postLink($item);
          $str .= '<div><a href=""><img src="'.$image->image.'"></a></div>';
      }
      return $str;
   }

   public function topCategories() {
      $results = $this->sql->get('categories');
      $out = '<div id="top-cat-wrapper"><ul id="top-cat-menu">';
      foreach ($results as $cat) {
         $num = $this->sql->countRows('posts','category=?',$cat["name"]);
         $icon = $this->getCatIcon( $cat["id"] );
         $out .= '<li ripple="ripple"><a href="category' . SR . $this->addLine($cat['name']).'"><img src="'.$icon.'">'.$cat['name'].'<span>'.($num).'</span></a></li>';
      }; $out .= '</ul></div>';

      return $out;

   }

   public function getJson($path) {
      $jsonFile = "Uploads/Media/$path/mediadata.json";
      return file_get_contents($jsonFile);
   }

   public function getFromJson($json, $elem = false) {
      if ($json = json_decode($json)) {
         return $elem && isset($json->{$elem}) ? $json->{$elem} : $json;
      }
   }

   public function postImage($wpis, $return = 'object') {

      if (gettype($wpis) != 'object') {
         $wpis = $this->sql
            ->ObjectBuilder()->where('id',$wpis)
            ->getOne('posts',['dir','type','hash','mediadata']);
      }

      $wpis->source = $this->getFromJson($wpis->mediadata, 'source');

      $path = 'Uploads/Media/'.$wpis->dir.'/';
      $Data['image'] = $path . $wpis->hash.'.jpg';
      $Data['thumb'] = $path . 'm_' . $wpis->hash.'.jpg';

      if ($return == 'object') return $this->array2obj($Data);
      return $Data;

   }

   public function more_posts($id) {

      $db = $this->sql;

      $prevTab = $db->where('id',$id,'<')->where('status', 2)
         ->orderBy('id','desc')->getOne('posts',['id','tytul','type','category']);

      if ($db->count) {
         $prevTab['url'] = $this->postLink($prevTab);
         $img = $this->postImage($prevTab['id']); $prevTab['thumb'] = $img->thumb;
      }

      $nextTab = $db->where('id',$id,'>')->orderBy('id','asc')
         ->getOne('posts',['id','tytul','type','category']);

      if ($db->count) {
         $nextTab['url'] = $this->postLink($nextTab);
         $img = $this->postImage($nextTab['id']); $nextTab['thumb'] = $img->thumb;
      }

      if ($prevTab || $nextTab) {
         $_out = new stdClass();
         $_out->prev = $prevTab ?: null;
         $_out->next = $nextTab ?: null;
         return $_out;
      } else {
         return false;
      }

   }

   public function tagList($tags) {

      if ($tags) {
         $E = explode(',',$tags);
         foreach($E as $i => $tag) {
            $R[] = '<a class="bluetag" href="tag' . SR . $this->addLine($tag).'">'.$tag.'</a>';
         }
         return '<div class="post-tags">' . implode(' ', $R) . '</div>';
      } else { ''; }

   }

   public function trending($num) {

      if ($pop = $this->sql->popular("WEEK")) {
         $pop = 'and ' . $pop;
      }

      $query = "select * from posts where status='2' {$pop} limit 0,$num";
      $result = $this->sql->ObjectBuilder()->rawQuery($query);

      if ($this->sql->count) {
         foreach ($result as $wpis) {

            $postLink = $this->postLink($wpis);
            $imgData = $this->postImage($wpis);
            $czas = $this->unixTimeArray($wpis->czas);

            echo '<div class="margin-top-10 trending-post bd">';
            if ($wpis->category) { echo '<a class="cat-tag" href="category' . SR . $this->addLine($wpis->category).'">' . mb_ucwords($wpis->category) . '</a>'; }
            echo '<a class="img-link" href="'.$postLink.'">
               <img alt="'.htmlspecialchars($wpis->tytul).'" src="'.$imgData->image.'">
               <h4>'.$wpis->tytul.'</h4>
            </a>
            <div class="authoricons">
               '.Lang::get('by_sidebar').' '.$this->fakeName($wpis->user).' '.$czas['day'].' '.$czas['month'].'
               <div class="right"><i class="icon-eye"></i> '.$wpis->views.' <i class="icon-speech-bubble"></i> '.($wpis->comments).'</div>
            </div>
         </div>';
         }
      }

   }

   public function smallList($start,$limit) {

      $result = $this->sql->ObjectBuilder()
         ->orderBy('id','desc')->where('status', '2')
         ->get('posts',[$start * $limit, $limit], ['id','tytul','dir','hash','czas','user','category','type']);

      foreach ($result as $item ) {

         $link = $this->postLink($item);
         $thumb = 'Uploads/Media/'.$item->dir.'/m_'.$item->hash.'.jpg';

         echo '<div class="stick-item" data-from="'.$start.'">
               <div class="stick-img">
                  <a class="clearfix" href="'.$link.'">
                     <img alt="'.htmlspecialchars($item->tytul).'" src="'.$thumb.'">
                  </a>
               </div>
               <div class="stick-title">
                  <h5><a class="clearfix" href="'.$link.'">'.$item->tytul.'</a></h5>
                  <div class="stick-info"><i class="icon-user"></i>'.$this->fakeName($item->user).' <i class="icon-clock"></i>'.$this->zmienczas($item->czas).'</div>
               </div>
            ';
         echo '</div>';
      }
   }

   function pagination($num, $total) {

      $_get = $this->getUrlParams();
      $pages = ceil($total / $num);

      $current = intval($_get->page);
      $next = $current + 1;
      $prev = $current - 1;
      $add = ''; $stages = 2;

      $iLeft = '<i class="icon-angle-double-left"></i>';
      $iRight = '<i class="icon-angle-double-right"></i>';

      isset($_get->sort) && $add = '?sort='.$_get->sort;
      isset($_get->tagi) && $target = 'tag' . SR . $this->addLine($_get->tagi);
      isset($_get->category) && $target = 'category' . SR . $this->addLine($_get->category);
      isset($_get->search) && $target = 'search' . SR . $this->addLine($_get->search);

      switch (isset($_get->type) ? $_get->type : false) {
         case 'articles': $target = 'articles'; break;
         case 'videos': $target = 'videos'; break;
         case 'photos': $target = 'photos'; break;
         case 'audio': $target = 'music'; break;
         case 'gif': $target = 'gifs'; break;
         default: $target = isset($target) ? $target : 'all';
      }

      if ($pages > 1) { // Displaying pagination

         $paginate = '<div class="pagination">' . ($current > 0
            ? '<a href="' . $target . ($prev > 0 ? SR . $prev : '') . $add . '">'.$iLeft.'</a>'
            : '<span class="current">'.$iLeft.'</span>');

         if ($pages < 7 + ($stages * 2)) {
            for ($counter = 0; $counter < $pages; $counter++) {
               $paginate .= ($counter == $current)
               ? '<span class="current pagenum">' . ($counter+1) . '</span>'
               : '<a href="' . $target . ($counter > 0 ? SR . $counter : '') . $add .'">' . ($counter+1) . '</a>';
            }
         }

         elseif ($pages > 5 + ($stages * 2)) {

            $iterate = function($i) use(&$current,&$target) {
               return ($i == $current)
                  ? '<span class="current pagenum">' . ($i + 1) . '</span>'
                  : '<a href="' . $target . SR . ($i > 0 ? $i : ''). $add . '">' . ($i + 1) . '</a>';
            };

            $stepDown =
               '<a onclick="return:false">...</a>' .
               '<a href="' . $target . SR . ($pages - 2) . $add . '">' . ($pages - 1) . '</a>' .
               '<a href="' . $target . SR . ($pages - 1) . $add . '">' . $pages . '</a>';

            $stepUp =
               '<a href="' . $target . $add . '">1</a>' .
               '<a href="' . $target . SR . 1 . $add . '">2</a>' .
               '<a onclick="return:false">...</a>';

            if ($current < 1 + ($stages * 2)) {
               for ($counter = 0; $counter < 4 + ($stages * 2); $counter++) {
                  $paginate .= $iterate($counter);
               }; $paginate .= $stepDown;
            }

            elseif ($pages - ($stages * 2) > $current && $current > ($stages * 2)) {
               $paginate .= $stepUp;
               for ($counter = $current - $stages; $counter <= $current + $stages; $counter++) {
                  $paginate .= $iterate($counter);
               }; $paginate .= $stepDown;
            }

            else {

               $paginate .= $stepUp;
               for ($counter = $pages - (2 + ($stages * 2)); $counter < $pages; $counter++) {
                  $paginate .= $iterate($counter);
               }
            }

         }

         $paginate .= (($current < $counter - 1)
            ? '<a href="' . $target . SR . $next . $add . '">' . $iRight . '</a>'
            : '<span class="current">' . $iRight . '</span>') . '</div>';

         return $paginate;

      } else { return '<span></span>'; }

   }

   public function getRewritePattern($id, $cat = false, $type = false) {

      $output = [];  $default_prefix = 'post';
      $pattern = App::get('rewrite_pattern');

      if (empty($pattern)) { return $default_prefix; }
      elseif (
         strpos($pattern, '{type}') !== false && $type === false ||
         strpos($pattern, '{category}') !== false && $cat === false
      ) {
         $data = $this->sql->where('id', $id)->getOne('posts', 'type,category');
         $type = $data["type"]; $cat = $data["category"];
      }

      // Splitting pattern into tags
      $parts = array_filter(explode(' ', $pattern));
      strpos($parts[0], '{') === false && $output[] = array_shift($parts);

      $type == "audio" && $type = "music";

      foreach ($parts as $tag) {
         $tag == '{type}' && $type && $output[] = $this->urlPrepare($type);
         $tag == '{category}' && $cat && $output[] = $this->urlPrepare($cat);
      }

      return implode(SR, $output) ?: $default_prefix;

   }

   public function postLink($id, $title = false, $abs = false, $cat = false, $type = false) {

      $object = in_array(gettype($id), ["object","array"]);
      $data = $object ? (object) $id : (object) [
         'id' => $id, 'tytul' => $title,
         'category' => $cat, 'type' => $type
      ];

      $abs = $object ? $title : $abs;
      $pref = $abs ? App::get('app_url') . '/' : '';
      $pattern = $this->getRewritePattern($data->id, $data->category, $data->type);
      return $pref . $pattern . SR . $data->id . SR . $this->urlPrepare($data->tytul);

   }

   public function getComments($id, $type='post') {
      return $this->sql->countRows('comments','type=? and wpis=? and status=1', [$type, intval($id)]);
   }

   public function getAvatar($user = false) {

      $user = $user ?: User::get('username');
      $default = 'Uploads/Avatars/default-avatar.png';

      if (!$user) { return $default; } else {
         $user = mb_strtolower($user, 'utf-8');
         $path = "Uploads/Avatars/{$user}.jpg";
         return (file_exists(PATH_ROOT .'/'. $path)) ? $path : $default;
      }

   }

   // Media url parser
   public function parseMediaUrl($url, $json = true, $save = true) {

      if (preg_match('/https?:\/\/streamable.com\/([a-z0-9]+)/', $url, $pregSTA) ) {

         $data['id'] = $pregSTA[1];
         $data['type'] = 'video';
         $data['display'] = 'embed';
         $data['source'] = 'streamable';

      } elseif (preg_match('/https?:\/\/openload\.co\/embed\/([a-zA-Z0-9-_]+)/i',$url,$pregOL)) {

         $data['id'] = $pregOL[1];
         $data['type'] = 'video';
         $data['display'] = 'embed';
         $data['source'] = 'openload';

      } elseif (preg_match('/^https?:\/\/(?:www\.)?mixcloud.com\/(.+)/i', $url, $pregMX)) {

         $data['id'] = $pregMX[1];
         $data['type'] = 'audio';
         $data['display'] = 'embed';
         $data['source'] = 'mixcloud';

      } elseif (preg_match('/^https?:\/\/soundcloud\.com\/([\w+-]+)\/([\w+-]+)$/i', $url, $pregSC)) {

         $web = Factory::get('WebData'); $siteData = $web->Request($url);
         preg_match('/content="soundcloud:\/\/sounds:(\d+)"/is',$siteData, $soundId);

         $data['id'] = $soundId[1];
         $data['type'] = 'audio';
         $data['display'] = 'embed';
         $data['source'] = 'soundcloud';

      } elseif (preg_match('/https?:\/\/(?:www.)?metacafe.com\/watch\/(\d+)(?:\/.*\/)?/i', $url, $pregMCF)) {

         $data['id'] = $pregMCF[1];
         $data['type'] = 'video';
         $data['display'] = 'embed';
         $data['source'] = 'metacafe';

      } elseif (preg_match('/https?:\/\/coub\.com\/view\/([a-zA-Z0-9]+)/i', $url, $pregCB)) {

         $data['id'] = $pregCB[1];
         $data['type'] = 'video';
         $data['display'] = 'embed';
         $data['source'] = 'coub';

      } elseif (preg_match('/https?:\/\/(?:www\.)cda\.pl\/video\/(\w+)/i', $url, $pregCDA)) {

         $data['id'] = $pregCDA[1];
         $data['type'] = 'video';
         $data['display'] = 'embed';
         $data['source'] = 'cda';

      } elseif (preg_match('/https?:\/\/(?:www.)?vimeo.com\/(\d+)/i', $url, $pregVIM)) {

         $data['id'] = $pregVIM[1];
         $data['type'] = 'video';
         $data['display'] = 'embed';
         $data['source'] = 'vimeo';

      } elseif(preg_match('/^https?:\/\/d\.tube\/#!\/v\/(\w+)\/(\w+)$/', $url, $pregDTUBE)) {

         $data['id'] = $pregDTUBE[2];
         $data['channel'] = $pregDTUBE[1];
         $data['type'] = 'video';
         $data['display'] = 'embed';
         $data['source'] = 'dtube';

      } elseif (preg_match('/(?:youtube(?:-nocookie)?\.com\/(?:[^\/\n\s]+\/\S+\/|(?:v|e(?:mbed)?)\/|\S*?[?&]v=)|youtu\.be\/)([a-zA-Z0-9_-]{11})/', $url, $pregYT)) {

         $data['id'] = $pregYT[1];
         $data['type'] = 'video';
         $data['display'] = 'embed';
         $data['source'] = 'youtube';

      } elseif (preg_match('/https?:\/\/(?:www\.)liveleak\.com\/view(?:\?\w=)(\w+)(?:.*)/i',$url,$pregLL)) {

         $data['id'] = $pregLL[1];
         $data['type'] = 'video';
         $data['display'] = 'embed';
         $data['source'] = 'liveleak';

      } elseif (preg_match('/https?:\/\/(www\.)?dailymotion\.com\/video\/(\w+)(_)?(.*)?/i',$url,$pregDM)) {

         $data['id'] = $pregDM[2];
         $data['type'] = 'video';
         $data['display'] = 'embed';
         $data['source'] = 'dailymotion';

      } elseif (preg_match('/https?:\/\/twitter.com\/(\w+)\/status(?:es)?\/(\d+)/i',$url,$pregTW)) {

         $data['id'] = $pregTW[2];
         $data['type'] = 'video';
         $data['display'] = 'embed';
         $data['source'] = 'twitter';

      } elseif (preg_match('/(?:(?:http|https):\/\/)?(?:www.)?facebook.com\/[\.\w\/]+\/videos\/(\d+)/i', $url, $pregFB)) {

         $data['id'] = $pregFB[1];
         $data['type'] = 'video';
         $data['display'] = 'embed';
         $data['source'] = 'facebook';

      } elseif (preg_match('/(?:(?:http|https):\/\/)?(?:www.)?giphy.com\/gifs\/(.*)\-(\w+)/i',$url,$pregGY)) {

         $data['id'] = $pregGY[2];
         $data['type'] = 'gif';
         $data['display'] = 'local';
         $data['source'] = 'giphy';

      } elseif (preg_match('/(?:(?:http|https):\/\/)?(?:www.)?9gag.com\/gag\/(\w+)/i',$url,$pregG9)) {

         $data['id'] = $pregG9[1];
         $data['type'] = 'video';
         $data['display'] = 'local';
         $data['source'] = 'ninegag';

      } elseif (preg_match('/https?:\/\/vk\.com\/video_ext\.php\?oid=([0-9-&a-z=]+)/i',$url,$pregVK)) {

         $data['id'] = $pregVK[1];
         $data['type'] = 'video';
         $data['display'] = 'embed';
         $data['source'] = 'vk';

      } elseif (preg_match('/(?:(?:http|https):\/\/)?(?:www.)?instagram.com\/p\/(\w+)\//i',$url,$pregIN)) {

         $data['id'] = $pregIN[1];
         $data['type'] = 'video';
         $data['display'] = 'local';
         $data['source'] = 'instagram';

      } elseif (preg_match('/^https?:\/\/(?:www\.)vevo\.com\/watch\/(?:.*?)\/(?:.*?)\/(\w+)$/i',$url,$pregVEVO)) {

         $data['id'] = $pregVEVO[1];
         $data['type'] = 'video';
         $data['display'] = 'embed';
         $data['source'] = 'vevo';

      } else {

         $web = Factory::get('WebData');
         $headers = $web->Request($url, ['head' => true, 'json' => false]);
         $mime = ['video/webm','video/mp4','audio/mpeg','video/quicktime'];

         if ($headers["status-code"] == 200 && in_array($headers["content-type"], $mime)) {

            $data['file'] = $url;
            $data['binary'] = true;
            $data['name'] = basename($url);
            $data['mime'] = $headers["content-type"];
            $data['bytes'] = $headers["content-length"];
            $data['type'] = $data['mime'][0] == 'v' ? 'video' : 'audio';
            $data['source'] = $this->mime2ext($data['mime']);
            $data['encoder'] = $this->mime2ext($data['mime']);

            if ($save) {

               $this->createTempdir();

               if (file_exists(PATH_TEMP.'/mediadata.json')) {
                  $file = file_get_contents(PATH_TEMP . '/mediadata.json');
                  $json = json_decode($file, true); $data = array_merge($data,$json);
               }

               file_put_contents(PATH_TEMP.'/mediadata.json', json_encode($data));
               chmod_u(PATH_TEMP.'/mediadata.json', 0666);

            }

         } else { $data['error'] = true; }

      }

      return $json ? json_encode($data) : (object) $data;

   }

   public function mime2ext($mime) {
      $source = [
         'video/mp4' => 'mp4',
         'video/webm' => 'webm',
         'audio/mpeg' => 'mp3',
         'video/quicktime' => 'mov',
         'image/png' => 'png',
         'image/jpeg' => 'jpg',
         'image/gif' => 'gif'
      ];
      return $source[ $mime ];
   }

   public function mime2type($mime) {
      return $mime == 'audio/mpeg' ? 'audio' : 'video';
   }

   public function getImageData($file) {
      if (file_exists($file)) {

         $image = array();
         $imgsize = getimagesize($file);

         $image["filename"] = basename($file);
         $image["extension"] = explode('.', $image["filename"])[1];
         $image["width"] = $imgsize[0];
         $image["height"] = $imgsize[1];
         return (object) $image;

      }; return false;
   }

   public function mediaData($url, $type = 'json') {

      if (filter_var($url, FILTER_VALIDATE_URL)) {

         $web = Factory::get('WebData');
         $media = $this->parseMediaUrl($url, false);

         $data['external'] = $url;

         if ($media->source == 'streamable') {

            $output = json_decode($web->Request('https://api.streamable.com/oembed?url=' . $url));

            $dom = new DOMDocument;
            $dom->loadHTML($output->html);
            $thumb = parse_url($output->thumbnail_url);

            if ($thumb['host'] && $thumb['path']) {

               $frm = $dom->getElementsByTagName('iframe')->item(0);

               $data['id'] = $media->id;
               $data['width'] = $output->width;
               $data['height'] = $output->height;
               $data['source'] = 'streamable';
               $data['embed_url'] = $frm->getAttribute('src');
               $data['image'] = 'http://' . $thumb['host'] . $thumb['path'];

            }

         } elseif ($media->source == 'mixcloud') {

            $siteData = file_get_contents($url);
            preg_match('/<meta property="og:image".*?content="(.*?)">/is',$siteData, $imgData);

            $data["id"] = $media->id;
            $data["source"] = $media->source;
            $data['image'] = str_replace('watermark','fw',$imgData[1]);
            $data["embed_url"] = 'https://www.mixcloud.com/widget/iframe/?autoplay=1&light=1&feed=%2F' . rawurlencode($media->id);

         } elseif ($media->source == 'openload') {

            $siteData = $web->Request($url);
            preg_match('/<meta name="og:image" content="(.*?)">/is',$siteData, $imgData);

            $data["id"] = $media->id;
            $data["source"] = $media->source;
            $data['image'] = $imgData[1];
            $data["embed_url"] = 'https://openload.co/embed/' . $media->id;

         } elseif ($media->source == 'dtube') {

            $data['id'] = $media->id;
            $data['source'] = 'dtube';
            $data['embed_url'] = 'https://d.tube/#!/v/'.$media->channel.'/'.$media->id;

         } elseif ($media->source == 'coub') {

            $siteData = $web->Request($url);
            preg_match('/<meta property="og:image" name="og:image" content="(.*?)".*>/is',$siteData, $imgData);

            $data['id'] = $media->id;
            $data['source'] = 'coub';
            $data['image'] = $imgData[1];
            $data['embed_url'] = '//coub.com/embed/'.$media->id . '?autostart=true';

         } elseif ($media->source == 'youtube') {

            $link = "http://i.ytimg.com/vi/{$media->id}/maxresdefault.jpg";
            $code = $web->Request($link, ['head'=>true]);

            if ($code["status-code"] == 200) { $img = $link; }
            else { $img = "http://i.ytimg.com/vi/{$media->id}/hqdefault.jpg"; }

            $data['id'] = $media->id;
            $data['source'] = 'youtube';
            $data['image'] = $img;

         } elseif ($media->source == 'liveleak') {

            $siteData = $web->Request($url);
            preg_match('/<meta property="og:image" content="(.*?)"\/>/is',$siteData, $imgData);
            preg_match('/iframe width="(.*?)" height="(.*?)"/is',$siteData, $dim);

            $data['id'] = $media->id;
            $data['source'] = 'liveleak';
            $data['width'] = $dim[1];
            $data['height'] = $dim[2];
            $data['image'] = $imgData[1];
            $data['embed_url'] = '//www.liveleak.com/e/'.$media->id;

         } elseif ($media->source == 'metacafe') {

            $siteData = $web->Request($url);
            preg_match('/<meta property=(?:"|\')og:image(?:"|\') content="(.*?)">/is',$siteData, $imgData);
            preg_match('/<meta property="og:video:width" content="(.*?)">/is',$siteData,$width);
            preg_match('/<meta property="og:video:height" content="(.*?)">/is',$siteData,$height);

            $data['id'] = $media->id;
            $data['source'] = 'metacafe';
            $data['width'] = $width[1];
            $data['height'] = $height[1];
            $data['image'] = $imgData[1];
            $data['embed_url'] = '//www.metacafe.com/embed/' . $media->id . '/';

         } elseif ($media->source == 'vimeo') {

            $siteData = $web->Request($url);
            $api = $web->Request('https://vimeo.com/api/v2/video/'.$media->id.'.json');
            $api_parsed = array_shift(json_decode($api));
            preg_match('/<meta property="og:video:width" content="(.*?)">/is',$siteData,$width);
            preg_match('/<meta property="og:video:height" content="(.*?)">/is',$siteData,$height);

            $data['id'] = $media->id;
            $data['source'] = 'vimeo';
            $data['width'] = $width[1];
            $data['height'] = $height[1];
            $data['image'] = $api_parsed->thumbnail_large;
            $data['embed_url'] = '//player.vimeo.com/video/' . $media->id;

         } elseif ($media->source == 'soundcloud') {

           $siteData = $web->Request($url);
           preg_match('/content="soundcloud:\/\/sounds:(\d+)"/is',$siteData, $soundId);
           preg_match('/<meta property="og:title" content="(.*?)"(\s+)?\/?>/is',$siteData, $title);
           preg_match('/<meta property="og:image" content="(.*?)"(\s+)?\/?>/is',$siteData, $imgData);

            $data['id'] = $media->id;
            $data['source'] = 'soundcloud';
            $data['image'] = $imgData[1];
            $data['more'] = json_encode(['title' => $title[1]]);

         } elseif ($media->source == 'vevo') {

            $siteData = $web->Request($url);
            preg_match('/<meta property="og:image" content="(.*?)"(\s+)?\/?>/is',$siteData,$imgData);
            preg_match('/<meta property="og:video:width" content="(.*?)"(\s+)?\/?>/is',$siteData,$width);
            preg_match('/<meta property="og:video:height" content="(.*?)"(\s+)?\/?>/is',$siteData,$height);

            $data['id'] = $media->id;
            $data['source'] = 'vevo';
            $data['width'] = $width[1];
            $data['height'] = $height[1];
            $data['image'] = $imgData[1];
            $data['embed_url'] = '//embed.vevo.com?isrc='.$media->id;

         } elseif ($media->source == 'dailymotion') {

            $siteData = $web->Request($url);
            preg_match('/<meta property="og:image" content="(.*?)"(\s+)\/>/is',$siteData,$imgData);
            preg_match('/<meta property="og:video:width" content="(.*?)"(\s+)\/>/is',$siteData,$width);
            preg_match('/<meta property="og:video:height" content="(.*?)"(\s+)\/>/is',$siteData,$height);

            $data['id'] = $media->id;
            $data['source'] = 'dailymotion';
            $data['width'] = $width[1];
            $data['height'] = $height[1];
            $data['image'] = $imgData[1];
            $data['embed_url'] = '//www.dailymotion.com/embed/video/' . $media->id;

         } elseif ($media->source == 'cda') {

            $siteData = $web->Request($url);
            preg_match('/<meta property="og:image" content="(.*?)"(\s)?\/>/is',$siteData,$imgData);
            preg_match('/<meta property="og:image:width" content="(.*?)"(\s)?\/>/is',$siteData,$width);
            preg_match('/<meta property="og:image:height" content="(.*?)"(\s)?\/>/is',$siteData,$height);

            $embedData = $web->Request('http://ebd.cda.pl/700x413/' . $media->id);
            preg_match('/player_data=\'(.*?)\'/is',$embedData,$JS);
            $JSONdata = json_decode($JS[1]);

            if ( substr($imgData[1], 0, 2) == '//' ) { $imgData[1] = 'https:' . $imgData[1]; }

            $data['id'] = $media->id;
            $data['source'] = 'cda';
            $data['width'] = $width[1];
            $data['height'] = $height[1];
            $data['image'] = $imgData[1];
            $data['file'] = $JSONdata->video->file;
            $data['embed_url'] = 'http://ebd.cda.pl/700x413/' . $media->id;

         } elseif ($media->source == 'twitter') {

            $siteData = $web->Request($url);
            preg_match('/<meta  property="og:video:url" content="(.*?)">/is',$siteData,$mp4Data);
            preg_match('/<meta  property="og:image" content="(.*?)">/is',$siteData,$imgData);
            preg_match('/<meta  property="og:video:width" content="(.*?)">/is',$siteData,$width);
            preg_match('/<meta  property="og:video:height" content="(.*?)">/is',$siteData,$height);

            $data['id'] = $media->id;
            $data['source'] = 'twitter';
            $data['width'] = $width[1];
            $data['height'] = $height[1];
            $data['image'] = $imgData[1];
            $data['embed_url'] = 'https://twitter.com/i/videos/'.($data['id']).'?embed_source=facebook';

         } elseif ($media->source == 'facebook') {

            $json  = $web->Request("https://graph.facebook.com/{$media->id}");
            $embed = $web->Request("https://www.facebook.com/plugins/video.php?href={$url}");

            preg_match('[data-video-width="(.*?)"]is',$embed,$FB_width);
            preg_match('[data-video-height="(.*?)"]is',$embed,$FB_height);
            preg_match('[<img(.*?)src="(.*?)"(.*?)>]is',$embed,$FB_img);

            $data['id'] = $media->id;
            $data['source'] = 'facebook';
            $data['width'] = $FB_width[1];
            $data['height'] = $FB_height[1];
            $data['image'] = $FB_img[2];

         } elseif ($media->source == 'giphy') {

            $link = "http://giphy.com/gifs/{$media->id}/html5";
            $html = $web->SimpleDom($link);

            foreach($html->find('video') as $e) { $width = $e->width; $height = $e->height; }
            foreach($html->find('source') as $e) { $src = $e->src; }
            foreach($html->find('meta[property="og:image"]') as $e) { $imgData = $e->content; }

            $data['id'] = $media->id;
            $data['source'] = 'giphy';
            $data['width'] = $width;
            $data['height'] = $height;
            $data['image'] = $imgData;
            $data['file'] = str_replace('media.g','i.g',$src);

         } elseif ($media->source == 'ninegag') {

            $siteData = $web->Request($url);
            $im = preg_match('/<meta property="og:image" content="(.*?)" \/>/is',$siteData,$imgData);
            $m4 = preg_match('/<source src="(.*?)" type="video\/mp4">/is',$siteData,$mp4Data);

            if (!$m4) { $mp4 = "http://img-9gag-fun.9cache.com/photo/{$media->id}_460sv.mp4"; }
            else { $mp4 = $mp4Data[1]; }

            $data['id'] = $media->id;
            $data['source'] = 'ninegag';
            $data['image'] = $imgData[1];
            $data['file'] = $mp4;

         } elseif ($media->source == 'vk') {

            $siteData = $web->Request($url);
            preg_match('/<div\sclass="video_box_msg_background"[^(]+\(([^\)]+)\);"/', $siteData, $im);

            $data['id'] = $media->id;
            $data['source'] = 'vk';
            $data['image'] = $im[1];
            $data['embed_url'] = $url;

         } elseif ($media->source == 'instagram') {

            $siteData = $web->Request($url);
            preg_match('/<meta property="og:video" content="(.*?)" \/>/is',$siteData,$mp4Data);
            preg_match('/<meta property="og:image" content="(.*?)" \/>/is',$siteData,$imgData);
            preg_match('/<meta property="og:video:width" content="(.*?)" \/>/is',$siteData,$width);
            preg_match('/<meta property="og:video:height" content="(.*?)" \/>/is',$siteData,$height);

            $data['id'] = $media->id;
            $data['source'] = 'instagram';
            $data['width'] = $width[1];
            $data['height'] = $height[1];
            $data['file'] = $mp4Data[1];
            $data['image'] = $imgData[1];

         } else { return false; }

         if ( !isset($data['error']) ) {

            isset($data['image']) && $data['image'] = htmlspecialchars_decode($data['image']);
            isset($data['file']) && $data['file'] = htmlspecialchars_decode($data['file']);
            return ($type=='json') ? json_encode($data) : $data;

         } else { return false; }

      } // Valid url

   }

   // Funkcja pobierające stronę jak przeglądarka

   public function unixTimeArray($time) {
      $time = (int) $time;
      $base['unix'] = $time;
      $base['day'] = date("d", $time);
      $base['month'] = $this->unixTimeMonth($time);
      return $base;
   }

   public function unixTimeMonth($time) {

      $time = (int) $time;
      $base = date("n",$time);
      $rok = ' '.date("Y",$time);

      if ($base==1) return Lang::get('jan') . $rok;
      if ($base==2) return Lang::get('feb') . $rok;
      if ($base==3) return Lang::get('mar') . $rok;
      if ($base==4) return Lang::get('apr') . $rok;
      if ($base==5) return Lang::get('may') . $rok;
      if ($base==6) return Lang::get('june') . $rok;
      if ($base==7) return Lang::get('july') . $rok;
      if ($base==8) return Lang::get('aug') . $rok;
      if ($base==9) return Lang::get('sep') . $rok;
      if ($base==10) return Lang::get('oct') . $rok;
      if ($base==11) return Lang::get('nov') . $rok;
      if ($base==12) return Lang::get('dec') . $rok;

   }

   public function zmienczas($unix , $user = false) {

      $czas = (int) ((time()-$unix)/60); // W minutach

      if ($czas <= 1) { $lastlogin = $user ? 'ONLINE' : Lang::get('one_min_ago'); }
      elseif ($czas < 5 and $czas > 1) { $lastlogin = $czas.' '.Lang::get('minutes_ago'); }
      elseif ($czas > 4 and $czas < 60) { $lastlogin = $czas.' '.Lang::get('minutes_ago'); }
      elseif ($czas >= 60 and $czas < 120) { $lastlogin = Lang::get('hour_ago'); }
      elseif ($czas >= 120 and $czas < 300) { $lastlogin = intval($czas/60).' '.Lang::get('hours_ago'); }
      elseif ($czas >= 300 and $czas < 1320) { $lastlogin = intval($czas/60).' '.Lang::get('hours_ago'); }
      elseif ($czas >= 1320 and $czas < 1440) { $lastlogin = intval($czas/60).' '.Lang::get('hours_ago'); }
      elseif ($czas >= 1440 and $czas < 2880) { $lastlogin = Lang::get('yesterday'); }
      elseif ($czas >= 2880 and $czas < 4320) { $lastlogin = Lang::get('two_days_ago'); }
      elseif ($czas >= 4320 and $czas < 43200) { $lastlogin = intval($czas/1440).' '.Lang::get('days_ago'); }
      elseif ($czas >= 43200) {
         $utm = $this->unixTimeMonth(time()-($czas*60));
         $lastlogin = date("d",time()-($czas*60)).' '.$utm;
      }

      return $lastlogin;

   }

}