<?php

Class WebData {
   
   private $url;        # (string)  Url to resource
   private $handlers;   # (array)   Available functions
   private $params;     # (mixed)   Query parameters
   private $headers;    # (array)   Response headers
   private $response;   # (string)  Fetched data
   private $connection; # (string)  Request status
   
   # Source: barebonescms.com/documentation/ultimate_web_scraper_toolkit

   private $webtools;
   private $simpledom;

   public function __construct( $url = NULL, $params = NULL) {
      
      $this->connection = array();
      $this->webtools = CLASSES . "/WebTools/web_browser.php";
      $this->simpledom = CLASSES . '/WebTools/simple_html_dom.php';

      $this->handlers['webtools'] = $this->setWebTools() ? true : false;
      $this->handlers['simpledom'] = $this->setSimpleDom() ? true : false;
      $this->handlers['fileget'] = ini_get('allow_url_fopen') ? true : false;
      $this->handlers['curl'] = function_exists('curl_version') ? true : false;
      
      if ($url) $this->Request($url, $params);
   
   }
   
   public function Request($url, $params = NULL, $chain = false) {

      try {

         $this->resetClass();
         $this->validateUrl($url);
         $this->setParams($params);
         
         isset($params['head']) ? $this->getHeaders() : $this->getData();
         
         if ($chain) { return $this; }
         else {
            if ( isset($this->params['head']) ) {
               return empty($this->params['json']) ? $this->headers : json_encode($this->headers);
            } return $this->response;
         }

      } catch (Exception $e) { 
         // $this->resetClass();
         $this->connection['error'][] = $e->getMessage();
         return false;
      }
   }
   
   public function SimpleDom($url) {
      if ($this->handlers['simpledom']) {
          if ($url) { return file_get_html($url); }
          elseif ($this->response) { 
             $html = new simple_html_dom(); 
             return $html->load($this->response); 
          } return false;
      } return false;
   }
   
   public function Html($callback) {
      if ($this->response) {
         if ($callback && function_exists($callback)) {
            call_user_func($callback, $this->response);
            return true;
         } else { 
            return $this->response;
         }
      } return false;
   }
   public function Head($asJson) {
      if ($this->headers) {
         return $asJson ? json_encode($this->headers) : $this->headers;
      } return false;
   }
   public function SaveToFile($path, $mode = 0666) {
   	if ($this->response) {
         $status = file_put_contents($path, $this->response);
         chmod_u($path, $mode); return $status;
      } else { $this->error = 'No response to save'; }
      return false;
   }
   public function getErrors() {
      if ($this->connection['error']) {
         return $this->connection['error'];
      } return false;
   }
   
   private function getData() {
      $fn = $this->handlers;
      $this->connection['method'] = 'GET';
      if ( ($fn['webtools'] && $this->webtools()) || ($fn['curl'] && $this->curl()) || ($fn['fileget'] && $this->fileget()) ) {
         $this->SetStatus(1); return true; 
      } else { 
         $this->SetStatus(2); 
         throw new Exception('GetData: Class was unable to fetch data from ' . $this->url); 
      }
   }
   private function getHeaders() {
      if ($this->curlHeaders() || $this->phpHeaders() || $this->fopenHeaders()){ 
         $this->connection['method'] = 'HEAD';
         $this->response = $this->headers;
         $this->SetStatus(1); return true; 
      } else { 
         $this->SetStatus(2); 
         throw new Exception('GetHeaders: Class was unable to fetch headers from ' . $this->url); 
      }
   }

   # Default request methods #
   private function curl() {

      $this->connection['handler'] = 'curl';
                                
      if ($this->params['postvars']) {
         $this->preparePostvars();
         $postvars = http_build_query($this->params['postvars']);
      }
                                
      $ch = curl_init(); 
      curl_setopt($ch, CURLOPT_HEADER, true);
      curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
      curl_setopt($ch, CURLOPT_URL, $this->url);
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
      if ($postvars) {
         curl_setopt($ch, CURLOPT_POST, 1);
         curl_setopt($ch, CURLOPT_POSTFIELDS, $postvars);
      }
      $response = curl_exec($ch);
      
      if (!curl_error($ch)) {
         
         $header_size = curl_getinfo($ch, CURLINFO_HEADER_SIZE); 
         $result = substr($response, $header_size);
         $temp['headers'] = trim(substr($response, 0, $header_size));
         
         $this->parseHeaders($temp);
         $this->response = $result;

         return true;
                  
      } else {
         $this->connection['error'][] = 'Curl: ' . curl_error($ch);
         return false;
      }
      
   }
   private function fileget() {
      
      $this->connection['handler'] = 'fileget';
      
      if ($this->params['postvars']) {
         
         $this->preparePostvars();
         $postvars = http_build_query($this->params['postvars']);
         
         $opts = array('http' => [
           'method'  => 'POST',
           'header'  => 'Content-type: application/x-www-form-urlencoded',
           'content' => $postvars
         ]);

         $context  = stream_context_create($opts);
         
      }
      
      if ($result = file_get_contents($this->url, false, $context)) {
         
         $temp['headers'] = $http_response_header;
         $this->parseHeaders($temp);
         $this->response = $result;
         
         return true;
         
      } else {
         $this->connection['error'][] = 'FileGet: Unable to retrieve content';
         return false;
      }
      
   }
   private function webtools() {

      $this->connection['handler'] = 'webtools';
      
      if ( isset($this->params['postvars']) ) {
         $this->preparePostvars();
         $options = [ "postvars" => $this->params['postvars'] ];
      }
      
      isset($options) ?: $options = [];
      
      $Browser = new WebBrowser();
      $result = $Browser->Process($this->url,'auto', $options);
      
      if ( isset($result['error']) ) {
         $options = array_merge(['protocol' => 'ssl'], $options);
         $result = $Browser->Process($this->url,'auto', $options);
      }
      
      if ($result['success']) {
         
         $temp['headers']  = $result['headers'];
         $temp['response'] = $result['response'];
         
         $this->parseHeaders($temp);
         $this->response = $result['body'];
         
         return true;
         
      } else {
         $this->connection['error'][] = 'WebBrowser: ' . $result['error'];
         return false;
      }
      
   }
   
   # Headers request methods #
   private function phpHeaders() {

      if ($_head = get_headers($this->url, true)) {
         $parts = explode(' ',$_head[0]);
         $head['status-text']  = $parts[2];
         $head['status-code']  = (int) $parts[1];
         $head['content-type'] = explode(';', $_head['Content-Type'])[0];
         if ($_head['Content-Length']) {
            $head['content-length'] = (int) $_head['Content-Length'];
         } $this->headers = $head; return true;
      } else {
         $this->connection['error'][] = 'phpHeaders: Unable to fetch headers using get_headers().';
         return false;
      }

   }
   private function curlHeaders() {

      $ch = curl_init(); 
      curl_setopt($ch, CURLOPT_HEADER, true);
      curl_setopt($ch, CURLOPT_NOBODY, true);
      curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
      curl_setopt($ch, CURLOPT_URL, $this->url);
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
      $rawHeaders = curl_exec($ch);
      
      if (!curl_error($ch)) {
         $temp['headers'] = $rawHeaders;
         $this->parseHeaders($temp); return true;
      } else {
         $this->connection['error'][] = 'curlHeaders: ' . curl_error($ch);
         return false;
      }

   }
   private function fopenHeaders() {
      $context  = stream_context_create(array('http' => ['method'=>'HEAD']));
      $fd = fopen($this->url, 'rb', false, $context);
      if ($fd) {
         $rawHeaders = stream_get_meta_data($fd)['wrapper_data']; fclose($fd);
         $temp['headers'] = $rawHeaders; $this->parseHeaders($temp); return true;
      } else {
         $this->connection['error'][] = 'fopenHeaders: Unable to fetch headers using stream.';
         return false;
      }
      
   }
   
   private function validateUrl($url) {
      if (!empty($url) && filter_var($url, FILTER_VALIDATE_URL)) {
         $this->url = filter_var($url, FILTER_SANITIZE_URL);
      } else {
         throw new Exception('Provided url is not valid.');
      }
   }
   private function SetStatus($status) {
      switch ($status) {
         case 1: $this->connection['status'] = 'success'; break;
         case 2: $this->connection['status'] = 'failed'; break;
      }
   }
   private function parseHeaders($temp) {
      
      $handler = isset($this->connection['handler']) 
         ? $this->connection['handler'] : false ;
      
      if ($handler == 'webtools') {

         $head['status-code']  = (int) $temp['response']['code'];
         $head['status-text']  = $temp['response']['meaning'];
         $head['content-type'] = explode(';', $temp['headers']['Content-Type'][0])[0];
         
         if (isset($temp['headers']['Content-Length'])) {
            
            $clen = $temp['headers']['Content-Length'];
            $head['content-length'] = intval($clen[0]);
            
         }

      } else {

         $data = (is_array($temp['headers'])) ? $temp['headers'] : explode("\r\n",$temp['headers']);
         
         foreach ($data as $k => $v) {
            $t = explode( ':', $v, 2 );
            if (isset($t[1])) $_head[trim($t[0])] = trim($t[1]);
            else { $_head[] = $v;
              if (preg_match("#HTTP/[0-9\.]+\s+([0-9]+)#", $v, $out))
                $_head['reponse_code'] = intval($out[1]);
            }
         }
         
         $response = explode(' ', $_head[0]);

         $head['status-code']  = (int) $response[1];
         $head['status-text']  = $response[2];
         $head['content-type'] = explode(';', $_head['Content-Type'])[0];
         
         if (isset($_head['Content-Length'])) {
            $head['content-length'] = (int) $_head['Content-Length'];
         }
         
      }
      
      $this->headers = $head;

      return true;
   
   }
   private function setParams($params) {
      if ($params) {
         if (is_array($params)) $this->params = $params;
         else throw new Exception('Connection parameters should be an Array.');
      }
   }
   private function preparePostvars() {
      $vars = $this->params['postvars'];
      if (isset($vars) && is_array($vars)) {
         $this->connection['method'] = 'POST';
         $this->params['postvars'] = filter_var_array($vars, FILTER_SANITIZE_STRING);
      }
   }
   private function setWebTools() {
      if (isset($this->webtools) && !empty($this->webtools)) {
         try { 
            if (file_exists($this->webtools)) {
               require_once $this->webtools;
               return class_exists('WebBrowser') ? true : false;
            } else { throw new Exception('Please provide valid path to WebBrowser class.'); }
         } catch (Exception $e) { $this->error = $e->getMessage(); }
      }
   }
   private function setSimpleDom() {
      if (isset($this->simpledom) && !empty($this->simpledom)) {
         try { 
            if (file_exists($this->simpledom)) {
               require_once $this->simpledom;
               return function_exists('file_get_html') ? true : false;
            } else { throw new Exception('Please provide valid path to Simple Html Dom class.'); }
         } catch (Exception $e) { $this->error = $e->getMessage(); }
      }
   }
   private function resetClass() {
      $vars = [ 'url', 'error', 'headers', 'params', 'response', 'handler', 'connection' ];
      foreach (get_object_vars($this) as $name => $value) { 
         if (in_array($name, $vars)) $this->$name = '';
      } 
   }
   
} // END Class