<?php

use samdark\sitemap\Sitemap;
use samdark\sitemap\Index;

class XMLSitemap extends AppController {

   const PATH = '/sitemap.xml';
   const DELAY = 86400; // Daily

   private $appurl;
   private $sitemap;

   public function __construct() {
      parent::__construct();
      $this->appurl = App::get('app_url') .'/';
      $this->sitemap = new Sitemap(PATH_ROOT . self::PATH);
      $this->shedule();
   }

   private function shedule() {
      $file = PATH_ROOT . self::PATH;
      if (!file_exists($file) || time() - @filemtime($file) > self::DELAY) {
         $this->create();
      }
   }

   public function create() {

      // Fetching all published posts data
      $prefix = $this->appurl . 'Uploads/Media/';

      // Homepage
      $this->sitemap->addItem($this->appurl, time(), 'daily', 1);

      $sql = $this->sql->where('status', 2)->orderBy('id','desc')->get('posts');
      if ($this->sql->count > 0) { foreach ($sql as $row) {
         
         $mediadir = $prefix . $row["dir"];
         $link = App::postLink($row['id'], $row['tytul']);

         // -------------------------------------
         // === Creating entry for video post ===
         // -------------------------------------

         if ($row['type'] == "video" && $row["mode"] == "local") {

            $j = json_decode($row["mediadata"]); $format = [
               'type'  => 'video',
               'title' => htmlentities($row["tytul"]),
               'image' => $mediadir .'/'. $row["hash"] . '.jpg',
               'thumb' => $mediadir .'/m_'. $row["hash"] . '.jpg',
               'video' => $mediadir .'/'. $row["hash"] . '.' . $j->encoder
            ];
         
         }

         // ---------------------------------
         // === Creating entry for photos ===
         // ---------------------------------
         
         elseif ($row['type'] == "photos") {

            if ($row["mode"] == "single") {

               $data = $this->sql->where('pid', $row["id"])->getOne('photos', 'file,title,opis');
               if ($this->sql->count > 0) { $format = [
                  'type'  => 'image',
                  'link'  => $mediadir .'/'. $data["file"],
                  'desc'  => htmlentities($data["opis"]),
                  'title' => htmlentities(isset($data["title"]) ? $data["title"] : $row["tytul"])
               ];}

            }

            // Gallery post
            elseif ($row["mode"] == "gallery") {

               $n = []; $format = ['type' => 'image', 'gallery' => []];
               $data = $this->sql->where('pid', $row["id"])->get('photos', null, ['file','title','opis']);
               
               if ($this->sql->count > 0) { foreach ($data as $image) {
                  $format["gallery"][] = [
                     'title' => htmlentities($image["title"]),
                     'desc'  => htmlentities($image["opis"]),
                     'link'  => $mediadir . '/' . $image["file"]
                  ];
               }}

               if (is_array($n)) { array_push($format, $n); }
               
            }

         } else {

            // Article, gif or audio post
            $format = [
               'title' => htmlentities($row["tytul"]),
               'image' => $mediadir .'/'. $row["hash"] . '.jpg'
            ];

         }

         $this->sitemap->addItem($this->appurl . $link, time(), 'daily', 0.8, $format);

      }} // END FOREACH

      // Generating links for categories
      $sql = $this->sql->get('categories');
      if ($this->sql->count > 0) { foreach ($sql as $row) {
         $link = 'category' . SR . App::addLine($row['name']);
         $this->sitemap->addItem($this->appurl . $link, time(), 'daily', 0.8);
      }}

      // Generating links for tags
      $sql = $this->sql->get('tags');
      if ($this->sql->count > 0) { foreach ($sql as $row) {
         $link = 'tag' . SR . App::addLine($row['keyword']);
         $this->sitemap->addItem($this->appurl . $link, time(), 'daily', 0.8);
      }}

      // Generating links for profiles
      $sql = $this->sql->get('login');
      if ($this->sql->count > 0) { foreach ($sql as $row) {
         $link = 'profile' . SR . App::addLine($row['username']);
         $this->sitemap->addItem($this->appurl . $link, time(), 'weekly', 0.6);
      }}
      
      $this->sitemap->write();
      @chmod(PATH_ROOT . self::PATH, 0666);
      
   }

}